@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.122 2000/08/22 06:04:40 guy Exp $
+ * $Id: capture.c,v 1.132 2000/11/15 05:41:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -69,10 +69,6 @@
 #include <sys/ioctl.h>
 #endif
 
-#ifdef HAVE_NET_IF_H
-#include <net/if.h>
-#endif
-
 #include <signal.h>
 #include <errno.h>
 
@@ -99,7 +95,7 @@
 #include "globals.h"
 
 #include "wiretap/libpcap.h"
-#include "wiretap/wtap-int.h"
+#include "wiretap/wtap.h"
 
 #include "packet-clip.h"
 #include "packet-eth.h"
@@ -108,7 +104,9 @@
 #include "packet-ppp.h"
 #include "packet-raw.h"
 #include "packet-tr.h"
+#include "packet-ieee80211.h"
 
+int promisc_mode = TRUE; /* capture in promiscuous mode */
 int sync_mode;	/* fork a child to do the capture, and sync between them */
 static int sync_pipe[2]; /* used to sync father */
 enum PIPES { READ, WRITE }; /* Constants 0 and 1 for READ and WRITE */
@@ -129,6 +127,7 @@ static void capture_pcap_cb(u_char *, const struct pcap_pkthdr *,
   const u_char *);
 static void send_errmsg_to_parent(const char *);
 static float pct(gint, gint);
+static void stop_capture(int signo);
 
 typedef struct _loop_data {
   gint           go;
@@ -158,6 +157,45 @@ static int pipe_dispatch(int, loop_data *, struct pcap_hdr *);
 int child_process;
 #endif
 
+/* Add a string pointer to a NULL-terminated array of string pointers. */
+static char **
+add_arg(char **args, int *argc, char *arg)
+{
+  /* Grow the array; "*argc" currently contains the number of string
+     pointers, *not* counting the NULL pointer at the end, so we have
+     to add 2 in order to get the new size of the array, including the
+     new pointer and the terminating NULL pointer. */
+  args = g_realloc(args, (*argc + 2) * sizeof (char *));
+
+  /* Stuff the pointer into the penultimate element of the array, which
+     is the one at the index specified by "*argc". */
+  args[*argc] = arg;
+
+  /* Now bump the count. */
+  (*argc)++;
+
+  /* We overwrite the NULL pointer; put it back right after the
+     element we added. */
+  args[*argc] = NULL;
+
+  return args;
+}
+
+#ifdef _WIN32
+/* Given a string, return a pointer to a quote-encapsulated version of
+   the string, so we can pass it as an argument with "spawnvp" even
+   if it contains blanks. */
+char *
+quote_encapsulate(const char *string)
+{
+  char *encapsulated_string;
+
+  encapsulated_string = g_new(char, strlen(string) + 3);
+  sprintf(encapsulated_string, "\"%s\"", string);
+  return encapsulated_string;
+}
+#endif
+
 /* Open a specified file, or create a temporary file, and start a capture
    to the file in question. */
 void
@@ -203,14 +241,43 @@ do_capture(char *capfile_name)
     char save_file_fd[24];
     char errmsg[1024+1];
     int error;
+    int argc;
+    char **argv;
 #ifdef _WIN32
     char sync_pipe_fd[24];
+    char *fontstring;
     char *filterstring;
 #endif
 
-    sprintf(ssnap,"%d",cfile.snap); /* in lieu of itoa */
+    /* Allocate the string pointer array with enough space for the
+       terminating NULL pointer. */
+    argc = 0;
+    argv = g_malloc(sizeof (char *));
+    *argv = NULL;
+
+    /* Now add those arguments used on all platforms. */
+    argv = add_arg(argv, &argc, CHILD_NAME);
+
+    argv = add_arg(argv, &argc, "-i");
+    argv = add_arg(argv, &argc, cfile.iface);
+
+    argv = add_arg(argv, &argc, "-w");
+    argv = add_arg(argv, &argc, cfile.save_file);
+
+    argv = add_arg(argv, &argc, "-W");
+    sprintf(save_file_fd,"%d",cfile.save_file_fd);	/* in lieu of itoa */
+    argv = add_arg(argv, &argc, save_file_fd);
+
+    argv = add_arg(argv, &argc, "-c");
     sprintf(scount,"%d",cfile.count);
-    sprintf(save_file_fd,"%d",cfile.save_file_fd);
+    argv = add_arg(argv, &argc, scount);
+
+    argv = add_arg(argv, &argc, "-s");
+    sprintf(ssnap,"%d",cfile.snap);
+    argv = add_arg(argv, &argc, ssnap);
+
+    if (!promisc_mode)
+      argv = add_arg(argv, &argc, "-p");
 
 #ifdef _WIN32
     /* Create a pipe for the child process */
@@ -226,20 +293,26 @@ do_capture(char *capfile_name)
       return;
     }
 
+    /* Convert font name to a quote-encapsulated string and pass to child */
+    argv = add_arg(argv, &argc, "-m");
+    fontstring = quote_encapsulate(prefs.gui_font_name);
+    argv = add_arg(argv, &argc, fontstring);
+
     /* Convert pipe write handle to a string and pass to child */
+    argv = add_arg(argv, &argc, "-Z");
     itoa(sync_pipe[WRITE], sync_pipe_fd, 10);
-    /* Convert filter string to a quote delimited string */
-    filterstring = g_new(char, strlen(cfile.cfilter) + 3);
-    sprintf(filterstring, "\"%s\"", cfile.cfilter);
-    filterstring[strlen(cfile.cfilter) + 2] = 0;
+    argv = add_arg(argv, &argc, sync_pipe_fd);
+
+    /* Convert filter string to a quote delimited string and pass to child */
+    if (cfile.cfilter != NULL && strlen(cfile.cfilter) != 0) {
+      argv = add_arg(argv, &argc, "-f");
+      filterstring = quote_encapsulate(cfile.cfilter);
+      argv = add_arg(argv, &argc, filterstring);
+    }
+
     /* Spawn process */
-    fork_child = spawnlp(_P_NOWAIT, ethereal_path, CHILD_NAME, "-i", cfile.iface,
-                         "-w", cfile.save_file, "-W", save_file_fd,
-                         "-c", scount, "-s", ssnap,
-                         "-Z", sync_pipe_fd,
-                         strlen(cfile.cfilter) == 0 ? (const char *)NULL : "-f",
-                         strlen(cfile.cfilter) == 0 ? (const char *)NULL : filterstring,
-                         (const char *)NULL);
+    fork_child = spawnvp(_P_NOWAIT, ethereal_path, argv);
+    g_free(fontstring);
     g_free(filterstring);
     /* Keep a copy for later evaluation by _cwait() */
     child_process = fork_child;
@@ -255,6 +328,15 @@ do_capture(char *capfile_name)
 			strerror(error));
       return;
     }
+
+    argv = add_arg(argv, &argc, "-m");
+    argv = add_arg(argv, &argc, prefs.gui_font_name);
+
+    if (cfile.cfilter != NULL && strlen(cfile.cfilter) != 0) {
+      argv = add_arg(argv, &argc, "-f");
+      argv = add_arg(argv, &argc, cfile.cfilter);
+    }
+
     if ((fork_child = fork()) == 0) {
       /*
        * Child process - run Ethereal with the right arguments to make
@@ -272,13 +354,7 @@ do_capture(char *capfile_name)
       close(1);
       dup(sync_pipe[WRITE]);
       close(sync_pipe[READ]);
-      execlp(ethereal_path, CHILD_NAME, "-i", cfile.iface,
-		"-w", cfile.save_file, "-W", save_file_fd,
-		"-c", scount, "-s", ssnap, 
-		"-m", prefs.gui_font_name,
-		(cfile.cfilter == NULL)? 0 : "-f",
-		(cfile.cfilter == NULL)? 0 : cfile.cfilter,
-		(const char *)NULL);	
+      execvp(ethereal_path, argv);
       snprintf(errmsg, sizeof errmsg, "Couldn't run %s in child process: %s",
 		ethereal_path, strerror(errno));
       send_errmsg_to_parent(errmsg);
@@ -293,6 +369,7 @@ do_capture(char *capfile_name)
 
     /* Parent process - read messages from the child process over the
        sync pipe. */
+    g_free(argv);	/* free up arg array */
 
     /* Close the write side of the pipe, so that only the child has it
        open, and thus it completely closes, and thus returns to us
@@ -913,7 +990,7 @@ pipe_dispatch(int fd, loop_data *ld, struct pcap_hdr *hdr)
     case WTAP_ENCAP_FDDI_BITSWAPPED:
       capture_fddi(pd, &ld->counts);
       break;
-    case WTAP_ENCAP_TR:
+    case WTAP_ENCAP_TOKEN_RING:
       capture_tr(pd, 0, &ld->counts);
       break;
     case WTAP_ENCAP_NULL:
@@ -928,6 +1005,9 @@ pipe_dispatch(int fd, loop_data *ld, struct pcap_hdr *hdr)
     case WTAP_ENCAP_LINUX_ATM_CLIP:
       capture_clip(pd, &ld->counts);
       break;
+    case WTAP_ENCAP_IEEE_802_11:
+      capture_ieee80211(pd,0,&ld->counts);
+      break;
     /* XXX - FreeBSD may append 4-byte ATM pseudo-header to DLT_ATM_RFC1483,
        with LLC header following; we should implement it at some
        point. */
@@ -937,21 +1017,25 @@ pipe_dispatch(int fd, loop_data *ld, struct pcap_hdr *hdr)
 }
 #endif
 
+/*
+ * This needs to be static, so that the SIGUSR1 handler can clear the "go"
+ * flag.
+ */
+static loop_data   ld;
+
 /* Do the low-level work of a capture.
    Returns TRUE if it succeeds, FALSE otherwise. */
 int
 capture(void)
 {
-  GtkWidget  *cap_w, *main_vb, *count_lb, *sctp_lb, *tcp_lb, *udp_lb, *icmp_lb,
-             *ospf_lb, *gre_lb, *netbios_lb, *ipx_lb, *vines_lb, *other_lb, *stop_bt;
+  GtkWidget  *cap_w, *main_vb, *stop_bt, *counts_tb;
   pcap_t     *pch;
   int         pcap_encap;
   int         snaplen;
   gchar       err_str[PCAP_ERRBUF_SIZE], label_str[64];
-  loop_data   ld;
   bpf_u_int32 netnum, netmask;
   time_t      upd_time, cur_time;
-  int         err, inpkts;
+  int         err, inpkts, i;
   char        errmsg[4096+1];
 #ifndef _WIN32
   static const char ppamsg[] = "can't find PPA for ";
@@ -970,6 +1054,25 @@ capture(void)
   int         pipe_fd = -1;
   struct pcap_hdr hdr;
 #endif
+  struct {
+      const gchar *title;
+      gint *value_ptr;
+      GtkWidget *label, *value, *percent;
+  } stats[] = {
+      { "Total", &ld.counts.total },
+      { "SCTP", &ld.counts.sctp },
+      { "TCP", &ld.counts.tcp },
+      { "UDP", &ld.counts.udp },
+      { "ICMP", &ld.counts.icmp },
+      { "OSPF", &ld.counts.ospf },
+      { "GRE", &ld.counts.gre },
+      { "NetBIOS", &ld.counts.netbios },
+      { "IPX", &ld.counts.ipx },
+      { "VINES", &ld.counts.vines },
+      { "Other", &ld.counts.other }
+  };
+
+#define N_STATS (sizeof stats / sizeof stats[0])
 
   /* Initialize Windows Socket if we are in a WIN32 OS 
      This needs to be done before querying the interface for network/netmask */
@@ -1003,7 +1106,8 @@ capture(void)
   ld.pdh            = NULL;
 
   /* Open the network interface to capture from it. */
-  pch = pcap_open_live(cfile.iface, cfile.snap, 1, CAP_READ_TIMEOUT, err_str);
+  pch = pcap_open_live(cfile.iface, cfile.snap, promisc_mode,
+			CAP_READ_TIMEOUT, err_str);
 
   if (pch == NULL) {
 #ifdef _WIN32
@@ -1056,7 +1160,7 @@ capture(void)
 	  "Ethereal may not be able to capture packets.\n"
 	  "\n"
 	  "To fix this, you will need to download the source to Ethereal\n"
-	  "from ethereal.zing.org if you have not already done so, read\n"
+	  "from www.ethereal.com if you have not already done so, read\n"
 	  "the instructions in the \"README.hpux\" file in the source\n"
 	  "distribution, download the source to libpcap if you have not\n"
 	  "already done so, patch libpcap as per the instructions, rebuild\n"
@@ -1079,9 +1183,16 @@ capture(void)
   if (cfile.cfilter && !ld.from_pipe) {
     /* A capture filter was specified; set it up. */
     if (pcap_lookupnet (cfile.iface, &netnum, &netmask, err_str) < 0) {
-      snprintf(errmsg, sizeof errmsg,
-        "Can't use filter:  Couldn't obtain netmask info (%s).", err_str);
-      goto error;
+      /*
+       * Well, we can't get the netmask for this interface; it's used
+       * only for filters that check for broadcast IP addresses, so
+       * we just punt and use 0.  It might be nice to warn the user,
+       * but that's a pain in a GUI application, as it'd involve popping
+       * up a message box, and it's not clear how often this would make
+       * a difference (only filters that check for IP broadcast addresses
+       * use the netmask).
+       */
+      netmask = 0;
     }
     if (pcap_compile(pch, &cfile.fcode, cfile.cfilter, 1, netmask) < 0) {
       snprintf(errmsg, sizeof errmsg, "Unable to parse filter string (%s).",
@@ -1176,49 +1287,35 @@ capture(void)
   gtk_container_add(GTK_CONTAINER(cap_w), main_vb);
   gtk_widget_show(main_vb);
 
-  count_lb = gtk_label_new("Count: 0");
-  gtk_box_pack_start(GTK_BOX(main_vb), count_lb, FALSE, FALSE, 3);
-  gtk_widget_show(count_lb);
+  /* Individual statistic elements */
+  counts_tb = gtk_table_new(N_STATS, 3, TRUE);
+  gtk_box_pack_start(GTK_BOX(main_vb), counts_tb, TRUE, TRUE, 3);
+  gtk_widget_show(counts_tb);
 
-  sctp_lb = gtk_label_new("SCTP: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), sctp_lb, FALSE, FALSE, 3);
-  gtk_widget_show(sctp_lb);
+  for (i = 0; i < N_STATS; i++) {
+      stats[i].label = gtk_label_new(stats[i].title);
+      gtk_misc_set_alignment(GTK_MISC(stats[i].label), 0.0f, 0.0f);
 
-  tcp_lb = gtk_label_new("TCP: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), tcp_lb, FALSE, FALSE, 3);
-  gtk_widget_show(tcp_lb);
+      stats[i].value = gtk_label_new("0");
+      gtk_misc_set_alignment(GTK_MISC(stats[i].value), 0.0f, 0.0f);
 
-  udp_lb = gtk_label_new("UDP: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), udp_lb, FALSE, FALSE, 3);
-  gtk_widget_show(udp_lb);
+      stats[i].percent = gtk_label_new("0.0%");
+      gtk_misc_set_alignment(GTK_MISC(stats[i].percent), 0.0f, 0.0f);
 
-  icmp_lb = gtk_label_new("ICMP: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), icmp_lb, FALSE, FALSE, 3);
-  gtk_widget_show(icmp_lb);
+      gtk_table_attach_defaults(GTK_TABLE(counts_tb),
+                                stats[i].label, 0, 1, i, i + 1);
 
-  ospf_lb = gtk_label_new("OSPF: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), ospf_lb, FALSE, FALSE, 3);
-  gtk_widget_show(ospf_lb);
+      gtk_table_attach(GTK_TABLE(counts_tb),
+                       stats[i].value,
+                       1, 2, i, i + 1, 0, 0, 5, 0);
 
-  gre_lb = gtk_label_new("GRE: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), gre_lb, FALSE, FALSE, 3);
-  gtk_widget_show(gre_lb);
+      gtk_table_attach_defaults(GTK_TABLE(counts_tb),
+                                stats[i].percent, 2, 3, i, i + 1);
 
-  netbios_lb = gtk_label_new("NetBIOS: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), netbios_lb, FALSE, FALSE, 3);
-  gtk_widget_show(netbios_lb);
-
-  ipx_lb = gtk_label_new("IPX: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), ipx_lb, FALSE, FALSE, 3);
-  gtk_widget_show(ipx_lb);
-
-  vines_lb = gtk_label_new("VINES: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), vines_lb, FALSE, FALSE, 3);
-  gtk_widget_show(vines_lb);
-
-  other_lb = gtk_label_new("Other: 0 (0.0%)");
-  gtk_box_pack_start(GTK_BOX(main_vb), other_lb, FALSE, FALSE, 3);
-  gtk_widget_show(other_lb);
+      gtk_widget_show(stats[i].label);
+      gtk_widget_show(stats[i].value);
+      gtk_widget_show(stats[i].percent);
+  }
 
   /* allow user to either click a stop button, or the close button on
 	the window to stop a capture in progress. */
@@ -1240,6 +1337,14 @@ capture(void)
 #ifdef linux
   if (!ld.from_pipe) pcap_fd = pcap_fileno(pch);
 #endif
+
+#ifndef _WIN32
+  /*
+   * Catch SIGUSR1, so that we exit cleanly if the parent process
+   * kills us with it due to the user selecting "Capture->Stop".
+   */
+  signal(SIGUSR1, stop_capture);
+#endif
   while (ld.go) {
     while (gtk_events_pending()) gtk_main_iteration();
 
@@ -1302,48 +1407,17 @@ capture(void)
     if (cur_time > upd_time) {
       upd_time = cur_time;
 
-      sprintf(label_str, "Count: %d", ld.counts.total);
-      gtk_label_set(GTK_LABEL(count_lb), label_str);
+      for (i = 0; i < N_STATS; i++) {
+          snprintf(label_str, sizeof(label_str), "%d",
+                   *stats[i].value_ptr);
 
-      sprintf(label_str, "SCTP: %d (%.1f%%)", ld.counts.sctp,
-                pct(ld.counts.sctp, ld.counts.total));
-      gtk_label_set(GTK_LABEL(sctp_lb), label_str);
+          gtk_label_set(GTK_LABEL(stats[i].value), label_str);
 
-      sprintf(label_str, "TCP: %d (%.1f%%)", ld.counts.tcp,
-		pct(ld.counts.tcp, ld.counts.total));
-      gtk_label_set(GTK_LABEL(tcp_lb), label_str);
+          snprintf(label_str, sizeof(label_str), "(%.1f%%)",
+                   pct(*stats[i].value_ptr, ld.counts.total));
 
-      sprintf(label_str, "UDP: %d (%.1f%%)", ld.counts.udp,
-		pct(ld.counts.udp, ld.counts.total));
-      gtk_label_set(GTK_LABEL(udp_lb), label_str);
-
-      sprintf(label_str, "ICMP: %d (%.1f%%)", ld.counts.icmp,
-		pct(ld.counts.icmp, ld.counts.total));
-      gtk_label_set(GTK_LABEL(icmp_lb), label_str);
-
-      sprintf(label_str, "OSPF: %d (%.1f%%)", ld.counts.ospf,
-		pct(ld.counts.ospf, ld.counts.total));
-      gtk_label_set(GTK_LABEL(ospf_lb), label_str);
-
-      sprintf(label_str, "GRE: %d (%.1f%%)", ld.counts.gre,
-		pct(ld.counts.gre, ld.counts.total));
-      gtk_label_set(GTK_LABEL(gre_lb), label_str);
-
-      sprintf(label_str, "NetBIOS: %d (%.1f%%)", ld.counts.netbios,
-		pct(ld.counts.netbios, ld.counts.total));
-      gtk_label_set(GTK_LABEL(netbios_lb), label_str);
-
-      sprintf(label_str, "IPX: %d (%.1f%%)", ld.counts.ipx,
-		pct(ld.counts.ipx, ld.counts.total));
-      gtk_label_set(GTK_LABEL(ipx_lb), label_str);
-
-      sprintf(label_str, "VINES: %d (%.1f%%)", ld.counts.vines,
-		pct(ld.counts.vines, ld.counts.total));
-      gtk_label_set(GTK_LABEL(vines_lb), label_str);
-
-      sprintf(label_str, "Other: %d (%.1f%%)", ld.counts.other,
-		pct(ld.counts.other, ld.counts.total));
-      gtk_label_set(GTK_LABEL(other_lb), label_str);
+          gtk_label_set(GTK_LABEL(stats[i].percent), label_str);
+      }
 
       /* do sync here, too */
       fflush(wtap_dump_file(ld.pdh));
@@ -1395,6 +1469,11 @@ capture(void)
 #endif
     pcap_close(pch);
 
+#ifdef WIN32
+  /* Shut down windows sockets */
+  WSACleanup();
+#endif
+
   gtk_grab_remove(GTK_WIDGET(cap_w));
   gtk_widget_destroy(GTK_WIDGET(cap_w));
 
@@ -1445,6 +1524,12 @@ pct(gint num, gint denom) {
   }
 }
 
+static void
+stop_capture(int signo)
+{
+  ld.go = FALSE;
+}
+
 static void
 capture_delete_cb(GtkWidget *w, GdkEvent *event, gpointer data) {
   capture_stop_cb(NULL, data);
@@ -1457,6 +1542,18 @@ capture_stop_cb(GtkWidget *w, gpointer data) {
   ld->go = FALSE;
 }
 
+void
+capture_stop(void)
+{
+  /*
+   * XXX - find some way of signaling the child in Win32.
+   */
+#ifndef _WIN32
+  if (fork_child != -1)
+      kill(fork_child, SIGUSR1);
+#endif
+}
+
 static void
 capture_pcap_cb(u_char *user, const struct pcap_pkthdr *phdr,
   const u_char *pd) {
@@ -1497,7 +1594,7 @@ capture_pcap_cb(u_char *user, const struct pcap_pkthdr *phdr,
     case WTAP_ENCAP_FDDI_BITSWAPPED:
       capture_fddi(pd, &ld->counts);
       break;
-    case WTAP_ENCAP_TR:
+    case WTAP_ENCAP_TOKEN_RING:
       capture_tr(pd, 0, &ld->counts);
       break;
     case WTAP_ENCAP_NULL:
