@@ -1,7 +1,7 @@
 /* file.c
  * File I/O routines
  *
- * $Id: file.c,v 1.221 2000/09/12 03:28:34 guy Exp $
+ * $Id: file.c,v 1.226 2000/11/19 08:53:53 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -69,6 +69,8 @@
 # include <netinet/in.h>
 #endif
 
+#include <epan.h>
+
 #include "gtk/main.h"
 #include "column.h"
 #include "packet.h"
@@ -83,6 +85,7 @@
 #include "dfilter.h"
 #include "conversation.h"
 #include "globals.h"
+#include "gtk/colors.h"
 
 #include "plugins.h"
 
@@ -142,7 +145,7 @@ open_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
   close_cap_file(cf, info_bar);
 
   /* Initialize the table of conversations. */
-  conversation_init();
+  epan_conversation_init();
 
   /* Initialize protocol-specific variables */
   init_all_protocols();
@@ -387,7 +390,7 @@ read_cap_file(capture_file *cf, int *err)
   if (cf->first_displayed != NULL)
     gtk_signal_emit_by_name(GTK_OBJECT(packet_list), "select_row", 0);
 
-  if (data_offset < 0) {
+  if (*err != 0) {
     /* Put up a message box noting that the read failed somewhere along
        the line.  Don't throw out the stuff we managed to read, though,
        if any. */
@@ -492,7 +495,7 @@ continue_tail_cap_file(capture_file *cf, int to_read, int *err)
        "finish_tail_cap_file()" will be called, and it will clean up
        and exit. */
     return READ_ABORTED;
-  } else if (data_offset < 0) {
+  } else if (*err != 0) {
     /* We got an error reading the capture file.
        XXX - pop up a dialog box? */
     return (READ_ERROR);
@@ -561,7 +564,7 @@ finish_tail_cap_file(capture_file *cf, int *err)
   set_menus_for_capture_file(TRUE);
   set_menus_for_unsaved_capture_file(!cf->user_saved);
 
-  if (data_offset < 0) {
+  if (*err != 0) {
     /* We got an error reading the capture file.
        XXX - pop up a dialog box? */
     return (READ_ERROR);
@@ -602,6 +605,7 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
   apply_color_filter_args args;
   gint          i, row;
   proto_tree   *protocol_tree = NULL;
+  epan_dissect_t *edt;
 
   /* We don't yet have a color filter to apply. */
   args.colorf = NULL;
@@ -616,7 +620,8 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
 
   fdata->cinfo = &cf->cinfo;
   for (i = 0; i < fdata->cinfo->num_cols; i++) {
-    fdata->cinfo->col_data[i][0] = '\0';
+    fdata->cinfo->col_buf[i][0] = '\0';
+    fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
   }
 
   /* If either
@@ -638,7 +643,7 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
     protocol_tree = proto_tree_create_root();
 
   /* Dissect the frame. */
-  dissect_packet(pseudo_header, buf, fdata, protocol_tree);
+  edt = epan_dissect_new(pseudo_header, buf, fdata, protocol_tree);
 
   /* If we have a display filter, apply it if we're refiltering, otherwise
      leave the "passed_dfilter" flag alone.
@@ -670,6 +675,8 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf,
   if (protocol_tree != NULL)
     proto_tree_free(protocol_tree);
 
+  epan_dissect_free(edt);
+
   if (fdata->flags.passed_dfilter) {
     /* This frame passed the display filter, so add it to the clist. */
 
@@ -757,6 +764,7 @@ read_packet(capture_file *cf, int offset)
   int           passed;
   proto_tree   *protocol_tree;
   frame_data   *plist_end;
+  epan_dissect_t *edt;
 
   /* Allocate the next list entry, and add it to the list. */
   fdata = g_mem_chunk_alloc(cf->plist_chunk);
@@ -778,9 +786,10 @@ read_packet(capture_file *cf, int offset)
   passed = TRUE;
   if (cf->rfcode) {
     protocol_tree = proto_tree_create_root();
-    dissect_packet(pseudo_header, buf, fdata, protocol_tree);
+    edt = epan_dissect_new(pseudo_header, buf, fdata, protocol_tree);
     passed = dfilter_apply(cf->rfcode, protocol_tree, buf, fdata->cap_len);
     proto_tree_free(protocol_tree);
+    epan_dissect_free(edt);
   }   
   if (passed) {
     plist_end = cf->plist_end;
@@ -901,7 +910,7 @@ rescan_packets(capture_file *cf, const char *action, gboolean refilter,
        by that dissector. */
 
     /* Initialize the table of conversations. */
-    conversation_init();
+    epan_conversation_init();
 
     /* Initialize protocol-specific variables */
     init_all_protocols();
@@ -1050,6 +1059,7 @@ print_packets(capture_file *cf, print_args_t *print_args)
   char        *cp;
   int         column_len;
   int         line_len;
+  epan_dissect_t *edt = NULL;
 
   cf->print_fh = open_print_dest(print_args->to_file, print_args->dest);
   if (cf->print_fh == NULL)
@@ -1163,9 +1173,10 @@ print_packets(capture_file *cf, print_args_t *print_args)
            the logical protocol tree. */
         fdata->cinfo = &cf->cinfo;
         for (i = 0; i < fdata->cinfo->num_cols; i++) {
-          fdata->cinfo->col_data[i][0] = '\0';
+          fdata->cinfo->col_buf[i][0] = '\0';
+          fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
         }
-        dissect_packet(&cf->pseudo_header, cf->pd, fdata, NULL);
+        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, NULL);
         fill_in_columns(fdata);
         cp = &line_buf[0];
         line_len = 0;
@@ -1202,7 +1213,7 @@ print_packets(capture_file *cf, print_args_t *print_args)
 
         /* Create the logical protocol tree. */
         protocol_tree = proto_tree_create_root();
-        dissect_packet(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
+        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
 
         /* Print the information in that tree. */
         proto_tree_print(FALSE, print_args, (GNode *)protocol_tree,
@@ -1219,6 +1230,7 @@ print_packets(capture_file *cf, print_args_t *print_args)
         /* Print a blank line if we print anything after this. */
         print_separator = TRUE;
       }
+      epan_dissect_free(edt);
     }
   }
 
@@ -1322,7 +1334,7 @@ change_time_formats(capture_file *cf)
           if (cf->cinfo.fmt_matx[i][COL_CLS_TIME]) {
             /* This is one of the columns that shows the time in
                "command-line-specified" format; update it. */
-            cf->cinfo.col_data[i][0] = '\0';
+            cf->cinfo.col_buf[i][0] = '\0';
             col_set_cls_time(fdata, i);
             gtk_clist_set_text(GTK_CLIST(packet_list), row, i,
 			  cf->cinfo.col_data[i]);
@@ -1378,6 +1390,7 @@ find_packet(capture_file *cf, dfilter *sfcode)
   proto_tree *protocol_tree;
   gboolean frame_matched;
   int row;
+  epan_dissect_t	*edt;
 
   start_fd = cf->current_frame;
   if (start_fd != NULL)  {
@@ -1442,9 +1455,10 @@ find_packet(capture_file *cf, dfilter *sfcode)
         protocol_tree = proto_tree_create_root();
         wtap_seek_read(cf->wth, fdata->file_off, &cf->pseudo_header,
         		cf->pd, fdata->cap_len);
-        dissect_packet(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
+        edt = epan_dissect_new(&cf->pseudo_header, cf->pd, fdata, protocol_tree);
         frame_matched = dfilter_apply(sfcode, protocol_tree, cf->pd, fdata->cap_len);
         proto_tree_free(protocol_tree);
+	epan_dissect_free(edt);
         if (frame_matched) {
           new_fd = fdata;
           break;	/* found it! */
@@ -1550,7 +1564,7 @@ select_packet(capture_file *cf, int row)
       proto_tree_free(cf->protocol_tree);
   cf->protocol_tree = proto_tree_create_root();
   proto_tree_is_visible = TRUE;
-  dissect_packet(&cf->pseudo_header, cf->pd, cf->current_frame,
+  cf->edt = epan_dissect_new(&cf->pseudo_header, cf->pd, cf->current_frame,
 		cf->protocol_tree);
   proto_tree_is_visible = FALSE;
 
@@ -1571,6 +1585,7 @@ unselect_packet(capture_file *cf)
   if (cf->protocol_tree != NULL) {
     proto_tree_free(cf->protocol_tree);
     cf->protocol_tree = NULL;
+    epan_dissect_free(cf->edt);
   }
 
   finfo_selected = NULL;
