@@ -1,7 +1,7 @@
 /* gui_prefs.c
  * Dialog box for GUI preferences
  *
- * $Id: gui_prefs.c,v 1.19 2000/09/09 10:35:53 guy Exp $
+ * $Id: gui_prefs.c,v 1.23 2000/11/18 21:41:38 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -28,6 +28,7 @@
 #endif
 
 #include <errno.h>
+#include <gtk/gtk.h>
 
 #include "globals.h"
 #include "gui_prefs.h"
@@ -45,11 +46,11 @@
 
 static void create_option_menu(GtkWidget *main_vb, const gchar *key,
     GtkWidget *main_tb, int table_position,
-    const gchar *label_text, const enum_val *enumvals, gint current_val);
+    const gchar *label_text, const enum_val_t *enumvals, gint current_val);
 static void font_browse_cb(GtkWidget *w, gpointer data);
 static void font_browse_ok_cb(GtkWidget *w, GtkFontSelectionDialog *fs);
 static void font_browse_destroy(GtkWidget *win, gpointer data);
-static gint fetch_enum_value(gpointer control, const enum_val *enumvals);
+static gint fetch_enum_value(gpointer control, const enum_val_t *enumvals);
 static void color_browse_cb(GtkWidget *w, gpointer data);
 static void update_text_color(GtkWidget *w, gpointer data);
 static void update_current_color(GtkWidget *w, gpointer data);
@@ -73,19 +74,19 @@ static void fetch_colors(void);
 #define COLOR_SAMPLE_PTR_KEY	"color_sample_ptr"
 #define COLOR_SELECTION_PTR_KEY	"color_selection_ptr"
 
-static const enum_val scrollbar_placement_vals[] = {
+static const enum_val_t scrollbar_placement_vals[] = {
 	{ "Left",  FALSE },
 	{ "Right", TRUE },
 	{ NULL,    0 }
 };
 
-static const enum_val selection_mode_vals[] = {
+static const enum_val_t selection_mode_vals[] = {
 	{ "Selects", FALSE },
 	{ "Browses", TRUE },
 	{ NULL,      0 }
 };
 
-static const enum_val line_style_vals[] = {
+static const enum_val_t line_style_vals[] = {
 	{ "None",   0 },
 	{ "Solid",  1 },
 	{ "Dotted", 2 },
@@ -93,7 +94,7 @@ static const enum_val line_style_vals[] = {
 	{ NULL,     0 }
 };
 
-static const enum_val expander_style_vals[] = {
+static const enum_val_t expander_style_vals[] = {
 	{ "None",     0 },
 	{ "Square",   1 },
 	{ "Triangle", 2 },
@@ -101,7 +102,7 @@ static const enum_val expander_style_vals[] = {
 	{ NULL,       0 }
 };
 
-static const enum_val highlight_style_vals[] = {
+static const enum_val_t highlight_style_vals[] = {
   	{ "Bold",     0 },
   	{ "Inverse",  1 },
 	{ NULL,       0 }
@@ -193,11 +194,11 @@ gui_prefs_show(void)
 static void
 create_option_menu(GtkWidget *main_vb, const gchar *key,
     GtkWidget *main_tb, int table_position,
-    const gchar *label_text, const enum_val *enumvals, gint current_val)
+    const gchar *label_text, const enum_val_t *enumvals, gint current_val)
 {
 	GtkWidget *label, *menu, *menu_item, *option_menu;
 	int menu_index, index;
-	const enum_val *enum_valp;
+	const enum_val_t *enum_valp;
 
 	label = gtk_label_new(label_text);
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -319,6 +320,14 @@ font_browse_ok_cb(GtkWidget *w, GtkFontSelectionDialog *fs)
 
 	font_name = g_strdup(gtk_font_selection_dialog_get_font_name(
 	      GTK_FONT_SELECTION_DIALOG(fs)));
+	if (font_name == NULL) {
+		/* No font was selected; let the user know, but don't
+		   tear down the font selection dialog, so they can
+		   try again. */
+		simple_dialog(ESD_TYPE_CRIT | ESD_TYPE_MODAL, NULL,
+		   "You have not selected a font.");
+		return;
+	}
 
 	/* Get the name that the boldface version of that font would have. */
 	bold_font_name = boldify(font_name);
@@ -329,7 +338,7 @@ font_browse_ok_cb(GtkWidget *w, GtkFontSelectionDialog *fs)
 		/* Oops, that font didn't work.
 		   Tell the user, but don't tear down the font selection
 		   dialog, so that they can try again. */
-		simple_dialog(ESD_TYPE_CRIT, NULL,
+		simple_dialog(ESD_TYPE_CRIT | ESD_TYPE_MODAL, NULL,
 		   "The font you selected cannot be loaded.");
 
 		g_free(font_name);
@@ -342,7 +351,7 @@ font_browse_ok_cb(GtkWidget *w, GtkFontSelectionDialog *fs)
 		/* Oops, that font didn't work.
 		   Tell the user, but don't tear down the font selection
 		   dialog, so that they can try again. */
-		simple_dialog(ESD_TYPE_CRIT, NULL,
+		simple_dialog(ESD_TYPE_CRIT | ESD_TYPE_MODAL, NULL,
 		   "The font you selected doesn't have a boldface version.");
 
 		g_free(font_name);
@@ -380,7 +389,7 @@ font_browse_destroy(GtkWidget *win, gpointer data)
 }
 
 static gint
-fetch_enum_value(gpointer control, const enum_val *enumvals)
+fetch_enum_value(gpointer control, const enum_val_t *enumvals)
 {
 	GtkWidget *label;
 	char *label_string;
