@@ -1,6 +1,6 @@
 /* main.c
  *
- * $Id: main.c,v 1.156 2000/09/09 10:26:53 guy Exp $
+ * $Id: main.c,v 1.164 2000/11/19 08:54:37 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -95,6 +95,8 @@
 #include "getopt.h"
 #endif
 
+#include <epan.h>
+
 #include "main.h"
 #include "timestamp.h"
 #include "packet.h"
@@ -116,6 +118,7 @@
 #include "packet_win.h"
 #include "gtkglobals.h"
 #include "plugins.h"
+#include "colors.h"
 
 packet_info  pi;
 capture_file cfile;
@@ -788,7 +791,7 @@ print_usage(void) {
   fprintf(stderr, "This is GNU " PACKAGE " " VERSION ", compiled with %s\n",
 	  comp_info_str);
 #ifdef HAVE_LIBPCAP
-  fprintf(stderr, "%s [ -vh ] [ -kQS ] [ -B <byte view height> ] [ -c count ]\n",
+  fprintf(stderr, "%s [ -vh ] [ -kpQS ] [ -B <byte view height> ] [ -c count ]\n",
 	  PACKAGE);
   fprintf(stderr, "\t[ -f <capture filter> ] [ -i interface ] [ -m <medium font> ] \n");
   fprintf(stderr, "\t[ -n ] [ -o <preference setting> ] ... [ -P <packet list height> ]\n");
@@ -815,6 +818,7 @@ main(int argc, char *argv[])
   int                  opt;
   extern char         *optarg;
   gboolean             arg_error = FALSE;
+
 #ifdef HAVE_LIBPCAP
 #ifdef WIN32
   char pcap_version[] = "0.4a6";
@@ -822,6 +826,11 @@ main(int argc, char *argv[])
   extern char          pcap_version[];
 #endif
 #endif
+  
+#ifdef WIN32
+  WSADATA 	       wsaData; 
+#endif
+
   char                *gpf_path, *pf_path;
   int                  gpf_open_errno, pf_open_errno;
   int                  err;
@@ -853,7 +862,7 @@ main(int argc, char *argv[])
      "-G" flag, as the "-G" flag dumps a list of fields registered
      by the dissectors, and we must do it before we read the preferences,
      in case any dissectors register preferences. */
-  dissect_init();
+  epan_init(PLUGIN_DIR);
 
   /* Now register the preferences for any non-dissector modules.
      We must do that before we read the preferences as well. */
@@ -896,10 +905,10 @@ main(int argc, char *argv[])
 
   /* Initialize the capture file struct */
   cfile.plist		= NULL;
-  cfile.plist_end		= NULL;
+  cfile.plist_end	= NULL;
   cfile.wth		= NULL;
-  cfile.filename		= NULL;
-  cfile.user_saved		= FALSE;
+  cfile.filename	= NULL;
+  cfile.user_saved	= FALSE;
   cfile.is_tempfile	= FALSE;
   cfile.rfcode		= NULL;
   cfile.dfilter		= NULL;
@@ -908,16 +917,17 @@ main(int argc, char *argv[])
   cfile.cfilter		= g_strdup(EMPTY_FILTER);
 #endif
   cfile.iface		= NULL;
-  cfile.save_file		= NULL;
+  cfile.save_file	= NULL;
   cfile.save_file_fd	= -1;
   cfile.snap		= WTAP_MAX_PACKET_SIZE;
   cfile.count		= 0;
   cfile.cinfo.num_cols	= prefs->num_cols;
-  cfile.cinfo.col_fmt      = (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_fmt	= (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
   cfile.cinfo.fmt_matx	= (gboolean **) g_malloc(sizeof(gboolean *) * cfile.cinfo.num_cols);
   cfile.cinfo.col_width	= (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
-  cfile.cinfo.col_title    = (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_title	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
   cfile.cinfo.col_data	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_buf	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
 
   /* Assemble the compile-time options */
   snprintf(comp_info_str, 256,
@@ -963,7 +973,7 @@ main(int argc, char *argv[])
    );
 
   /* Now get our args */
-  while ((opt = getopt(argc, argv, "B:c:Df:hi:km:no:P:Qr:R:Ss:t:T:w:W:vZ:")) != EOF) {
+  while ((opt = getopt(argc, argv, "B:c:Df:hi:km:no:pP:Qr:R:Ss:t:T:w:W:vZ:")) != EOF) {
     switch (opt) {
       case 'B':        /* Byte view pane height */
         bv_size = atoi(optarg);
@@ -1029,6 +1039,14 @@ main(int argc, char *argv[])
           break;
         }
         break;
+      case 'p':        /* Don't capture in promiscuous mode */
+#ifdef HAVE_LIBPCAP
+	promisc_mode = 0;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+	break;
       case 'P':        /* Packet list pane height */
         pl_size = atoi(optarg);
         break;
@@ -1071,13 +1089,15 @@ main(int argc, char *argv[])
           timestamp_type = RELATIVE;
         else if (strcmp(optarg, "a") == 0)
           timestamp_type = ABSOLUTE;
+        else if (strcmp(optarg, "ad") == 0)
+          timestamp_type = ABSOLUTE_WITH_DATE;
         else if (strcmp(optarg, "d") == 0)
           timestamp_type = DELTA;
         else {
           fprintf(stderr, "ethereal: Invalid time stamp type \"%s\"\n",
             optarg);
           fprintf(stderr, "It must be \"r\" for relative, \"a\" for absolute,\n");
-          fprintf(stderr, "or \"d\" for delta.\n");
+          fprintf(stderr, "\"ad\" for absolute with date, or \"d\" for delta.\n");
           exit(1);
         }
         break;
@@ -1128,6 +1148,11 @@ main(int argc, char *argv[])
     }
   }
 
+#ifdef WIN32
+  /* Start windows sockets */
+  WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
+#endif
+
   /* Notify all registered modules that have had any of their preferences
      changed either from one of the preferences file or from the command
      line that its preferences have changed. */
@@ -1181,10 +1206,11 @@ main(int argc, char *argv[])
     cfile.cinfo.fmt_matx[i] = (gboolean *) g_malloc0(sizeof(gboolean) *
       NUM_COL_FMTS);
     get_column_format_matches(cfile.cinfo.fmt_matx[i], cfile.cinfo.col_fmt[i]);
+    cfile.cinfo.col_data[i] = NULL;
     if (cfile.cinfo.col_fmt[i] == COL_INFO)
-      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
+      cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
     else
-      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+      cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
   }
 
   if (cfile.snap < 1)
@@ -1343,9 +1369,14 @@ main(int argc, char *argv[])
 
   gtk_main();
 
-  dissect_cleanup();
+  epan_cleanup();
   g_free(rc_file);
 
+#ifdef WIN32
+  /* Shutdown windows sockets */
+  WSACleanup();
+#endif
+
   gtk_exit(0);
 
   /* This isn't reached, but we need it to keep GCC from complaining
