@@ -1,7 +1,7 @@
-/* ethereal.h
+/* main.h
  * Global defines, etc.
  *
- * $Id: main.h,v 1.14 2000/02/20 14:52:28 deniel Exp $
+ * $Id: main.h,v 1.20 2000/09/08 10:59:17 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -30,17 +30,8 @@
 #include "config.h"
 #endif
 
-#ifndef __GLOBALS_H__
 #include "globals.h"
-#endif
 
-#ifdef WIN32
-#define MONO_MEDIUM_FONT "-*-lucida console-medium-*-*-*-*-100-*-*-*-*-*-*"
-#define MONO_BOLD_FONT "-*-lucida console-bold-*-*-*-*-100-*-*-*-*-*-*"
-#else
-#define MONO_MEDIUM_FONT "-*-lucidatypewriter-medium-r-normal-*-*-120-*-*-*-*-iso8859-1"
-#define MONO_BOLD_FONT "-*-lucidatypewriter-bold-r-normal-*-*-120-*-*-*-*-iso8859-1"
-#endif
 #define RC_FILE PF_DIR "/gtkrc"
 #define DEF_WIDTH 750
 #define DEF_HEIGHT 550
@@ -59,7 +50,6 @@ extern GtkStyle *item_style;
 
 void about_ethereal( GtkWidget *, gpointer);
 void blank_packetinfo();
-void follow_stream_cb( GtkWidget *, gpointer);
 void match_selected_cb( GtkWidget *, gpointer);
 void file_quit_cmd_cb(GtkWidget *, gpointer);
 void file_print_cmd_cb(GtkWidget *, gpointer);
@@ -68,5 +58,11 @@ void tools_plugins_cmd_cb(GtkWidget *, gpointer);
 void expand_all_cb(GtkWidget *, gpointer);
 void collapse_all_cb(GtkWidget *, gpointer);
 void resolve_name_cb(GtkWidget *, gpointer);
+void mark_frame_cb(GtkWidget *, gpointer);
+void mark_all_frames_cb(GtkWidget *w, gpointer);
+void unmark_all_frames_cb(GtkWidget *w, gpointer);
+void update_marked_frames(void);
+
+char *boldify(const char *);
 
 #endif /* __MAIN_H__ */
