@@ -1,7 +1,7 @@
 /* plugins_dlg.c
  * Dialog boxes for plugins
  *
- * $Id: plugins_dlg.c,v 1.18 2000/08/23 06:56:06 guy Exp $
+ * $Id: plugins_dlg.c,v 1.21 2000/11/18 09:59:04 grahamb Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,6 +35,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <gtk/gtk.h>
 
 #include "globals.h"
 #include "plugins.h"
@@ -43,11 +44,6 @@
 #include "prefs_dlg.h"
 #include "simple_dialog.h"
 
-#ifdef PLUGINS_NEED_ADDRESS_TABLE
-#include "plugins/plugin_api.h"
-extern plugin_address_table_t	patable;
-#endif
-
 #ifdef HAVE_PLUGINS
 
 static gint selected_row;
@@ -245,39 +241,20 @@ void plugins_clist_unselect_cb(GtkWidget *clist, gint row, gint column,
 static void
 plugins_enable_cb(GtkWidget *button, gpointer clist)
 {
-    plugin    *pt_plug;
-    gpointer symbol;
-    void     (*plugin_init)(void*);
+    char      *errmsg;
 
     /* nothing selected */
     if (selected_row == -1) return;
     /* already enabled */
     if (strcmp(selected_enabled, "Yes") == 0) return;
 
-    if ((pt_plug = enable_plugin(selected_name, selected_version)) == NULL)
+    errmsg = enable_plugin(selected_name, selected_version);
+    if (errmsg != NULL)
     {
-	simple_dialog(ESD_TYPE_CRIT, NULL, "Plugin not found");
+	simple_dialog(ESD_TYPE_CRIT, NULL, errmsg);
 	return;
     }
 
-    /* Try to get the initialization routine for the plugin, and, if it
-       has one, call it. */
-    if (g_module_symbol(pt_plug->handle, "plugin_init", &symbol) == TRUE) {
-	plugin_init = symbol;
-#ifdef PLUGINS_NEED_ADDRESS_TABLE
-		    plugin_init(&patable);
-#else
-		    plugin_init(NULL);
-#endif
-	}
-#ifdef PLUGINS_NEED_ADDRESS_TABLE
-	else {
-		simple_dialog(ESD_TYPE_WARN, NULL, "Failed to find plugin_init()");
-		return;
-	}
-#endif
-
-
     gtk_clist_set_text(GTK_CLIST(clist), selected_row, 3, "Yes");
 }
 
