@@ -1,10 +1,13 @@
 /* packet-bxxp.c
  * Routines for BXXP packet disassembly
  *
- * $Id: packet-bxxp.c,v 1.3 2000/09/12 02:24:19 sharpe Exp $
+ * $Id: packet-bxxp.c,v 1.11 2000/11/19 08:53:56 guy Exp $
  *
  * Copyright (c) 2000 by Richard Sharpe <rsharpe@ns.aus.com>
  *
+ * Developed with funding from InvisibleWorlds (www.invisibleworlds.com) via
+ * Collab.Net.
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs
  * Copyright 1999 Gerald Combs
@@ -49,14 +52,22 @@
 #define TCP_PORT_BXXP 10288
 void proto_reg_handoff_bxxp(void);
 
+
+static int global_bxxp_tcp_port = TCP_PORT_BXXP;
+static int global_bxxp_strict_term = TRUE;
+
 static int proto_bxxp = -1;
 
 static int hf_bxxp_req = -1;
+static int hf_bxxp_req_chan = -1;
+static int hf_bxxp_rsp_chan = -1;
+static int hf_bxxp_seq_chan = -1;
 static int hf_bxxp_rsp = -1;
 static int hf_bxxp_seq = -1;
-static int hf_bxxp_end = -1;    /* Do we need this one?        */
-static int hf_bxxp_complete = -1;   /* More data follows */
-static int hf_bxxp_intermediate = -1; /* No More data follows */
+static int hf_bxxp_end = -1;
+static int hf_bxxp_proto_viol = -1;
+static int hf_bxxp_complete = -1;   /* No More data follows */
+static int hf_bxxp_intermediate = -1; /* More data follows */
 static int hf_bxxp_serial = -1;
 static int hf_bxxp_seqno = -1;
 static int hf_bxxp_size = -1;
@@ -66,6 +77,22 @@ static int hf_bxxp_negative = -1;
 static int hf_bxxp_ackno = -1;
 static int hf_bxxp_window = -1;
 
+/* Arrays of hf entry pointers for some routines to use. If you want more
+ * hidden items added for a field, add them to the list before the NULL, 
+ * and the various routines that these are passed to will add them.
+ */
+
+static int *req_serial_hfa[] = { &hf_bxxp_serial, NULL };
+static int *req_seqno_hfa[]  = { &hf_bxxp_seqno, NULL };
+static int *req_size_hfa[]   = { &hf_bxxp_size, NULL };
+static int *req_chan_hfa[]   = { &hf_bxxp_channel, &hf_bxxp_req_chan, NULL };
+static int *rsp_serial_hfa[] = { &hf_bxxp_serial, NULL };
+static int *rsp_seqno_hfa[]  = { &hf_bxxp_seqno, NULL };
+static int *rsp_size_hfa[]   = { &hf_bxxp_size, NULL };
+static int *seq_chan_hfa[]   = { &hf_bxxp_channel, &hf_bxxp_seq_chan, NULL };
+static int *seq_ackno_hfa[]  = { &hf_bxxp_ackno, NULL };
+static int *seq_window_hfa[] = { &hf_bxxp_window, NULL };
+
 static int ett_bxxp = -1;
 static int ett_mime_header = -1;
 static int ett_header = -1;
@@ -90,6 +117,9 @@ static int tcp_port = 0;
 struct bxxp_proto_data {
   int pl_left;   /* Payload at beginning of frame */
   int pl_size;   /* Payload in current message ...*/
+  int mime_hdr;  /* Whether we expect a mime header. 1 on first, 0 on rest 
+		  * in a message
+		  */
 };
 
 /*
@@ -104,6 +134,10 @@ struct bxxp_request_key {
 struct bxxp_request_val {
   guint16 processed;     /* Have we processed this conversation? */
   int size;              /* Size of the message                  */
+                         /* We need an indication in each dirn of
+			  * whether on not a mime header is expected 
+			  */
+  int c_mime_hdr, s_mime_hdr;
 };
 
 GHashTable *bxxp_request_hash = NULL;
@@ -189,30 +223,51 @@ int bxxp_get_more(char more)
   return BXXP_VIOL;
 }
 
-void
+/* dissect the more flag, and return a value of:
+ *  1 -> more
+ *  0 -> no more
+ *  -1 -> Proto violation
+ */
+
+int
 dissect_bxxp_more(tvbuff_t *tvb, int offset, frame_data *fd, 
 		  proto_tree *tree)
 {
 
+
   switch (bxxp_get_more(tvb_get_guint8(tvb, offset))) {
 
   case BXXP_COMPLETE:
 
-    proto_tree_add_boolean_hidden(tree, hf_bxxp_complete, tvb, offset, 1, TRUE);
-    proto_tree_add_text(tree, tvb, offset, 1, "More: Complete");
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_complete, tvb, offset, 1, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 1, "More: Complete");
+    }
+
+    return 0;
 
     break;
 
   case BXXP_INTERMEDIATE:
-	
-    proto_tree_add_boolean_hidden(tree, hf_bxxp_intermediate, tvb, offset, 1, TRUE);
-    proto_tree_add_text(tree, tvb, offset, 1, "More: Intermediate");
+
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_intermediate, tvb, offset, 1, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 1, "More: Intermediate");
+    }
+
+    return 1;
 
     break;
 
-  default:  /* FIXME: Add code for this case ... */
+  default:  
+
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_proto_viol, tvb, offset, 1, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 1, "PROTOCOL VIOLATION: Expected More Flag (* or .)");
+    }
+
+    return -1;
 
-    fprintf(stderr, "Error from bxxp_get_more ...\n");
     break;
   }
 
@@ -221,20 +276,26 @@ dissect_bxxp_more(tvbuff_t *tvb, int offset, frame_data *fd,
 void dissect_bxxp_status(tvbuff_t *tvb, int offset, frame_data *fd,
 			 proto_tree *tree)
 {
+
+  /* FIXME: We should return a value to indicate all OK. */
   
   switch(tvb_get_guint8(tvb, offset)) {
 
   case '+':
-  
-    proto_tree_add_boolean_hidden(tree, hf_bxxp_positive, tvb, offset, 1, TRUE);
-    proto_tree_add_text(tree, tvb, offset, 1, "Status: Positive");
+
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_positive, tvb, offset, 1, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 1, "Status: Positive");
+    }
 
     break;
 
   case '-':
 
-    proto_tree_add_boolean_hidden(tree, hf_bxxp_negative, tvb, offset, 1, TRUE);
-    proto_tree_add_text(tree, tvb, offset, 1, "Status: Negative");
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_negative, tvb, offset, 1, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 1, "Status: Negative");
+    }
 
     break;
 
@@ -256,47 +317,148 @@ int num_len(tvbuff_t *tvb, int offset)
 
 }
 
-/* Get the MIME header length */
-int header_len(tvbuff_t *tvb, int offset)
+/*
+ * We check for a terminator. This can be CRLF, which will be recorded
+ * as a terminator, or CR or LF by itself, which will be redorded as
+ * an incorrect terminator ... We build the tree at this point
+ * However, we depend on the variable bxxp_strict_term
+ */ 
+
+int 
+check_term(tvbuff_t *tvb, int offset, proto_tree *tree)
 {
-  int i = 0;
 
-  while (tvb_get_guint8(tvb, offset + i) != 0x0d 
-	 && tvb_get_guint8(tvb, offset + i + 1) != 0x0a) i++;
+  /* First, check for CRLF, or, if global_bxxp_strict_term is false, 
+   * one of CR or LF ... If neither of these hold, we add an element
+   * that complains of a protocol violation, and return -1, else
+   * we add a terminator to the tree (possibly non-standard) and return
+   * the count of characters we saw ... This may throw off the rest of the 
+   * dissection ... so-be-it!
+   */
 
-  return i;
+  if ((tvb_get_guint8(tvb, offset) == 0x0d && 
+       tvb_get_guint8(tvb, offset + 1) == 0x0a)){ /* Correct terminator */
+
+    if (tree) {
+      proto_tree_add_text(tree, tvb, offset, 2, "Terminator: CRLF");
+    }
+    return 2;
+
+  }
+  else if ((tvb_get_guint8(tvb, offset) == 0x0d) && !global_bxxp_strict_term) {
+
+    if (tree) {
+      proto_tree_add_text(tree, tvb, offset, 1, "Nonstandard Terminator: CR");
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_proto_viol, tvb, offset, 1, TRUE);
+    }
+    return 1;
+
+  }
+  else if ((tvb_get_guint8(tvb, offset) == 0x0a) && !global_bxxp_strict_term) {
+
+    if (tree) {
+      proto_tree_add_text(tree, tvb, offset, 1, "Nonstandard Terminator: LF");
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_proto_viol, tvb, offset, 1, TRUE);
+    }
+    return 1;
+
+  }
+  else {    
+
+    if (tree) {
+      proto_tree_add_text(tree, tvb, offset, 2, "PROTOCOL VIOLATION, Invalid Terminator: %s", tvb_format_text(tvb, offset, 2));
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_proto_viol, tvb, offset, 2, TRUE);
+    }
+    return -1;
+
+  }
 
 }
 
+/* Get the header length, up to CRLF or CR or LF */
+int header_len(tvbuff_t *tvb, int offset)
+{
+  int i = 0;
+  guint8 sc;
+
+  /* FIXME: Have to make sure we stop looking at the end of the tvb ... */
+
+  /* We look for CRLF, or CR or LF if global_bxxp_strict_term is 
+   * not set.
+   */
+
+  while (1) {
+
+    if (tvb_length_remaining(tvb, offset + i) < 1)
+      return i;   /* Not enough characters left ... */
+
+    if ((sc = tvb_get_guint8(tvb, offset + i)) == 0x0d 
+	&& tvb_get_guint8(tvb, offset + i + 1) == 0x0a)
+      return i;   /* Done here ... */
+
+    if (!global_bxxp_strict_term && (sc == 0x0d || sc == 0x0a))
+      return i;   /* Done here also ... */
+	  
+    i++;
+
+  }
+}
+
 int
-dissect_bxxp_mime_header(tvbuff_t *tvb, int offset, frame_data *fd,
+dissect_bxxp_mime_header(tvbuff_t *tvb, int offset, 
+			 struct bxxp_proto_data *frame_data,
 			 proto_tree *tree)
 {
-  proto_tree    *ti, *mime_tree;
-  int           mime_length = header_len(tvb, offset);
+  proto_tree    *ti = NULL, *mime_tree = NULL;
+  int           mime_length = header_len(tvb, offset), cc = 0;
 
-  ti = proto_tree_add_text(tree, tvb, offset, mime_length + 2, "Mime header: %s", tvb_format_text(tvb, offset, mime_length + 2));
-  mime_tree = proto_item_add_subtree(ti, ett_mime_header);
+  if (frame_data && !frame_data->mime_hdr) return 0;
+
+  if (tree) {
+
+    /* FIXME: Should calculate the whole length of the mime headers */
+
+    ti = proto_tree_add_text(tree, tvb, offset, mime_length, "Mime header: %s", tvb_format_text(tvb, offset, mime_length));
+    mime_tree = proto_item_add_subtree(ti, ett_mime_header);
+  }
 
   if (mime_length == 0) { /* Default header */
 
-    proto_tree_add_text(mime_tree, tvb, offset, 2, "Default values");
+    if (tree) {
+      proto_tree_add_text(mime_tree, tvb, offset, 0, "Default values");
+    }
+
+    if ((cc = check_term(tvb, offset, mime_tree)) <= 0) {
+
+      /* Ignore it, it will cause funnies in the rest of the dissect */
+
+    }
 
   }
   else {  /* FIXME: Process the headers */
 
+    if (tree) {
+      proto_tree_add_text(mime_tree, tvb, offset, mime_length, "Header: %s", 
+			  tvb_format_text(tvb, offset, mime_length));
+    }
+
+    if ((cc = check_term(tvb, offset + mime_length, mime_tree)) <= 0) {
+
+      /* Ignore it, it will cause funnies in the rest of the dissect */
+
+    }
 
   }
 
-  return mime_length + 2;  /* FIXME: Check that the CRLF is there */
+  return mime_length + cc;  /* FIXME: Check that the CRLF is there */
 
 }
 
 int
 dissect_bxxp_int(tvbuff_t *tvb, int offset, frame_data *fd,
-		    proto_tree *tree, int hf, int *val)
+		    proto_tree *tree, int hf, int *val, int *hfa[])
 {
-  int ival, i = num_len(tvb, offset);
+  int ival, ind = 0, i = num_len(tvb, offset);
   guint8 int_buff[100];
 
   memset(int_buff, '\0', sizeof(int_buff));
@@ -305,7 +467,16 @@ dissect_bxxp_int(tvbuff_t *tvb, int offset, frame_data *fd,
 
   sscanf(int_buff, "%d", &ival);  /* FIXME: Dangerous */
 
-  proto_tree_add_uint(tree, hf, tvb, offset, i, ival);
+  if (tree) {
+    proto_tree_add_uint(tree, hf, tvb, offset, i, ival);
+  }
+
+  while (hfa[ind]) {
+
+    proto_tree_add_uint_hidden(tree, *hfa[ind], tvb, offset, i, ival);
+    ind++;
+
+  }
 
   *val = ival;  /* Return the value */
 
@@ -313,37 +484,81 @@ dissect_bxxp_int(tvbuff_t *tvb, int offset, frame_data *fd,
 
 }
 
-int 
-check_crlf(tvbuff_t *tvb, int offset)
+void
+set_mime_hdr_flags(int more, struct bxxp_request_val *request_val, 
+		   struct bxxp_proto_data *frame_data)
 {
 
-  return (tvb_get_guint8(tvb, offset) == 0x0d
-	  && tvb_get_guint8(tvb, offset + 1) == 0x0a);
+  if (!request_val) return; /* Nothing to do ??? */
+
+  if (pi.destport == tcp_port) { /* Going to the server ... client */
+
+    if (request_val->c_mime_hdr) {
+
+      frame_data->mime_hdr = 0;
+
+      if (!more) request_val->c_mime_hdr = 0; 
+
+    }
+    else {
+
+      frame_data->mime_hdr = 1;
+
+      if (more) request_val->c_mime_hdr = 1;
+
+    }
+
+  }
+  else {
+
+    if (request_val->s_mime_hdr) {
+
+      frame_data->mime_hdr = 0;
+
+      if (!more) request_val->s_mime_hdr = 0; 
+
+    }
+    else {
+
+      frame_data->mime_hdr = 1;
+
+      if (more) request_val->s_mime_hdr = 1;
+
+    }
+
+  }
 
 }
 
-static int global_bxxp_tcp_port = TCP_PORT_BXXP;
-
-/* Build the tree */
+/* Build the tree
+ *
+ * A return value of <= 0 says we bailed out, skip the rest of this message,
+ * if any.
+ *
+ * A return value > 0 is the count of bytes we consumed ...
+ */
 
 int
 dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo, 
 		  proto_tree *tree, struct bxxp_request_val *request_val, 
 		  struct bxxp_proto_data *frame_data)
 {
-  proto_tree     *ti, *hdr;
-  int            st_offset, serial, seqno, size, channel, ackno, window;
+  proto_tree     *ti = NULL, *hdr = NULL;
+  int            st_offset, serial, seqno, size, channel, ackno, window, cc,
+                 more;
 
   st_offset = offset;
 
   if (tvb_strneql(tvb, offset, "REQ ", 4) == 0) {
 
-    ti = proto_tree_add_text(tree, tvb, offset, header_len(tvb, offset) + 2, "Header");
+    if (tree) {
+      ti = proto_tree_add_text(tree, tvb, offset, header_len(tvb, offset) + 2, "Header");
 
-    hdr = proto_item_add_subtree(ti, ett_header);
+      hdr = proto_item_add_subtree(ti, ett_header);
 
-    proto_tree_add_boolean_hidden(hdr, hf_bxxp_req, tvb, offset, 3, TRUE);
-    proto_tree_add_text(hdr, NullTVB, offset, 3, "Command: REQ");
+      proto_tree_add_boolean_hidden(hdr, hf_bxxp_req, tvb, offset, 3, TRUE);
+      proto_tree_add_text(hdr, NullTVB, offset, 3, "Command: REQ");
+    }
 
     offset += 3;
 
@@ -359,7 +574,34 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     /* Insert the more elements ... */
 
-    dissect_bxxp_more(tvb, offset, pinfo->fd, hdr);
+    if ((more = dissect_bxxp_more(tvb, offset, pinfo->fd, hdr)) >= 0) {
+
+      /* Figure out which direction this is in and what mime_hdr flag to 
+       * add to the frame_data. If there are missing segments, this code
+       * will get it wrong!
+       */
+
+      set_mime_hdr_flags(more, request_val, frame_data);
+
+    }
+    else {  /* Protocol violation, so dissect rest as undisectable */
+
+      if (tree) {
+
+	proto_tree_add_text(hdr, tvb, offset, 
+			    tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s",
+			    tvb_format_text(tvb, offset,
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+
+      }
+
+      return -1;
+
+    }
+
     offset += 1;
       
     /* Check the space ... */
@@ -368,25 +610,25 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     /* Dissect the serial */
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_serial, &serial);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_serial, &serial, req_serial_hfa);
     /* skip the space */
 
     offset += 1;
 
     /* now for the seqno */
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_seqno, &seqno);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_seqno, &seqno, req_seqno_hfa);
 
     /* skip the space */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_size, &size);
-    if (request_val)
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_size, &size, req_size_hfa);
+    if (request_val)   /* FIXME, is this the right order ... */
       request_val -> size = size;  /* Stash this away */
     else {
       frame_data->pl_size = size;
-      if (frame_data->pl_size < 0) frame_data->pl_size = 0;
+      if (frame_data->pl_size < 0) frame_data->pl_size = 0; /* FIXME: OK? */
     }
 
     /* Check the space */
@@ -395,24 +637,32 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     /* Get the channel */
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_channel, &channel);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_channel, &channel, req_chan_hfa);
       
-    if (check_crlf(tvb, offset)) {
+    if ((cc = check_term(tvb, offset, hdr)) <= 0) {
 
-      proto_tree_add_text(hdr, tvb, offset, 2, "Terminator: CRLF");
+      /* We dissect the rest as data and bail ... */
 
-    }
-    else {  /* Protocol violation */
+      if (tree) {
+	proto_tree_add_text(hdr, tvb, offset, 
+			    tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s", 
+			    tvb_format_text(tvb, offset, 
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+      }
 
-      /* FIXME: implement */
+      return -1;
 
     }
 
-    offset += 2;
+    offset += cc;
     
     /* Insert MIME header ... */
 
-    offset += dissect_bxxp_mime_header(tvb, offset, pinfo->fd, hdr);
+    if (frame_data && frame_data->mime_hdr)
+      offset += dissect_bxxp_mime_header(tvb, offset, frame_data, hdr);
 
     /* Now for the payload, if any */
 
@@ -427,7 +677,10 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
        * and what is the next message
        */
 
-      proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s", tvb_format_text(tvb, offset, pl_size));
+      if (tree) {
+	proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s", tvb_format_text(tvb, offset, pl_size));
+
+      }
 
       offset += pl_size;
 
@@ -441,21 +694,23 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
       }
     }
       
-    /* If anything else left, dissect it ... As what? */
+    /* If anything else left, dissect it ... */
 
     if (tvb_length_remaining(tvb, offset) > 0)
       offset += dissect_bxxp_tree(tvb, offset, pinfo, tree, request_val, frame_data);
 
   } else if (tvb_strneql(tvb, offset, "RSP ", 4) == 0) {
 
-    /* FIXME: Fix the header length */
+    if (tree) {
 
-    ti = proto_tree_add_text(tree, tvb, offset, header_len(tvb, offset) + 2, "Header");
+      ti = proto_tree_add_text(tree, tvb, offset, header_len(tvb, offset) + 2, "Header");
 
-    hdr = proto_item_add_subtree(ti, ett_header);
+      hdr = proto_item_add_subtree(ti, ett_header);
 
-    proto_tree_add_boolean_hidden(hdr, hf_bxxp_rsp, NullTVB, offset, 3, TRUE);
-    proto_tree_add_text(hdr, tvb, offset, 3, "Command: RSP");
+      proto_tree_add_boolean_hidden(hdr, hf_bxxp_rsp, NullTVB, offset, 3, TRUE);
+      proto_tree_add_text(hdr, tvb, offset, 3, "Command: RSP");
+
+    }
 
     offset += 3;
 
@@ -465,27 +720,49 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     /* Next, the 'more' flag ... */
 
-    dissect_bxxp_more(tvb, offset, pinfo->fd, hdr);
+    if ((more = dissect_bxxp_more(tvb, offset, pinfo->fd, hdr)) >= 0) {
+
+      set_mime_hdr_flags(more, request_val, frame_data);
+
+    }
+    else { 
+
+      if (tree) {
+
+	proto_tree_add_text(hdr, tvb, offset, 
+			    tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s",
+			    tvb_format_text(tvb, offset,
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+
+      }
+
+      return -1;
+
+    }
+
     offset += 1;
 
     /* Check the space */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_serial, &serial);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_serial, &serial, rsp_serial_hfa);
     /* skip the space */
 
     offset += 1;
 
     /* now for the seqno */
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_seqno, &seqno);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_seqno, &seqno, rsp_seqno_hfa);
 
     /* skip the space */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_size, &size);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, hdr, hf_bxxp_size, &size, rsp_size_hfa);
     if (request_val)
       request_val->size = size;
     else
@@ -499,22 +776,30 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
     offset += 1;
 
-    if (check_crlf(tvb, offset)) {
+    if ((cc = check_term(tvb, offset, hdr)) <= 0) {
 
-      proto_tree_add_text(hdr, tvb, offset, 2, "Terminator: CRLF");
+      /* We dissect the rest as data and bail ... */
 
-    }
-    else {  /* Protocol violation */
+      if (tree) {
+	proto_tree_add_text(hdr, tvb, offset, 
+			    tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s", 
+			    tvb_format_text(tvb, offset, 
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+      }
 
-      /* FIXME: implement */
+      return -1;
 
     }
 
-    offset += 2;
+    offset += cc;
     
     /* Insert MIME header ... */
 
-    offset += dissect_bxxp_mime_header(tvb, offset, pinfo->fd, hdr);
+    if (frame_data && frame_data->mime_hdr)
+      offset += dissect_bxxp_mime_header(tvb, offset, frame_data, hdr);
 
     /* Now for the payload, if any */
 
@@ -524,7 +809,9 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
       
       /* Except, check the payload length, and only dissect that much */
 
-      proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s", tvb_format_text(tvb, offset, pl_size));
+      if (tree) {
+	proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s", tvb_format_text(tvb, offset, pl_size));
+      }
 
       offset += pl_size;
 
@@ -545,67 +832,85 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
   } else if (tvb_strneql(tvb, offset, "SEQ ", 4) == 0) {
 
-    proto_tree_add_boolean_hidden(tree, hf_bxxp_seq, NullTVB, offset, 3, TRUE);
-    proto_tree_add_text(tree, tvb, offset, 3, "Command: SEQ");
-      
+    if (tree) {
+      proto_tree_add_boolean_hidden(tree, hf_bxxp_seq, NullTVB, offset, 3, TRUE);
+      proto_tree_add_text(tree, tvb, offset, 3, "Command: SEQ");
+    }
+
     offset += 3;
 
     /* Now check the space: FIXME */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_channel, &channel);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_channel, &channel, seq_chan_hfa);
 
     /* Check the space: FIXME */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_ackno, &ackno);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_ackno, &ackno, seq_ackno_hfa);
 
     /* Check the space: FIXME */
 
     offset += 1;
 
-    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_window, &window);
+    offset += dissect_bxxp_int(tvb, offset, pinfo->fd, tree, hf_bxxp_window, &window, seq_window_hfa);
 
-    if (check_crlf(tvb, offset)) {
+    if ((cc = check_term(tvb, offset, tree)) <= 0) {
 
-      proto_tree_add_text(tree, tvb, offset, 2, "Terminator: CRLF");
+      /* We dissect the rest as data and bail ... */
 
-    }
-    else {  /* Protocol violation */
+      if (tree) {
+	proto_tree_add_text(tree, tvb, offset, 
+			    tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s", 
+			    tvb_format_text(tvb, offset, 
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+      }
 
-      /* FIXME: implement */
+      return -1;
 
     }
 
-    offset += 2;
+    offset += cc;
 
   } else if (tvb_strneql(tvb, offset, "END", 3) == 0) {
 
-    proto_tree *tr;
+    proto_tree *tr = NULL;
 
-    ti = proto_tree_add_text(tree, tvb, offset, MIN(5, tvb_length_remaining(tvb, offset)), "Trailer");
+    if (tree) {
+      ti = proto_tree_add_text(tree, tvb, offset, MIN(5, tvb_length_remaining(tvb, offset)), "Trailer");
 
-    tr = proto_item_add_subtree(ti, ett_trailer);
+      tr = proto_item_add_subtree(ti, ett_trailer);
 
-    proto_tree_add_boolean_hidden(tr, hf_bxxp_end, NullTVB, offset, 3, TRUE);
-    proto_tree_add_text(tr, tvb, offset, 3, "Command: END");
+      proto_tree_add_boolean_hidden(tr, hf_bxxp_end, NullTVB, offset, 3, TRUE);
+      proto_tree_add_text(tr, tvb, offset, 3, "Command: END");
+
+    }
 
     offset += 3;
 
-    if (check_crlf(tvb, offset)) {
+    if ((cc = check_term(tvb, offset, tr)) <= 0) {
 
-      proto_tree_add_text(tr, tvb, offset, 2, "Terminator: CRLF");
+      /* We dissect the rest as data and bail ... */
 
-    }
-    else {  /* Protocol violation */
+      if (tree) { 
+	proto_tree_add_text(tr, tvb, offset, tvb_length_remaining(tvb, offset),
+			    "Undissected Payload: %s", 
+			    tvb_format_text(tvb, offset, 
+					    tvb_length_remaining(tvb, offset)
+					    )
+			    );
+      }
 
-      /* FIXME: implement */
+      return -1;
 
     }
 
-    offset += 2;
+    offset += cc;
 
   }
 
@@ -617,7 +922,9 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
       pl_size = MIN(request_val->size, tvb_length_remaining(tvb, offset));
 
-      if (pl_size == 0) { /* The whole of the rest must be payload */
+      /* FIXME: May be redundent ... */
+
+      if (pl_size == 0 && offset == st_offset) { /* The whole of the rest must be payload */
       
 	pl_size = tvb_length_remaining(tvb, offset); /* Right place ? */
       
@@ -633,18 +940,20 @@ dissect_bxxp_tree(tvbuff_t *tvb, int offset, packet_info *pinfo,
      * another message here, then handle it correctly as well.
      */
 
-    /* If the pl_size == 0 and the offset == 0, then we have not processed
-     * anything in this frame above, so we better treat all this data as 
-     * payload to avoid recursion loops
+    /* If the pl_size == 0 and the offset == st_offset, then we have not 
+     * processed anything in this frame above, so we better treat all this
+     * data as payload to avoid recursion loops
      */
 
-    if (pl_size == 0 && offset == 0)
+    if (pl_size == 0 && offset == st_offset) 
       pl_size = tvb_length_remaining(tvb, offset);
 
     if (pl_size > 0) {
 
-      proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s",
-			  tvb_format_text(tvb, offset, pl_size));
+      if (tree) {
+	proto_tree_add_text(tree, tvb, offset, pl_size, "Payload: %s",
+			    tvb_format_text(tvb, offset, pl_size));
+      }
 
       offset += pl_size;            /* Advance past the payload */
 
@@ -672,7 +981,7 @@ dissect_bxxp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
   int offset;
   struct bxxp_proto_data  *frame_data = NULL;
-  proto_tree              *bxxp_tree, *ti;
+  proto_tree              *bxxp_tree = NULL, *ti = NULL;
   conversation_t          *conversation = NULL;
   struct bxxp_request_key request_key, *new_request_key;
   struct bxxp_request_val *request_val = NULL;
@@ -708,10 +1017,11 @@ dissect_bxxp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (!frame_data) {
 
     conversation = find_conversation(&pinfo->src, &pinfo->dst, pi.ptype,
-				       pinfo->srcport, pinfo->destport);
+				       pinfo->srcport, pinfo->destport, 0);
     if (conversation == NULL) { /* No conversation, create one */
 	conversation = conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype,
-					pinfo->srcport, pinfo->destport, NULL);
+					pinfo->srcport, pinfo->destport, NULL,
+					0);
 
       }
 
@@ -737,7 +1047,7 @@ dissect_bxxp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
   if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_add_str(pinfo->fd, COL_PROTOCOL, "BXXP");
+    col_set_str(pinfo->fd, COL_PROTOCOL, "BXXP");
 
   if (check_col(pinfo->fd, COL_INFO)) {  /* Check the type ... */
 
@@ -745,68 +1055,88 @@ dissect_bxxp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   }
 
-  if (tree) {  /* Build the tree info ... */
+  /* Here, we parse the message so we can retrieve the info we need, which 
+   * is that there is some payload left from a previous segment on the 
+   * front of this segment ... This all depends on TCP segments not getting
+   * out of order ... 
+   *
+   * As a huge kludge, we push the checking for the tree down into the code
+   * and process as if we were given a tree but not call the routines that
+   * adorn the protocol tree if they were NULL.
+   */
 
-    /* Check the per-frame data and the conversation for any left-over 
-     * payload from the previous frame 
-     */
+  if (tree) {  /* Build the tree info ... */
 
     ti = proto_tree_add_item(tree, proto_bxxp, tvb, offset, tvb_length(tvb), FALSE);
 
     bxxp_tree = proto_item_add_subtree(ti, ett_bxxp);
 
-    /* FIXME: This conditional is not correct */
-    if ((frame_data && frame_data->pl_left > 0) ||
-	(request_val && request_val->size > 0)) {
-      int pl_left = 0;
+  }
+  
+  /* Check the per-frame data and the conversation for any left-over 
+   * payload from the previous frame 
+   *
+   * We check that per-frame data exists first, and if so, use it,
+   * else we use the conversation data.
+   *
+   * We create per-frame data here as well, but we must ensure we create it
+   * after we have done the check for per-frame or conversation data.
+   *
+   * We also depend on the first frame in a group having a pl_size of 0.
+   */
 
-      if (frame_data) {
-	pl_left = frame_data->pl_left;
-      }
-      else {
-	pl_left = request_val->size;
+  if (frame_data && frame_data->pl_left > 0) {
 
-	request_val->size = 0;
+    int pl_left = frame_data->pl_left;
 
-	/* We create the frame data here for this case, and 
-	 * elsewhere for other frames
-	 */
+    pl_left = MIN(pl_left, tvb_length_remaining(tvb, offset));
 
-	frame_data = g_mem_chunk_alloc(bxxp_packet_infos);
-
-	frame_data->pl_left = pl_left;
-	frame_data->pl_size = 0;
-
-	p_add_proto_data(pinfo->fd, proto_bxxp, frame_data);
-
-      }
-
-      pl_left = MIN(pl_left, tvb_length_remaining(tvb, offset));
-
-      /* Add the payload bit */
+    /* Add the payload bit, only if we have a tree */
+    if (tree) {
       proto_tree_add_text(bxxp_tree, tvb, offset, pl_left, "Payload: %s",
 			  tvb_format_text(tvb, offset, pl_left));
-      offset += pl_left;
     }
+    offset += pl_left;
+  }
+  else if (request_val && request_val->size > 0) {
 
-    if (tvb_length_remaining(tvb, offset) > 0) {
+    int pl_left = request_val->size;
 
-      offset += dissect_bxxp_tree(tvb, offset, pinfo, bxxp_tree, request_val, frame_data);
+    request_val->size = 0;
 
-    }
+    /* We create the frame data here for this case, and 
+     * elsewhere for other frames
+     */
 
-    /* Set up the per-frame data here if not already done so */
+    frame_data = g_mem_chunk_alloc(bxxp_packet_infos);
 
-    if (frame_data == NULL) { 
+    frame_data->pl_left = pl_left;
+    frame_data->pl_size = 0;
+    frame_data->mime_hdr = 0;
+      
+    p_add_proto_data(pinfo->fd, proto_bxxp, frame_data);
 
-      frame_data = g_mem_chunk_alloc(bxxp_packet_infos);
+  }
 
-      frame_data->pl_left = 0;
-      frame_data->pl_size = 0;
+  /* Set up the per-frame data here if not already done so
+   * This _must_ come after the checks above ...
+   */
 
-      p_add_proto_data(pinfo->fd, proto_bxxp, frame_data);
+  if (frame_data == NULL) { 
+
+    frame_data = g_mem_chunk_alloc(bxxp_packet_infos);
+
+    frame_data->pl_left = 0;
+    frame_data->pl_size = 0;
+    frame_data->mime_hdr = 0;
+
+    p_add_proto_data(pinfo->fd, proto_bxxp, frame_data);
 	
-    }
+  }
+
+  if (tvb_length_remaining(tvb, offset) > 0) {
+
+    offset += dissect_bxxp_tree(tvb, offset, pinfo, bxxp_tree, request_val, frame_data);
 
   }
 
@@ -818,15 +1148,27 @@ void
 proto_register_bxxp(void)
 {
   static hf_register_info hf[] = {
+    { &hf_bxxp_proto_viol,
+      { "Protocol Violation", "bxxp.violation", FT_BOOLEAN, BASE_NONE, NULL, 0x0, "" }},
+
     { &hf_bxxp_req,
       { "Request", "bxxp.req", FT_BOOLEAN, BASE_NONE, NULL, 0x0, "" }},
 
+    { &hf_bxxp_req_chan,
+      { "Request Channel Number", "bxxp.req.channel", FT_UINT32, BASE_DEC, NULL, 0x0, ""}},
+
     { &hf_bxxp_rsp,
       { "Response", "bxxp.rsp", FT_BOOLEAN, BASE_NONE, NULL, 0x0, "" }},
 
+    { &hf_bxxp_rsp_chan,
+      { "Response Channel Number", "bxxp.rsp.channel", FT_UINT32, BASE_DEC, NULL, 0x0, ""}},
+
     { &hf_bxxp_seq,
       { "Sequence", "bxxp.seq", FT_BOOLEAN, BASE_NONE, NULL, 0x0, "" }},
 
+    { &hf_bxxp_seq_chan,
+      { "Sequence Channel Number", "bxxp.seq.channel", FT_UINT32, BASE_DEC, NULL, 0x0, ""}},
+
     { &hf_bxxp_end,
       { "End", "bxxp.end", FT_BOOLEAN, BASE_NONE, NULL, 0x0, "" }},
 
@@ -869,7 +1211,7 @@ proto_register_bxxp(void)
   };
   module_t *bxxp_module; 
 
-  /* Register our configuration options for BXXP, particularly out port */
+  /* Register our configuration options for BXXP, particularly our port */
 
   bxxp_module = prefs_register_module("bxxp", "BXXP", proto_reg_handoff_bxxp);
 
@@ -878,6 +1220,12 @@ proto_register_bxxp(void)
 				 " than the default of 10288)",
 				 10, &global_bxxp_tcp_port);
 
+  prefs_register_bool_preference(bxxp_module, "strict_header_terminator", 
+				 "BXXP Header Requires CRLF", 
+				 "Specifies that BXXP requires CRLF as a "
+				 "terminator, and not just CR or LF",
+				 &global_bxxp_strict_term);
+
   proto_bxxp = proto_register_protocol("Blocks eXtensible eXchange Protocol",
 				       "bxxp");
 
