@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c,v 1.44 2000/08/13 14:08:10 deniel Exp $
+ * $Id: packet-eth.c,v 1.49 2000/11/19 08:53:57 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -147,28 +147,23 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   const guint8		*pd;
 
   volatile guint16	etype;
-  volatile int		ethhdr_type;	/* the type of ethernet frame */
+  volatile int		ethhdr_type;	/* the type of Ethernet frame */
   volatile int		eth_offset;
   volatile guint16  	length;
-
-  /* These are static because gcc says that they might get clobbered
-   * otherwise. */
-  static tvbuff_t	*next_tvb = NULL;
-  static tvbuff_t	*trailer_tvb;
-  static proto_tree	*fh_tree;
+  tvbuff_t		*volatile next_tvb;
+  tvbuff_t		*volatile trailer_tvb;
+  proto_tree		*volatile fh_tree;
+  guint			length_before;
 
   CHECK_DISPLAY_AS_DATA(proto_eth, tvb, pinfo, tree);
 
   tvb_compat(tvb, &pd, (int*)&eth_offset);
 
-  /* Reset this static variable to NULL since I test it's value later */
-  trailer_tvb = NULL;
-
   pinfo->current_proto = "Ethernet";
   orig_captured_len = pinfo->captured_len;
 
   if (check_col(pinfo->fd, COL_PROTOCOL))
-    col_add_str(pinfo->fd, COL_PROTOCOL, "Ethernet");
+    col_set_str(pinfo->fd, COL_PROTOCOL, "Ethernet");
 
   src = tvb_get_ptr(tvb, 6, 6);
   dst = tvb_get_ptr(tvb, 0, 6);
@@ -241,14 +236,14 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        and set the payload and captured-payload lengths to the minima
        of the total length and the frame lengths. */
     length += eth_offset + ETH_HEADER_SIZE;
-    if (pi.len > length)
-      pi.len = length;
-    if (pi.captured_len > length)
-      pi.captured_len = length;
+    if (pinfo->len > length)
+      pinfo->len = length;
+    if (pinfo->captured_len > length)
+      pinfo->captured_len = length;
   } else {
     ethhdr_type = ETHERNET_II;
     if (check_col(pinfo->fd, COL_INFO))
-      col_add_str(pinfo->fd, COL_INFO, "Ethernet II");
+      col_set_str(pinfo->fd, COL_INFO, "Ethernet II");
     if (tree) {
 
 	ti = proto_tree_add_protocol_format(tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
@@ -265,19 +260,38 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   }
   eth_offset += ETH_HEADER_SIZE;
 
-  /* Give the next dissector only 'length' number of bytes */
   if (etype <= IEEE_802_3_MAX_LEN) {
+	  /* Give the next dissector only 'length' number of bytes */
 	  TRY {
-	     next_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE, etype, etype);
-	     trailer_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE + etype, -1, -1);
+	    next_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE, etype, etype);
+	    trailer_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE + etype, -1, -1);
 	  }
 	  CATCH2(BoundsError, ReportedBoundsError) {
-	     next_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE, -1, etype);
+	    /* Either:
+
+		 the packet doesn't have "etype" bytes worth of
+		 captured data left in it - or it may not even have
+		 "etype" bytes worth of data in it, period -
+		 so the "tvb_new_subset()" creating "next_tvb"
+		 threw an exception
+
+	       or
+
+		 the packet has exactly "etype" bytes worth of
+		 captured data left in it, so the "tvb_new_subset()"
+		 creating "trailer_tvb" threw an exception.
+
+	      In either case, this means that all the data in the frame
+	      is within the length value, so we give all the data to the
+	      next protocol and have no trailer. */
+	    next_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE, -1, etype);
+	    trailer_tvb = NULL;
 	  }
 	  ENDTRY;
   }
   else {
-     next_tvb = tvb_new_subset(tvb, ETH_HEADER_SIZE, -1, -1);
+     next_tvb = NULL;	/* "ethertype()" will create the next tvb for us */
+     trailer_tvb = NULL;	/* we don't know how big the trailer is */
   }
 
   switch (ethhdr_type) {
@@ -288,20 +302,34 @@ dissect_eth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       dissect_llc(next_tvb, pinfo, tree);
       break;
     case ETHERNET_II:
-      ethertype(etype, tvb, ETH_HEADER_SIZE, pinfo, tree, fh_tree, hf_eth_type);
+      length_before = tvb_reported_length(tvb);
+      length = ethertype(etype, tvb, ETH_HEADER_SIZE, pinfo, tree, fh_tree,
+          hf_eth_type) + ETH_HEADER_SIZE;
+      if (length < length_before) {
+	/*
+	 * Create a tvbuff for the padding.
+	 */
+	TRY {
+	  trailer_tvb = tvb_new_subset(tvb, length, -1, -1);
+	}
+	CATCH2(BoundsError, ReportedBoundsError) {
+	  /* The packet doesn't have "length" bytes worth of captured
+	     data left in it.  No trailer to display. */
+	  trailer_tvb = NULL;
+	}
+	ENDTRY;
+      }
       break;
   }
 
   /* If there's some bytes left over, mark them. */
   if (trailer_tvb && tree) {
-	  int             trailer_length;
-	  const guint8    *ptr;
+	  guint           trailer_length;
 
 	  trailer_length = tvb_length(trailer_tvb);
-	  if (trailer_length > 0) {
-		  ptr = tvb_get_ptr(trailer_tvb, 0, trailer_length);
-		  proto_tree_add_bytes(fh_tree, hf_eth_trailer, tvb, ETH_HEADER_SIZE + etype,
-			  trailer_length, ptr);
+	  if (trailer_length != 0) {
+		  proto_tree_add_item(fh_tree, hf_eth_trailer, trailer_tvb, 0,
+			  trailer_length, FALSE);
 	  }
   }
 
@@ -345,4 +373,6 @@ proto_register_eth(void)
 	proto_eth = proto_register_protocol ("Ethernet", "eth" );
 	proto_register_field_array(proto_eth, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+
+	register_dissector("eth", dissect_eth);
 }
