@@ -2,7 +2,7 @@
  * Routines for Sinec H1 packet disassembly
  * Gerrit Gehnen <G.Gehnen@atrie.de>
  *
- * $Id: packet-h1.c,v 1.12 2000/08/13 14:08:11 deniel Exp $
+ * $Id: packet-h1.c,v 1.14 2000/11/19 08:53:57 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -122,7 +122,7 @@ static gboolean dissect_h1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
       if (check_col (pinfo->fd, COL_PROTOCOL))
-	col_add_str (pinfo->fd, COL_PROTOCOL, "H1");
+	col_set_str (pinfo->fd, COL_PROTOCOL, "H1");
       if (check_col (pinfo->fd, COL_INFO))
 	col_add_str (pinfo->fd, COL_INFO, "S5: ");
       if (tree)
@@ -242,7 +242,7 @@ static gboolean dissect_h1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  position += tvb_get_guint8(tvb,offset + position + 1);	/* Goto next section */
 	}			/* ..while */
     next_tvb = tvb_new_subset(tvb, offset+tvb_get_guint8(tvb,offset+2), -1, -1);
-    dissect_data(next_tvb, pinfo, tree);
+    dissect_data(next_tvb, 0, pinfo, tree);
 
     return TRUE;
 }
