@@ -1,7 +1,7 @@
 /* packet-icq.c
  * Routines for ICQ packet disassembly
  *
- * $Id: packet-icq.c,v 1.21 2000/08/13 14:08:15 deniel Exp $
+ * $Id: packet-icq.c,v 1.23 2000/11/19 19:23:54 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Johan Feyaerts
@@ -561,10 +561,10 @@ dissect_icqv4(const u_char *pd,
 {
     /* Not really implemented yet */
     if (check_col(fd, COL_PROTOCOL)) {
-	col_add_str(fd, COL_PROTOCOL, "ICQv4 (UDP)");
+	col_set_str(fd, COL_PROTOCOL, "ICQv4 (UDP)");
     }
     if (check_col(fd, COL_INFO)) {
-	col_add_str(fd, COL_INFO, "ICQ Version 4 protocol");
+	col_set_str(fd, COL_INFO, "ICQ Version 4 protocol");
     }
 }
 
@@ -576,10 +576,10 @@ dissect_icqv3(const u_char *pd,
 {
     /* Not really implemented yet */
     if (check_col(fd, COL_PROTOCOL)) {
-	col_add_str(fd, COL_PROTOCOL, "ICQv3 (UDP)");
+	col_set_str(fd, COL_PROTOCOL, "ICQv3 (UDP)");
     }
     if (check_col(fd, COL_INFO)) {
-	col_add_str(fd, COL_INFO, "ICQ Version 3 protocol");
+	col_set_str(fd, COL_INFO, "ICQ Version 3 protocol");
     }
 }
 
@@ -591,10 +591,10 @@ dissect_icqv2(const u_char *pd,
 {
     /* Not really implemented yet */
     if (check_col(fd, COL_PROTOCOL)) {
-	col_add_str(fd, COL_PROTOCOL, "ICQv2 (UDP)");
+	col_set_str(fd, COL_PROTOCOL, "ICQv2 (UDP)");
     }
     if (check_col(fd, COL_INFO)) {
-	col_add_str(fd, COL_INFO, "ICQ Version 2 protocol");
+	col_set_str(fd, COL_INFO, "ICQ Version 2 protocol");
     }
 }
 
@@ -2051,10 +2051,17 @@ dissect_icqv5Client(const u_char *pd,
     guint16 seqnum1 = 0 , seqnum2 = 0;
     guint32 uin = -1, sessionid = -1;
     guint32 key = -1;
-    guint16 pktsize = -1;	/* The size of the ICQ content */
-    u_char decr_pd[1600];	/* Decrypted content, size should be dynamic */
+    guint16 pktsize = -1;		/* The size of the ICQ content */
+    static u_char *decr_pd = NULL;	/* Decrypted content */
     
     pktsize = END_OF_FRAME;
+
+    if (decr_pd == NULL)
+	decr_pd = (u_char *) g_malloc(sizeof (u_char) * 128);
+    
+    while (sizeof(decr_pd) < pktsize + 3)
+	decr_pd = (u_char *) g_realloc(decr_pd, sizeof (decr_pd) * 2);
+    
     /* First copy the memory, we don't want to overwrite the old content */
     memcpy(decr_pd, &pd[offset], pktsize);
     if (pktsize>0x14) {
@@ -2410,9 +2417,9 @@ static void dissect_icqv5(const u_char *pd,
   guint32 unknown = pletohl(&pd[offset + ICQ5_UNKNOWN]);
   
   if (check_col(fd, COL_PROTOCOL))
-      col_add_str(fd, COL_PROTOCOL, "ICQv5 (UDP)");
+      col_set_str(fd, COL_PROTOCOL, "ICQv5 (UDP)");
   if (check_col(fd, COL_INFO))
-      col_add_str(fd, COL_INFO, "ICQv5 packet");
+      col_set_str(fd, COL_INFO, "ICQv5 packet");
   if (unknown == 0x0L) {
       dissect_icqv5Client(pd, offset, fd, tree);
   } else {
