@@ -1,7 +1,7 @@
 /* packet-ipv6.h
  * Definitions for IPv6 packet disassembly 
  *
- * $Id: packet-ipv6.h,v 1.13 2000/08/18 12:05:27 itojun Exp $
+ * $Id: packet-ipv6.h,v 1.16 2000/11/11 10:23:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -94,13 +94,8 @@ struct ip6_hdr {
 #define	IP6H_SRC	8
 #define	IP6H_DST	24
 
-#ifdef WORDS_BIGENDIAN
 #define IPV6_FLOWINFO_MASK	0x0fffffff	/* flow info (28 bits) */
 #define IPV6_FLOWLABEL_MASK	0x000fffff	/* flow label (20 bits) */
-#else
-#define IPV6_FLOWINFO_MASK	0xffffff0f	/* flow info (28 bits) */
-#define IPV6_FLOWLABEL_MASK	0xffff0f00	/* flow label (20 bits) */
-#endif
 
 /*
  * Extension Headers
@@ -174,15 +169,9 @@ struct ip6_frag {
 	guint32 ip6f_ident;		/* identification */
 };
 
-#if BYTE_ORDER == BIG_ENDIAN
 #define IP6F_OFF_MASK		0xfff8	/* mask out offset from _offlg */
 #define IP6F_RESERVED_MASK	0x0006	/* reserved bits in ip6f_offlg */
 #define IP6F_MORE_FRAG		0x0001	/* more-fragments flag */
-#else /* BYTE_ORDER == LITTLE_ENDIAN */
-#define IP6F_OFF_MASK		0xf8ff	/* mask out offset from _offlg */
-#define IP6F_RESERVED_MASK	0x0600	/* reserved bits in ip6f_offlg */
-#define IP6F_MORE_FRAG		0x0100	/* more-fragments flag */
-#endif /* BYTE_ORDER == LITTLE_ENDIAN */
 
 /*
  * Definition for ICMPv6.
@@ -340,15 +329,9 @@ struct nd_neighbor_advert {	/* neighbor advertisement */
 #define nd_na_code		nd_na_hdr.icmp6_code
 #define nd_na_cksum		nd_na_hdr.icmp6_cksum
 #define nd_na_flags_reserved	nd_na_hdr.icmp6_data32[0]
-#if BYTE_ORDER == BIG_ENDIAN
 #define ND_NA_FLAG_ROUTER		0x80000000
 #define ND_NA_FLAG_SOLICITED		0x40000000
 #define ND_NA_FLAG_OVERRIDE		0x20000000
-#elif BYTE_ORDER == LITTLE_ENDIAN
-#define ND_NA_FLAG_ROUTER		0x80
-#define ND_NA_FLAG_SOLICITED		0x40
-#define ND_NA_FLAG_OVERRIDE		0x20
-#endif
 
 struct nd_redirect {		/* redirect */
 	struct icmp6_hdr	nd_rd_hdr;
@@ -425,15 +408,9 @@ struct icmp6_nodeinfo {
 #define NI_QTYPE_NODEADDR	3 /* Node Addresses */
 #define NI_QTYPE_IPV4ADDR	4 /* IPv4 Addresses */
 
-#if BYTE_ORDER == BIG_ENDIAN
 #define NI_SUPTYPE_FLAG_COMPRESS	0x1
 #define NI_FQDN_FLAG_VALIDTTL		0x1
-#elif BYTE_ORDER == LITTLE_ENDIAN
-#define NI_SUPTYPE_FLAG_COMPRESS	0x0100
-#define NI_FQDN_FLAG_VALIDTTL		0x0100
-#endif
 
-#if BYTE_ORDER == BIG_ENDIAN
 #define NI_NODEADDR_FLAG_TRUNCATE	0x1
 #define NI_NODEADDR_FLAG_ALL		0x2
 #define NI_NODEADDR_FLAG_COMPAT		0x4
@@ -441,15 +418,6 @@ struct icmp6_nodeinfo {
 #define NI_NODEADDR_FLAG_SITELOCAL	0x10
 #define NI_NODEADDR_FLAG_GLOBAL		0x20
 #define NI_NODEADDR_FLAG_ANYCAST	0x40 /* just experimental. not in spec */
-#elif BYTE_ORDER == LITTLE_ENDIAN
-#define NI_NODEADDR_FLAG_TRUNCATE	0x0100
-#define NI_NODEADDR_FLAG_ALL		0x0200
-#define NI_NODEADDR_FLAG_COMPAT		0x0400
-#define NI_NODEADDR_FLAG_LINKLOCAL	0x0800
-#define NI_NODEADDR_FLAG_SITELOCAL	0x1000
-#define NI_NODEADDR_FLAG_GLOBAL		0x2000
-#define NI_NODEADDR_FLAG_ANYCAST	0x4000 /* just experimental. not in spec */
-#endif
 
 struct ni_reply_fqdn {
 	guint32	ni_fqdn_ttl;	/* TTL */
@@ -460,33 +428,13 @@ struct ni_reply_fqdn {
 /*
  * Router Renumbering. as router-renum-05.txt
  */
-#if BYTE_ORDER == BIG_ENDIAN /* net byte order */
 struct icmp6_router_renum {	/* router renumbering header */
 	struct icmp6_hdr	rr_hdr;
 	guint8		rr_segnum;
-	guint32		rr_test : 1;
-	guint32		rr_reqresult : 1;
-	guint32		rr_forceapply : 1;
-	guint32		rr_specsite : 1;
-	guint32		rr_prevdone : 1;
-	guint32		rr_flags_reserved : 3;
+	guint8		rr_flags;
 	guint16		rr_maxdelay;
 	guint32		rr_reserved;
 };
-#elif BYTE_ORDER == LITTLE_ENDIAN
-struct icmp6_router_renum {	/* router renumbering header */
-	struct icmp6_hdr	rr_hdr;
-	guint8		rr_segnum;
-	guint32		rr_flags_reserved : 3;
-	guint32		rr_prevdone : 1;
-	guint32		rr_specsite : 1;
-	guint32		rr_forceapply : 1;
-	guint32		rr_reqresult : 1;
-	guint32		rr_test : 1;
-	guint16		rr_maxdelay;
-	guint32		rr_reserved;
-};
-#endif /* BYTE_ORDER */
 
 #define rr_type			rr_hdr.icmp6_type
 #define rr_code			rr_hdr.icmp6_code
@@ -509,66 +457,34 @@ struct rr_pco_match {		/* match prefix part */
 #define RPM_PCO_SETGLOBAL	3
 #define RPM_PCO_MAX		4
 
-#if BYTE_ORDER == BIG_ENDIAN /* net byte order */
 struct rr_pco_use {		/* use prefix part */
 	guint8	rpu_uselen;
 	guint8	rpu_keeplen;
-	guint32	rpu_mask_onlink : 1;
-	guint32	rpu_mask_autonomous : 1;
-	guint32	rpu_mask_reserved : 6;
-	guint32	rpu_onlink : 1;
-	guint32	rpu_autonomous : 1;
-	guint32	rpu_raflags_reserved : 6;
+	guint8	rpu_ramask;
+	guint8	rpu_raflags;
 	guint32	rpu_vltime;
 	guint32	rpu_pltime;
-	guint32	rpu_decr_vltime : 1;
-	guint32	rpu_decr_pltime : 1;
-	guint32	rpu_flags_reserved : 6;
-	guint32	rpu_reserved : 24;
+	guint32	rpu_flags;
 	struct e_in6_addr rpu_prefix;
 };
-#elif BYTE_ORDER == LITTLE_ENDIAN
-struct rr_pco_use {		/* use prefix part */
-	guint8	rpu_uselen;
-	guint8	rpu_keeplen;
-	guint32	rpu_mask_reserved : 6;
-	guint32	rpu_mask_autonomous : 1;
-	guint32	rpu_mask_onlink : 1;
-	guint32	rpu_raflags_reserved : 6;
-	guint32	rpu_autonomous : 1;
-	guint32	rpu_onlink : 1;
-	guint32	rpu_vltime;
-	guint32	rpu_pltime;
-	guint32	rpu_flags_reserved : 6;
-	guint32	rpu_decr_pltime : 1;
-	guint32	rpu_decr_vltime : 1;
-	guint32	rpu_reserved : 24;
-	struct e_in6_addr rpu_prefix;
-};
-#endif /* BYTE_ORDER */
 
-#if BYTE_ORDER == BIG_ENDIAN /* net byte order */
+#define ICMP6_RR_PCOUSE_RAFLAGS_ONLINK  0x80
+#define ICMP6_RR_PCOUSE_RAFLAGS_AUTO    0x40
+
+/* network endian */
+#define ICMP6_RR_PCOUSE_FLAGS_DECRVLTIME	0x80000000
+#define ICMP6_RR_PCOUSE_FLAGS_DECRPLTIME	0x40000000
+
 struct rr_result {		/* router renumbering result message */
-	guint8	rrr_reserved;
-	guint32	rrr_flags_reserved : 6;
-	guint32	rrr_outofbound : 1;
-	guint32	rrr_forbidden : 1;
+	guint16	rrr_flags;
 	guint8	rrr_ordinal;
 	guint8	rrr_matchedlen;
 	guint32	rrr_ifid;
 	struct e_in6_addr rrr_prefix;
 };
-#elif BYTE_ORDER == LITTLE_ENDIAN
-struct rr_result {		/* router renumbering result message */
-	guint8	rrr_reserved;
-	guint32	rrr_forbidden : 1;
-	guint32	rrr_outofbound : 1;
-	guint32	rrr_flags_reserved : 6;
-	guint8	rrr_ordinal;
-	guint8	rrr_matchedlen;
-	guint32	rrr_ifid;
-	struct e_in6_addr rrr_prefix;
-};
-#endif /* BYTE_ORDER */
+
+/* network endian */
+#define ICMP6_RR_RESULT_FLAGS_OOB		0x0002
+#define ICMP6_RR_RESULT_FLAGS_FORBIDDEN		0x0001
 
 #endif /* __PACKET_IPV6_H_DEFINED__ */
