@@ -2,7 +2,7 @@
  * Routines for NetWare's IPX
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-ipx.c,v 1.64 2000/08/13 14:08:18 deniel Exp $
+ * $Id: packet-ipx.c,v 1.70 2000/11/19 08:53:58 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -33,10 +33,12 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <glib.h>
 #include "etypes.h"
 #include "ppptypes.h"
 #include "llcsaps.h"
+#include "aftypes.h"
 #include "packet.h"
 #include "packet-ipx.h"
 #include "packet-nbipx.h"
@@ -319,7 +321,7 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	SET_ADDRESS(&pi.dst,		AT_IPX, 10, dst_net_node);
 
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, "IPX");
+		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX");
 	if (check_col(pinfo->fd, COL_INFO))
 		col_add_fstr(pinfo->fd, COL_INFO, "%s (0x%04X)",
 				socket_text(ipx_dsocket), ipx_dsocket);
@@ -369,7 +371,7 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		case IPX_PACKET_TYPE_WANBCAST:
 		case IPX_PACKET_TYPE_PEP:
 			if (ipx_dsocket == IPX_SOCKET_NETBIOS) {
-				dissect_nbipx(next_pd, next_offset, pinfo->fd, tree);
+				dissect_nbipx(next_tvb, pinfo, tree);
 				return;
 			}
 			/* else fall through */
@@ -386,7 +388,7 @@ dissect_ipx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (dissector_try_port(ipx_socket_dissector_table, ipx_ssocket,
 	    next_tvb, pinfo, tree))
 		return;
-	dissect_data(next_tvb, pinfo, tree);
+	dissect_data(next_tvb, 0, pinfo, tree);
 }
 
 
@@ -445,9 +447,9 @@ dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	pinfo->current_proto = "SPX";
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, "SPX");
+		col_set_str(pinfo->fd, COL_PROTOCOL, "SPX");
 	if (check_col(pinfo->fd, COL_INFO))
-		col_add_str(pinfo->fd, COL_INFO, "SPX");
+		col_set_str(pinfo->fd, COL_INFO, "SPX");
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_spx, tvb, 0, SPX_HEADER_LEN, FALSE);
@@ -472,7 +474,7 @@ dissect_spx(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		proto_tree_add_item(spx_tree, hf_spx_all_nr, tvb, 10, 2, FALSE);
 
 		next_tvb = tvb_new_subset(tvb, SPX_HEADER_LEN, -1, -1);
-		dissect_data(next_tvb, pinfo, tree);
+		dissect_data(next_tvb, 0, pinfo, tree);
 	}
 }
 
@@ -491,12 +493,12 @@ dissect_ipxmsg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	pinfo->current_proto = "IPX MSG";
 
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-	 col_add_str(pinfo->fd, COL_PROTOCOL, "IPX MSG");
+	 col_set_str(pinfo->fd, COL_PROTOCOL, "IPX MSG");
 
 	conn_number = tvb_get_guint8(tvb, 0);
 	sig_char = tvb_get_guint8(tvb, 1);
 
-	if (check_col(pinfo->fd, COL_PROTOCOL)) {
+	if (check_col(pinfo->fd, COL_INFO)) {
 		col_add_fstr(pinfo->fd, COL_INFO, 
 			"%s, Connection %d", 
 			val_to_str(sig_char, ipxmsg_sigchar_vals, "Unknown Signature Char"), conn_number);
@@ -531,11 +533,11 @@ dissect_ipxrip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	pinfo->current_proto = "IPX RIP";
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-	 col_add_str(pinfo->fd, COL_PROTOCOL, "IPX RIP");
+	 col_set_str(pinfo->fd, COL_PROTOCOL, "IPX RIP");
 
 	operation = tvb_get_ntohs(tvb, 0) - 1;
 
-	if (check_col(pinfo->fd, COL_PROTOCOL)) {
+	if (check_col(pinfo->fd, COL_INFO)) {
 		/* rip_types 0 and 1 are valid, anything else becomes 2 or "Unknown" */
 		col_add_str(pinfo->fd, COL_INFO, rip_type[MIN(operation, 2)]);
 	}
@@ -679,7 +681,7 @@ dissect_ipxsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	pinfo->current_proto = "IPX SAP";
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, "IPX SAP");
+		col_set_str(pinfo->fd, COL_PROTOCOL, "IPX SAP");
 
 	query.query_type = tvb_get_ntohs(tvb, 0);
 	query.server_type = tvb_get_ntohs(tvb, 2);
@@ -689,7 +691,7 @@ dissect_ipxsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			col_add_str(pinfo->fd, COL_INFO, sap_type[query.query_type - 1]);
 		}
 		else {
-			col_add_str(pinfo->fd, COL_INFO, "Unknown Packet Type");
+			col_set_str(pinfo->fd, COL_INFO, "Unknown Packet Type");
 		}
 	}
 
@@ -906,6 +908,7 @@ proto_reg_handoff_ipx(void)
 	dissector_add("ethertype", ETHERTYPE_IPX, dissect_ipx);
 	dissector_add("ppp.protocol", PPP_IPX, dissect_ipx);
 	dissector_add("llc.dsap", SAP_NETWARE, dissect_ipx);
+	dissector_add("null.type", BSD_AF_IPX, dissect_ipx);
 	dissector_add("ipx.packet_type", IPX_PACKET_TYPE_SPX, dissect_spx);
 	dissector_add("ipx.socket", IPX_SOCKET_SAP, dissect_ipxsap);
 	dissector_add("ipx.socket", IPX_SOCKET_IPXRIP, dissect_ipxrip);
