@@ -1,7 +1,7 @@
 /* packet-irc.c
  * Routines for MSX irc packet dissection
  *
- * $Id: packet-irc.c,v 1.8 2000/08/13 14:08:18 deniel Exp $
+ * $Id: packet-irc.c,v 1.9 2000/11/19 08:53:58 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -81,7 +81,7 @@ dissect_irc(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	OLD_CHECK_DISPLAY_AS_DATA(proto_irc, pd, offset, fd, tree);
 
 	if (check_col(fd, COL_PROTOCOL))
-	col_add_str(fd, COL_PROTOCOL, "IRC");
+	col_set_str(fd, COL_PROTOCOL, "IRC");
 
 	if (check_col(fd, COL_INFO))
 	{
