@@ -3,7 +3,7 @@
  * (ISAKMP) (RFC 2408)
  * Brad Robel-Forrest <brad.robel-forrest@watchguard.com>
  *
- * $Id: packet-isakmp.c,v 1.26 2000/08/13 14:08:18 deniel Exp $
+ * $Id: packet-isakmp.c,v 1.30 2000/11/19 08:53:58 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -91,7 +91,7 @@ static const char *p1_atttypestr[NUM_P1_ATT_TYPES] = {
   "Group-Order"
 };
 
-#define NUM_ATT_TYPES	10
+#define NUM_ATT_TYPES	11
 #define atttype2str(t)	\
   ((t < NUM_ATT_TYPES) ? atttypestr[t] : "UNKNOWN-ATTRIBUTE-TYPE")
 
@@ -105,7 +105,8 @@ static const char *atttypestr[NUM_ATT_TYPES] = {
   "Key-Length",
   "Key-Rounds",
   "Compress-Dictinary-Size",
-  "Compress-Private-Algorithm"
+  "Compress-Private-Algorithm",
+  "ECN Tunnel"
 };
 
 #define NUM_TRANS_TYPES	2
@@ -117,7 +118,7 @@ static const char *transtypestr[NUM_TRANS_TYPES] = {
   "KEY_IKE"
 };
 
-#define NUM_AH_TRANS_TYPES	5
+#define NUM_AH_TRANS_TYPES	8
 #define ah_trans2str(t)		\
   ((t < NUM_AH_TRANS_TYPES) ? ah_transtypestr[t] : "UNKNOWN-AH-TRANS-TYPE")
 
@@ -126,10 +127,13 @@ static const char *ah_transtypestr[NUM_AH_TRANS_TYPES] = {
   "RESERVED",
   "MD5",
   "SHA",
-  "DES"
+  "DES",
+  "SHA2-256",
+  "SHA2-384",
+  "SHA2-512"
 };
 
-#define NUM_ESP_TRANS_TYPES	12
+#define NUM_ESP_TRANS_TYPES	13
 #define esp_trans2str(t)	\
   ((t < NUM_ESP_TRANS_TYPES) ? esp_transtypestr[t] : "UNKNOWN-ESP-TRANS-TYPE")
 
@@ -145,7 +149,8 @@ static const char *esp_transtypestr[NUM_ESP_TRANS_TYPES] = {
   "3IDEA",
   "DES-IV32",
   "RC4",
-  "NULL"
+  "NULL",
+  "AES"
 };
 
 #define NUM_ID_TYPES	12
@@ -354,7 +359,7 @@ dissect_isakmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   OLD_CHECK_DISPLAY_AS_DATA(proto_isakmp, pd, offset, fd, tree);
   
   if (check_col(fd, COL_PROTOCOL))
-    col_add_str(fd, COL_PROTOCOL, "ISAKMP");
+    col_set_str(fd, COL_PROTOCOL, "ISAKMP");
   
   len = pntohl(&hdr->length);
   
@@ -547,6 +552,7 @@ dissect_transform(const u_char *pd, int offset, frame_data *fd,
 
   struct trans_hdr *	hdr	= (struct trans_hdr *)(pd + offset);
   guint16		length	= pntohs(&hdr->length);
+  int                   slength = length;
   proto_item *		ti	= proto_tree_add_text(tree, NullTVB, offset, length, "Transform payload");
   proto_tree *		ntree;
 
@@ -585,8 +591,8 @@ dissect_transform(const u_char *pd, int offset, frame_data *fd,
   }
   offset += sizeof(hdr->transform_id) + sizeof(hdr->reserved2);
   
-  length -= sizeof(*hdr);
-  while (length) {
+  slength -= sizeof(*hdr);
+  while (slength>0) {
     const char *str = NULL;
     int ike_phase1 = 0;
     guint16 type    = pntohs(pd + offset) & 0x7fff;
@@ -606,7 +612,7 @@ dissect_transform(const u_char *pd, int offset, frame_data *fd,
 			  str, type,
 			  value2str(ike_phase1, type, val_len), val_len);
       offset += 4;
-      length -= 4;
+      slength -= 4;
     }
     else {
       guint16	pack_len = 4 + val_len;
@@ -616,7 +622,12 @@ dissect_transform(const u_char *pd, int offset, frame_data *fd,
 			  str, type,
 			  num2str(pd + offset + 4, val_len));
       offset += pack_len;
-      length -= pack_len;
+      slength -= pack_len;
+    }
+    if (!IS_DATA_IN_FRAME(offset)) {
+	    proto_tree_add_text(ntree, NullTVB, 0, 0,
+			    "Bad Offset: %u", offset);
+	    return;
     }
   }
 
@@ -930,7 +941,7 @@ dissect_notif(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     offset += hdr->spi_size;
   }
 
-  if (length - sizeof(*hdr)) {
+  if (((int)length - sizeof(*hdr)) > 0) {
     proto_tree_add_text(ntree, NullTVB, offset, length - sizeof(*hdr) - hdr->spi_size,
 			"Notification Data");
     offset += (length - sizeof(*hdr) - hdr->spi_size);
@@ -1038,6 +1049,7 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
   struct cfg_hdr * 	hdr	 = (struct cfg_hdr *)(pd + offset);
   guint16		length	 = pntohs(&hdr->length);
+  int                   slength  = length;
   proto_item *		ti	 = proto_tree_add_text(tree, NullTVB, offset, length, "Attribute payload");
   proto_tree *		ntree;
 
@@ -1060,9 +1072,9 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   proto_tree_add_text(ntree, NullTVB, offset, sizeof(hdr->identifier),
                       "Identifier: %u", pntohs(&hdr->identifier));
   offset += sizeof(hdr->identifier);
-  length -= sizeof(*hdr);
+  slength -= sizeof(*hdr);
   
-  while(length) {
+  while(slength>0) {
     guint16 type = pntohs(pd + offset) & 0x7fff;
     guint16 val_len = pntohs(pd + offset + 2);
     
@@ -1070,7 +1082,7 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       proto_tree_add_text(ntree, NullTVB, offset, 4,
 			  "%s (%u)",cfgattrident2str(type),val_len);
       offset += 4;
-      length -= 4;
+      slength -= 4;
     }
     else {
       guint pack_len = 4 + val_len;
@@ -1078,7 +1090,7 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       proto_tree_add_text(ntree, NullTVB, offset, 4,
 			  "%s (%se)", cfgattrident2str(type), num2str(pd + offset + 4, val_len));
       offset += pack_len;
-      length -= pack_len;
+      slength -= pack_len;
     }
   }
 
@@ -1247,6 +1259,9 @@ value2str(int ike_p1, guint16 att_type, guint16 value) {
         case 2:  return "HMAC-SHA";
         case 3:  return "DES-MAC";
         case 4:  return "KPDK";
+	case 5:  return "HMAC-SHA2-256";
+	case 6:  return "HMAC-SHA2-384";
+	case 7:  return "HMAC-SHA2-512";
         default: return "UNKNOWN-AUTHENTICATION-VALUE";
       }
     case 6:
@@ -1268,6 +1283,7 @@ value2str(int ike_p1, guint16 att_type, guint16 value) {
           case 4:  return "RC5-R16-B64-CBC";
           case 5:  return "3DES-CBC";
           case 6:  return "CAST-CBC";
+	  case 7:  return "AES-CBC";
           default: return "UNKNOWN-ENCRYPTION-ALG";
         }
       case 2:
@@ -1275,6 +1291,9 @@ value2str(int ike_p1, guint16 att_type, guint16 value) {
           case 1:  return "MD5";
           case 2:  return "SHA";
           case 3:  return "TIGER";
+	  case 4:  return "SHA2-256";
+	  case 5:  return "SHA2-384";
+	  case 6:  return "SHA2-512";
           default: return "UNKNOWN-HASH-ALG";
         }
       case 3:
