@@ -1,6 +1,6 @@
 /* packet-lapb.h
  *
- * $Id: packet-lapb.h,v 1.4 2000/05/25 08:45:54 gram Exp $
+ * $Id: packet-lapb.h,v 1.5 2000/08/11 13:34:10 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,4 +22,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_LAPB_H__
+#define __PACKET_LAPB_H__
+
 void dissect_lapb(tvbuff_t *, packet_info *, proto_tree *);
+
+#endif
