@@ -2,7 +2,7 @@
  * Routines for LAPD frame disassembly
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-lapd.c,v 1.13 2000/08/13 14:08:24 deniel Exp $
+ * $Id: packet-lapd.c,v 1.16 2000/11/19 08:53:59 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -99,26 +99,26 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	pinfo->current_proto = "LAPD";
 
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, "LAPD");
+		col_set_str(pinfo->fd, COL_PROTOCOL, "LAPD");
 
 	address = tvb_get_ntohs(tvb, 0);
 	cr = address & LAPD_CR;
 	sapi = (address & LAPD_SAPI) >> LAPD_SAPI_SHIFT;
 	lapd_header_len = 2;	/* address */
 
-	if (pinfo->pseudo_header->lapd.from_network_to_user) {
-		is_response = cr ? FALSE : TRUE;
-		if(check_col(pinfo->fd, COL_RES_DL_DST))
-		    col_add_str(pinfo->fd, COL_RES_DL_DST, "User");
-		if(check_col(pinfo->fd, COL_RES_DL_SRC))
-		    col_add_str(pinfo->fd, COL_RES_DL_SRC, "Network");
-	}
-	else {
+	if (pinfo->pseudo_header->p2p.sent) {
 		is_response = cr ? TRUE : FALSE;
 		if(check_col(pinfo->fd, COL_RES_DL_DST))
-			col_add_str(pinfo->fd, COL_RES_DL_DST, "Network");
+			col_set_str(pinfo->fd, COL_RES_DL_DST, "Network");
 		if(check_col(pinfo->fd, COL_RES_DL_SRC))
-			col_add_str(pinfo->fd, COL_RES_DL_SRC, "User");
+			col_set_str(pinfo->fd, COL_RES_DL_SRC, "User");
+	}
+	else {
+		is_response = cr ? FALSE : TRUE;
+		if(check_col(pinfo->fd, COL_RES_DL_DST))
+		    col_set_str(pinfo->fd, COL_RES_DL_DST, "User");
+		if(check_col(pinfo->fd, COL_RES_DL_SRC))
+		    col_set_str(pinfo->fd, COL_RES_DL_SRC, "Network");
 	}
 
 
@@ -153,11 +153,11 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 
 		default:
-			dissect_data(next_tvb, pinfo, tree);
+			dissect_data(next_tvb, 0, pinfo, tree);
 			break;
 		}
 	} else
-		dissect_data(next_tvb, pinfo, tree);
+		dissect_data(next_tvb, 0, pinfo, tree);
 }
 
 void
