@@ -3,7 +3,7 @@
  * 
  * (c) Copyright Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: packet-mpls.c,v 1.9 2000/08/13 14:08:29 deniel Exp $
+ * $Id: packet-mpls.c,v 1.11 2000/11/19 08:54:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -45,7 +45,6 @@
 #include <glib.h>
 #include "etypes.h"
 #include "packet.h"
-#include "packet-ip.h"
 
 static gint proto_mpls = -1;
 
@@ -108,6 +107,8 @@ static hf_register_info mplsf_info[] = {
       "" }},
 };
 
+static dissector_handle_t ip_handle;
+
 /*
  * Given a 4-byte MPLS label starting at "start", decode this.
  * Return the label in "label", EXP bits in "exp",
@@ -137,7 +138,7 @@ dissect_mpls(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     OLD_CHECK_DISPLAY_AS_DATA(proto_mpls, pd, offset, fd, tree);
 
     if (check_col(fd, COL_PROTOCOL)) {
-	col_add_str(fd,COL_PROTOCOL, "MPLS");
+	col_set_str(fd,COL_PROTOCOL, "MPLS");
     }
     
     if (check_col(fd,COL_INFO)) {
@@ -177,7 +178,7 @@ dissect_mpls(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	offset += 4;
 	if (bos) break;
     }
-    dissect_ip(pd, offset, fd, tree);
+    old_call_dissector(ip_handle, pd, offset, fd, tree);
 }
 
 void
@@ -196,4 +197,9 @@ void
 proto_reg_handoff_mpls(void)
 {
 	old_dissector_add("ethertype", ETHERTYPE_MPLS, dissect_mpls);
+
+	/*
+	 * Get a handle for the IP dissector.
+	 */
+	ip_handle = find_dissector("ip");
 }
