@@ -2,7 +2,7 @@
  * Routines for Microsoft Proxy packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-msproxy.c,v 1.11 2000/08/21 18:36:34 guy Exp $
+ * $Id: packet-msproxy.c,v 1.14 2000/11/19 08:54:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -220,16 +220,17 @@ static void msproxy_sub_dissector( const u_char *pd, int offset, frame_data *fd,
 	conversation_t *conversation;
 	proto_tree      *msp_tree;
 	proto_item      *ti;
+	tvbuff_t        *tvb;
 	
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
-		pi.srcport, pi.destport);
+		pi.srcport, pi.destport, 0);
 
 	g_assert( conversation);	/* should always find a conversation */
 
 	redirect_info = (redirect_entry_t*)conversation->data;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "MS Proxy");
+		col_set_str(fd, COL_PROTOCOL, "MS Proxy");
 
 	if (check_col(fd, COL_INFO))
 		col_add_fstr(fd, COL_INFO, "%s", 
@@ -259,10 +260,11 @@ static void msproxy_sub_dissector( const u_char *pd, int offset, frame_data *fd,
 
         *ptr = redirect_info->remote_port;
    	
+	tvb = tvb_create_from_top(0);
 	if ( redirect_info->proto == PT_TCP)
-		decode_tcp_ports( pd, offset, fd, tree, pi.srcport, pi.destport);
+		decode_tcp_ports( tvb, offset, &pi, tree, pi.srcport, pi.destport);
 	else
-		decode_udp_ports( pd, offset, fd, tree, pi.srcport, pi.destport);
+		decode_udp_ports( tvb, offset, &pi, tree, pi.srcport, pi.destport);
  
         *ptr = redirect_info->server_int_port;
 }
@@ -286,14 +288,15 @@ static void add_msproxy_conversation( hash_entry_t *hash_info){
 	
 	conversation_t *conversation = find_conversation( &pi.src, &pi.dst, 
 		hash_info->proto, hash_info->server_int_port,
-		hash_info->clnt_port);
+		hash_info->clnt_port, 0);
 
 	if ( conversation)
 		return;
 
 	new_conv_info = g_mem_chunk_alloc(redirect_vals);
 	conversation = conversation_new( &pi.src, &pi.dst, hash_info->proto,
-		hash_info->server_int_port, hash_info->clnt_port, new_conv_info);
+		hash_info->server_int_port, hash_info->clnt_port,
+		new_conv_info, 0);
 
 	g_assert( new_conv_info);
 	g_assert( conversation);
@@ -1166,7 +1169,7 @@ static void dissect_msproxy(const u_char *pd, int offset, frame_data *fd, proto_
 	OLD_CHECK_DISPLAY_AS_DATA(proto_msproxy, pd, offset, fd, tree);
 
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
-		pi.srcport, pi.destport);
+		pi.srcport, pi.destport, 0);
 
 	if ( conversation)			/* conversation found */
 		hash_info = conversation->data;
@@ -1176,11 +1179,11 @@ static void dissect_msproxy(const u_char *pd, int offset, frame_data *fd, proto_
     		hash_info = g_mem_chunk_alloc(vals);
 
 		conversation_new( &pi.src, &pi.dst, pi.ptype,
-			pi.srcport, pi.destport, hash_info);
+			pi.srcport, pi.destport, hash_info, 0);
 	}
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "MSproxy");
+		col_set_str(fd, COL_PROTOCOL, "MSproxy");
 
 
 						/* display packet info */
