@@ -4,7 +4,7 @@
  * Gilbert Ramirez <gram@xiexie.org>
  * Much stuff added by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-nbns.c,v 1.45 2000/08/13 14:08:30 deniel Exp $
+ * $Id: packet-nbns.c,v 1.48 2000/11/19 16:58:57 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -194,13 +194,15 @@ nbns_type_name (int type)
 	return "unknown";
 }
 
+#define NBNAME_BUF_LEN 128
+
 static int
 get_nbns_name(const u_char *pd, int offset, int nbns_data_offset,
     char *name_ret, int *name_type_ret)
 {
 	int name_len;
 	char name[MAXDNAME];
-	char nbname[NETBIOS_NAME_LEN];
+	char nbname[NBNAME_BUF_LEN];
 	char *pname, *pnbname, cname, cnbname;
 	int name_type;
 
@@ -1138,10 +1140,12 @@ dissect_nbns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	nbns_data_offset = offset;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "NBNS");
+		col_set_str(fd, COL_PROTOCOL, "NBNS");
 
 	if (pi.captured_len < NBNS_HDRLEN) {
-		col_add_str(fd, COL_INFO, "Short NBNS packet");
+		if (check_col(fd, COL_INFO)) {
+			col_set_str(fd, COL_INFO, "Short NBNS packet");
+		}
 		old_dissect_data(pd, offset, fd, tree);
 		return;
 	}
@@ -1323,7 +1327,7 @@ dissect_nbdgm(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	}
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "NBDS");
+		col_set_str(fd, COL_PROTOCOL, "NBDS");
 	if (check_col(fd, COL_INFO)) {
 		col_add_fstr(fd, COL_INFO, "%s", message[message_index]);
 	}
@@ -1633,7 +1637,7 @@ dissect_nbss(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	    (memcmp(pd + offset + 4, "\377SMB", 4) != 0))) {
  
 	  if (check_col(fd, COL_PROTOCOL))
-	    col_add_str(fd, COL_PROTOCOL, "NBSS");
+	    col_set_str(fd, COL_PROTOCOL, "NBSS");
 	  if (check_col(fd, COL_INFO)) {
 	    col_add_fstr(fd, COL_INFO, "NBSS Continuation Message");
 	  }
@@ -1646,7 +1650,7 @@ dissect_nbss(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 #endif
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "NBSS");
+		col_set_str(fd, COL_PROTOCOL, "NBSS");
 	if (check_col(fd, COL_INFO)) {
 		col_add_fstr(fd, COL_INFO,
 		    val_to_str(msg_type, message_types, "Unknown (%x)"));
