@@ -2,7 +2,7 @@
  * Routines for NTP packet dissection
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-ntp.c,v 1.16 2000/08/13 14:08:25 deniel Exp $
+ * $Id: packet-ntp.c,v 1.20 2000/11/19 21:18:10 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -168,6 +168,9 @@ static const struct {
 	{ "DTS\0",	"Digital Time Service" },
 	{ "ATOM",	"Atomic clock (calibrated)" },
 	{ "VLF\0",	"VLF radio (OMEGA,, etc.)" },
+	{ "IRIG",	"IRIG-B timecode" },
+	{ "1PPS",	"External 1 PPS input" },
+	{ "FREE",	"(Internal clock)" },
 	{ NULL,		NULL}
 };
 
@@ -245,10 +248,10 @@ dissect_ntp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	pkt = (struct ntp_packet *) &pd[offset];
 	
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "NTP");
+		col_set_str(fd, COL_PROTOCOL, "NTP");
 
 	if (check_col(fd, COL_INFO))
-		col_add_str(fd, COL_INFO, "NTP");
+		col_set_str(fd, COL_INFO, "NTP");
 
 	if (tree) {
 		/* Adding NTP item and subtree */
@@ -330,8 +333,12 @@ dissect_ntp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 					break;
 				}
 			}
-		} else
-			strcpy (buff, get_hostname (pntohl(pkt->refid)));
+		} else {
+			buff[sizeof(buff) - 1] = '\0';
+			strncpy (buff, get_hostname (pntohl(pkt->refid)), sizeof(buff));
+			if (buff[sizeof(buff) - 1] != '\0')
+				strcpy(&buff[sizeof(buff) - 4], "...");
+		}
 		proto_tree_add_bytes_format(ntp_tree, hf_ntp_refid, NullTVB, offset+12, 4, pkt->refid,
 					   "Reference Clock ID: %s", buff);
 		/* Reference Timestamp: This is the time at which the local clock was
