@@ -2,7 +2,7 @@
  * Routines for PIM disassembly
  * (c) Copyright Jun-ichiro itojun Hagino <itojun@itojun.org>
  *
- * $Id: packet-pim.c,v 1.17 2000/08/13 14:08:37 deniel Exp $
+ * $Id: packet-pim.c,v 1.19 2000/11/18 10:38:25 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -36,6 +36,7 @@
 #include <netinet/in.h>
 #endif
 
+#include <string.h>
 #include <glib.h>
 
 #ifdef NEED_SNPRINTF_H
@@ -69,6 +70,8 @@ static int hf_pim_cksum = -1;
 
 static gint ett_pim = -1;
 
+static dissector_handle_t ip_handle;
+
 static const char *
 dissect_pim_addr(const u_char *bp, const u_char *ep, enum pimv2_addrtype at,
 	int *advance) {
@@ -283,9 +286,9 @@ dissect_pim(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	    switch((*ip & 0xf0) >> 4) {
 	    case 4:	/* IPv4 */
 #if 0
-		    dissect_ip(pd, ip - pd, fd, tree);
+		    old_call_dissector(ip_handle, pd, ip - pd, fd, tree);
 #else
-		    dissect_ip(pd, ip - pd, fd, pimopt_tree);
+		    old_call_dissector(ip_handle, pd, ip - pd, fd, pimopt_tree);
 #endif
 		    break;
 	    case 6:	/* IPv6 */
@@ -606,5 +609,9 @@ void
 proto_reg_handoff_pim(void)
 {
     old_dissector_add("ip.proto", IP_PROTO_PIM, dissect_pim);
-}
 
+    /*
+     * Get a handle for the IP dissector.
+     */
+    ip_handle = find_dissector("ip");
+}
