@@ -4,7 +4,7 @@
  * Uwe Girlich <uwe@planetquake.com>
  *	http://www.idsoftware.com/q1source/q1source.zip
  *
- * $Id: packet-quake.c,v 1.6 2000/08/30 02:50:01 gram Exp $
+ * $Id: packet-quake.c,v 1.9 2000/11/19 08:54:04 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -238,7 +238,8 @@ dissect_quake_CCREP_ACCEPT
 	conversation_t *c;
 
 	port = tvb_get_letohl(tvb, 0);
-	c = conversation_new( &pi.src, &pi.dst, PT_UDP, port, pi.destport, NULL);
+	c = conversation_new( &pi.src, &pi.dst, PT_UDP, port, pi.destport,
+	    NULL, 0);
 	if (c) {
 		conversation_set_dissector(c, dissect_quake);
 	}
@@ -498,7 +499,7 @@ dissect_quake_control(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			(next_tvb, pinfo, control_tree);
 		break;
 		default:
-			dissect_data(next_tvb, pinfo, control_tree);
+			dissect_data(next_tvb, 0, pinfo, control_tree);
 		break;
 	}
 }
@@ -524,7 +525,7 @@ dissect_quake(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	length &= NETFLAG_LENGTH_MASK;
 
 	if (check_col(pinfo->fd, COL_PROTOCOL))
-		col_add_str(pinfo->fd, COL_PROTOCOL, "QUAKE");
+		col_set_str(pinfo->fd, COL_PROTOCOL, "QUAKE");
 
 	if (tree) {
 		quake_item = proto_tree_add_item(tree, proto_quake,
@@ -585,7 +586,7 @@ dissect_quake(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	rest_length = tvb_reported_length(tvb) - 8;
 	next_tvb = tvb_new_subset(tvb, 8, rest_length , rest_length);
-	dissect_data(next_tvb, pinfo, quake_tree);
+	dissect_data(next_tvb, 0, pinfo, quake_tree);
 }
 
 void
