@@ -1,7 +1,7 @@
 /* packet-radius.c
  * Routines for RADIUS packet disassembly
  *
- * $Id: packet-radius.c,v 1.17 2000/08/13 14:08:35 deniel Exp $
+ * $Id: packet-radius.c,v 1.19 2000/11/19 08:54:04 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Johan Feyaerts
@@ -35,6 +35,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <glib.h>
 #include "packet.h"
@@ -694,7 +695,7 @@ proto_tree
 	codestrval="Unknown Packet";
   }
   if (check_col(fd, COL_PROTOCOL))
-        col_add_str(fd, COL_PROTOCOL, "RADIUS");
+        col_set_str(fd, COL_PROTOCOL, "RADIUS");
   if (check_col(fd, COL_INFO))
   {
 	sprintf(textbuffer,"%s(%d) (id=%d, l=%d)",
