@@ -1,7 +1,7 @@
 /* packet-raw.c
  * Routines for raw packet disassembly
  *
- * $Id: packet-raw.c,v 1.18 2000/08/13 08:53:51 guy Exp $
+ * $Id: packet-raw.c,v 1.22 2000/11/19 08:54:04 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -32,6 +32,7 @@
 # include <sys/types.h>
 #endif
 
+#include <string.h>
 #include <glib.h>
 #include "packet.h"
 #include "packet-raw.h"
@@ -42,6 +43,9 @@ static gint ett_raw = -1;
 
 static const char zeroes[10];
 
+static dissector_handle_t ip_handle;
+static dissector_handle_t ppp_handle;
+
 void
 capture_raw(const u_char *pd, packet_counts *ld)
 {
@@ -81,19 +85,17 @@ dissect_raw(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   proto_tree	*fh_tree;
   proto_item	*ti;
   tvbuff_t	*next_tvb;
-  const guint8	*next_pd;
-  int		next_offset;
 
   /* load the top pane info. This should be overwritten by
      the next protocol in the stack */
   if(check_col(pinfo->fd, COL_RES_DL_SRC))
-    col_add_str(pinfo->fd, COL_RES_DL_SRC, "N/A" );
+    col_set_str(pinfo->fd, COL_RES_DL_SRC, "N/A" );
   if(check_col(pinfo->fd, COL_RES_DL_DST))
-    col_add_str(pinfo->fd, COL_RES_DL_DST, "N/A" );
+    col_set_str(pinfo->fd, COL_RES_DL_DST, "N/A" );
   if(check_col(pinfo->fd, COL_PROTOCOL))
-    col_add_str(pinfo->fd, COL_PROTOCOL, "N/A" );
+    col_set_str(pinfo->fd, COL_PROTOCOL, "N/A" );
   if(check_col(pinfo->fd, COL_INFO))
-    col_add_str(pinfo->fd, COL_INFO, "Raw packet data" );
+    col_set_str(pinfo->fd, COL_INFO, "Raw packet data" );
 
   /* populate a tree in the second pane with the status of the link
      layer (ie none) */
@@ -112,32 +114,32 @@ dissect_raw(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * sometimes.  This check should be removed when 2.2 is out.
    */
   if (tvb_get_ntohs(tvb, 0) == 0xff03) {
-	dissect_ppp(tvb, pinfo, tree);
+	call_dissector(ppp_handle, tvb, pinfo, tree);
 	return;
   }
   /* The Linux ISDN driver sends a fake MAC address before the PPP header
    * on its ippp interfaces... */
   else if (tvb_get_ntohs(tvb, 6) == 0xff03) {
 	next_tvb = tvb_new_subset(tvb, 6, -1, -1);
-	dissect_ppp(next_tvb, pinfo, tree);
+	call_dissector(ppp_handle, next_tvb, pinfo, tree);
 	return;
   }
   /* ...except when it just puts out one byte before the PPP header... */
   else if (tvb_get_ntohs(tvb, 1) == 0xff03) {
 	next_tvb = tvb_new_subset(tvb, 1, -1, -1);
-	dissect_ppp(next_tvb, pinfo, tree);
+	call_dissector(ppp_handle, next_tvb, pinfo, tree);
 	return;
   }
   /* ...and if the connection is currently down, it sends 10 bytes of zeroes
    * instead of a fake MAC address and PPP header. */
   else if (memcmp(tvb_get_ptr(tvb, 0, 10), zeroes, 10) == 0) {
-	tvb_compat(tvb, &next_pd, &next_offset);
-	dissect_ip(next_pd, next_offset + 10, pinfo->fd, tree);
+	next_tvb = tvb_new_subset(tvb, 10, -1, -1);
+	call_dissector(ip_handle, next_tvb, pinfo, tree);
 	return;
   }
   else {
-	tvb_compat(tvb, &next_pd, &next_offset);
-	dissect_ip(next_pd, next_offset, pinfo->fd, tree);
+	next_tvb = tvb_new_subset(tvb, 0, -1, -1);
+	call_dissector(ip_handle, next_tvb, pinfo, tree);
 	return;
   }
   g_assert_not_reached();
@@ -152,3 +154,13 @@ proto_register_raw(void)
 
   proto_register_subtree_array(ett, array_length(ett));
 }
+
+void
+proto_reg_handoff_raw(void)
+{
+  /*
+   * Get handles for the IP and PPP dissectors.
+   */
+  ip_handle = find_dissector("ip");
+  ppp_handle = find_dissector("ppp");
+}
