@@ -2,7 +2,7 @@
  * Routines for rpc dissection
  * Copyright 1999, Uwe Girlich <Uwe.Girlich@philosys.de>
  * 
- * $Id: packet-rpc.c,v 1.41 2000/08/24 23:09:37 guy Exp $
+ * $Id: packet-rpc.c,v 1.43 2000/11/19 08:54:04 guy Exp $
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -1133,7 +1133,7 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		   no guarantee that the reply will come from the address
 		   to which the call was sent.) */
 		conversation = find_conversation(&null_address, &pi.dst,
-		    pi.ptype, pi.srcport, pi.destport);
+		    pi.ptype, pi.srcport, pi.destport, 0);
 		if (conversation == NULL) {
 			/* We haven't seen an RPC call for that conversation,
 			   so we can't check for a reply to that call. */
@@ -1158,7 +1158,7 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	}
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "RPC");
+		col_set_str(fd, COL_PROTOCOL, "RPC");
 
 	if (tree) {
 		rpc_item = proto_tree_add_item(tree, proto_rpc, NullTVB, offset, END_OF_FRAME, FALSE);
@@ -1273,11 +1273,11 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		   guarantee that the reply will come from the address
 		   to which the call was sent.) */
 		conversation = find_conversation(&pi.src, &null_address,
-		    pi.ptype, pi.srcport, pi.destport);
+		    pi.ptype, pi.srcport, pi.destport, 0);
 		if (conversation == NULL) {
 			/* It's not part of any conversation - create a new one. */
 			conversation = conversation_new(&pi.src, &null_address,
-			    pi.ptype, pi.srcport, pi.destport, NULL);
+			    pi.ptype, pi.srcport, pi.destport, NULL, 0);
 		}
 
 		/* prepare the key data */
