@@ -2,7 +2,7 @@
  * Routines for SNMP (simple network management protocol)
  * D.Jorand (c) 1998
  *
- * $Id: packet-snmp.c,v 1.48 2000/09/08 07:14:01 guy Exp $
+ * $Id: packet-snmp.c,v 1.53 2000/11/19 08:54:07 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -54,6 +54,8 @@
 #include <glib.h>
 
 #include "packet.h"
+#include "strutil.h"
+#include "conversation.h"
 #include "etypes.h"
 #include "packet-ipx.h"
 
@@ -1561,7 +1563,7 @@ dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
 			    "Encrypted PDU (%d bytes)", length);
 			g_free(cryptpdu);
 			if (check_col(fd, COL_INFO))
-				col_add_str(fd, COL_INFO, "Encrypted PDU");
+				col_set_str(fd, COL_INFO, "Encrypted PDU");
 			return;
 		}
 		ret = asn1_sequence_decode(&asn1, &global_length, &length);
@@ -1658,7 +1660,7 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 	guint cls, con;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "SMUX");
+		col_set_str(fd, COL_PROTOCOL, "SMUX");
 
 	if (tree) {
 		item = proto_tree_add_item(tree, proto, NullTVB, offset,
@@ -1893,7 +1895,35 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 static void
 dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) 
 {
+	conversation_t  *conversation;
+
 	OLD_CHECK_DISPLAY_AS_DATA(proto_snmp, pd, offset, fd, tree);
+
+	/*
+	 * The first SNMP packet goes to the SNMP port; the second one
+	 * may come from some *other* port, but goes back to the same
+	 * IP address and port as the ones from which the first packet
+	 * came; all subsequent packets presumably go between those two
+	 * IP addresses and ports.
+	 *
+	 * If this packet went to the SNMP port, we check to see if
+	 * there's already a conversation with the source IP address
+	 * and port of this packet, the destination IP address of this
+	 * packet, and any destination UDP port.  If not, we create
+	 * one, with a wildcard UDP port, and give it the SNMP dissector
+	 * as a dissector.
+	 */
+	if (pi.destport == UDP_PORT_SNMP) {
+	  conversation = find_conversation(&pi.src, &pi.dst, PT_UDP,
+					   pi.srcport, 0, NO_DST_PORT);
+	  if (conversation == NULL) {
+	    conversation = conversation_new(&pi.src, &pi.dst, PT_UDP,
+					    pi.srcport, 0, NULL,
+					    NO_DST_PORT);
+	    old_conversation_set_dissector(conversation, dissect_snmp);
+	  }
+	}
+
 	dissect_snmp_pdu(pd, offset, fd, tree, "SNMP", proto_snmp, ett_snmp);
 }
 
@@ -2041,6 +2071,7 @@ proto_register_snmp(void)
 				(*ds_set_int_p)(0, 4, 2);
 			}
 		}
+		dlclose(libsnmp_handle);
 	}
 #else /* linux */
 	snmp_set_suffix_only(2);
