@@ -2,7 +2,7 @@
  * Routines for socks versions 4 &5  packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-socks.c,v 1.12 2000/09/11 16:16:07 gram Exp $
+ * $Id: packet-socks.c,v 1.15 2000/11/19 08:54:08 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -366,16 +366,17 @@ static void socks_udp_dissector( const u_char *pd, int offset, frame_data *fd,
 	conversation_t *conversation;
 	proto_tree      *socks_tree;
 	proto_item      *ti;
+	tvbuff_t        *tvb;
 	
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
-		pi.srcport, pi.destport);
+		pi.srcport, pi.destport, 0);
 
 	g_assert( conversation);	/* should always find a conversation */
 
 	hash_info = (socks_hash_entry_t*)conversation->data;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "Socks");
+		col_set_str(fd, COL_PROTOCOL, "Socks");
 
 	if (check_col(fd, COL_INFO))
 		col_add_fstr(fd, COL_INFO, "Version: 5, UDP Associated packet");
@@ -420,7 +421,8 @@ static void socks_udp_dissector( const u_char *pd, int offset, frame_data *fd,
 
         *ptr = hash_info->udp_remote_port;
    	
-	decode_udp_ports( pd, offset, fd, tree, pi.srcport, pi.destport);
+	tvb = tvb_create_from_top(0);
+	decode_udp_ports( tvb, offset, &pi, tree, pi.srcport, pi.destport);
  
         *ptr = hash_info->udp_port;
 
@@ -430,7 +432,7 @@ static void socks_udp_dissector( const u_char *pd, int offset, frame_data *fd,
 void new_udp_conversation( socks_hash_entry_t *hash_info){
 
 	conversation_t *conversation = conversation_new( &pi.src, &pi.dst,  PT_UDP,
-			hash_info->udp_port, hash_info->port, hash_info);
+			hash_info->udp_port, hash_info->port, hash_info, 0);
 			
 	g_assert( conversation);
 	
@@ -923,6 +925,7 @@ static void call_next_dissector( const u_char *pd, int offset, frame_data *fd,
 /* payload, and restore the pi port after that is done.		     	*/
 
 	guint32 *ptr;
+	tvbuff_t *tvb;
  
  	if (( hash_info->command  == PING_COMMAND) ||
  	    ( hash_info->command  == TRACERT_COMMAND))
@@ -939,7 +942,8 @@ static void call_next_dissector( const u_char *pd, int offset, frame_data *fd,
         		ptr = &pi.srcport;
 
 	        *ptr = hash_info->port;
-		decode_tcp_ports( pd, offset, fd, tree, pi.srcport, pi.destport);
+		tvb = tvb_create_from_top(0);
+		decode_tcp_ports( tvb, offset, &pi, tree, pi.srcport, pi.destport);
 	        *ptr = TCP_PORT_SOCKS;
 	}
 }                
@@ -957,7 +961,7 @@ dissect_socks(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	OLD_CHECK_DISPLAY_AS_DATA(proto_socks, pd, offset, fd, tree);
 
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
-		pi.srcport, pi.destport);
+		pi.srcport, pi.destport, 0);
 
 	if ( conversation)			/* conversation found */
 		hash_info = conversation->data;
@@ -975,13 +979,13 @@ dissect_socks(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     			hash_info->state = Done;
 
 		conversation_new( &pi.src, &pi.dst, pi.ptype,
-			pi.srcport, pi.destport, hash_info);
+			pi.srcport, pi.destport, hash_info, 0);
 	}
 
 /* display summary window information  */
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "Socks");
+		col_set_str(fd, COL_PROTOCOL, "Socks");
 
 	if (check_col(fd, COL_INFO)){
 		if (( hash_info->version == 4) || ( hash_info->version == 5)){
@@ -989,7 +993,7 @@ dissect_socks(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 				hash_info->version);
 		}		
 		else			/* unknown version display error */
-			col_add_str(fd, COL_INFO, "Unknown");
+			col_set_str(fd, COL_INFO, "Unknown");
 		
 
 		if ( hash_info->command == PING_COMMAND)
