@@ -2,7 +2,7 @@
  * Routines for x25 packet disassembly
  * Olivier Abad <oabad@cybercable.fr>
  *
- * $Id: packet-x25.c,v 1.35 2000/08/13 14:09:13 deniel Exp $
+ * $Id: packet-x25.c,v 1.39 2000/11/19 08:54:10 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -41,7 +41,6 @@
 #include "packet-x25.h"
 #include "packet-ip.h"
 #include "packet-osi.h"
-#include "packet-clnp.h"
 #include "nlpid.h"
 
 #define FROM_DCE			0x80
@@ -160,6 +159,9 @@ static const value_string vals_x25_type[] = {
 	{ 0,   NULL}
 };
 
+static dissector_handle_t ip_handle;
+static dissector_handle_t ositp_handle;
+
 /*
  * each vc_info node contains :
  *   the time of the first frame using this dissector (secs and usecs)
@@ -173,7 +175,7 @@ static const value_string vals_x25_type[] = {
 typedef struct _vc_info {
 	guint32 first_frame_secs, first_frame_usecs;
 	guint32 last_frame_secs, last_frame_usecs;
-	void (*dissect)(const u_char *, int, frame_data *, proto_tree *);
+	dissector_handle_t dissect;
 	struct _vc_info *next;
 } vc_info;
 
@@ -191,7 +193,7 @@ typedef struct _global_vc_info {
 
 static global_vc_info *hash_table[64];
 
-void
+static void
 free_vc_info(vc_info *pt)
 {
   vc_info *vci = pt;
@@ -225,10 +227,9 @@ reinit_x25_hashtable(void)
   }
 }
 
-void
+static void
 x25_hash_add_proto_start(guint16 vc, guint32 frame_secs, guint32 frame_usecs,
-		         void (*dissect)(const u_char *, int, frame_data *,
-				       proto_tree *))
+		         dissector_handle_t dissect)
 {
   int idx = vc % 64;
   global_vc_info *hash_ent;
@@ -308,7 +309,7 @@ x25_hash_add_proto_start(guint16 vc, guint32 frame_secs, guint32 frame_usecs,
   }
 }
 
-void
+static void
 x25_hash_add_proto_end(guint16 vc, guint32 frame_secs, guint32 frame_usecs)
 {
   global_vc_info *hash_ent = hash_table[vc%64];
@@ -324,7 +325,8 @@ x25_hash_add_proto_end(guint16 vc, guint32 frame_secs, guint32 frame_usecs)
   vci->last_frame_usecs = frame_usecs;
 }
 
-void (*x25_hash_get_dissect(guint32 frame_secs, guint32 frame_usecs, guint16 vc))(const u_char *, int, frame_data *, proto_tree *)
+static dissector_handle_t
+x25_hash_get_dissect(guint32 frame_secs, guint32 frame_usecs, guint16 vc)
 {
   global_vc_info *hash_ent = hash_table[vc%64];
   vc_info *vci;
@@ -621,7 +623,7 @@ static char *registration_code(unsigned char code)
     return buffer;
 }
 
-void
+static void
 dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 {
     guint8 fac, byte1, byte2, byte3;
@@ -1200,7 +1202,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
     }
 }
 
-void
+static void
 x25_ntoa(proto_tree *tree, int *offset, tvbuff_t *tvb,
 	 frame_data *fd, gboolean toa)
 {
@@ -1289,7 +1291,7 @@ x25_ntoa(proto_tree *tree, int *offset, tvbuff_t *tvb,
     (*offset) += ((len1 + len2 + 1) / 2);
 }
 
-int
+static int
 get_x25_pkt_len(tvbuff_t *tvb)
 {
     int length, called_len, calling_len, dte_len, dce_len;
@@ -1394,18 +1396,16 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     int x25_pkt_len;
     int modulo;
     guint16 vc;
-    void (*dissect)(const u_char *, int, frame_data *, proto_tree *);
+    dissector_handle_t dissect;
     gboolean toa;         /* TOA/NPI address format */
     guint16 bytes0_1;
     guint8 pkt_type;
     tvbuff_t *next_tvb;
-    const guint8 *next_pd;
-    int next_offset;
 
     pinfo->current_proto = "X.25";
 
     if (check_col(pinfo->fd, COL_PROTOCOL))
-	col_add_str(pinfo->fd, COL_PROTOCOL, "X.25");
+	col_set_str(pinfo->fd, COL_PROTOCOL, "X.25");
 
     bytes0_1 = tvb_get_ntohs(tvb, 0);
 
@@ -1425,7 +1425,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     if (x25_pkt_len < 3) /* packet too short */
     {
 	if (check_col(pinfo->fd, COL_INFO))
-	    col_add_str(pinfo->fd, COL_INFO, "Invalid/short X.25 packet");
+	    col_set_str(pinfo->fd, COL_INFO, "Invalid/short X.25 packet");
 	if (tree)
 	    proto_tree_add_protocol_format(tree,
 		    (modulo == 8 ? proto_x25 : proto_ex25), tvb, 0,
@@ -1488,7 +1488,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	    case NLPID_IP:
 		x25_hash_add_proto_start(vc, pinfo->fd->abs_secs,
-					 pinfo->fd->abs_usecs, dissect_ip);
+					 pinfo->fd->abs_usecs, ip_handle);
 		if (x25_tree)
 		    proto_tree_add_text(x25_tree, tvb, localoffset, 1,
 					"X.224 secondary protocol ID: IP");
@@ -1548,7 +1548,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		    case PRT_ID_ISO_8073:
 			/* ISO 8073 COTP */
 			x25_hash_add_proto_start(vc, pinfo->fd->abs_secs,
-					 pinfo->fd->abs_usecs, dissect_ositp);
+					 pinfo->fd->abs_usecs, ositp_handle);
 			break;
 
 		    default:
@@ -1728,7 +1728,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	break;
     case X25_RESTART_CONFIRMATION:
 	if(check_col(pinfo->fd, COL_INFO))
-	    col_add_str(pinfo->fd, COL_INFO, "Restart conf.");
+	    col_set_str(pinfo->fd, COL_INFO, "Restart conf.");
 	if (x25_tree)
 	    proto_tree_add_uint(x25_tree, (modulo == 8) ? hf_x25_type : hf_ex25_type, tvb,
 		    2, 1, X25_RESTART_CONFIRMATION);
@@ -1736,7 +1736,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	break;
     case X25_REGISTRATION_REQUEST:
 	if(check_col(pinfo->fd, COL_INFO))
-	    col_add_str(pinfo->fd, COL_INFO, "Registration req.");
+	    col_set_str(pinfo->fd, COL_INFO, "Registration req.");
 	if (x25_tree)
 	    proto_tree_add_uint(x25_tree, (modulo == 8) ? hf_x25_type : hf_ex25_type, tvb,
 		    2, 1, X25_REGISTRATION_REQUEST);
@@ -1758,7 +1758,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	break;
     case X25_REGISTRATION_CONFIRMATION:
 	if(check_col(pinfo->fd, COL_INFO))
-	    col_add_str(pinfo->fd, COL_INFO, "Registration conf.");
+	    col_set_str(pinfo->fd, COL_INFO, "Registration conf.");
 	if (x25_tree) {
 	    proto_tree_add_uint(x25_tree, (modulo == 8) ? hf_x25_type : hf_ex25_type, tvb,
 		    2, 1, X25_REGISTRATION_CONFIRMATION);
@@ -1921,19 +1921,18 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     if (localoffset >= tvb_reported_length(tvb)) return;
 
     next_tvb = tvb_new_subset(tvb, localoffset, -1, -1);
-    tvb_compat(next_tvb, &next_pd, &next_offset);
     /* search the dissector in the hash table */
     if ((dissect = x25_hash_get_dissect(pinfo->fd->abs_secs, pinfo->fd->abs_usecs, vc)))
-	(*dissect)(next_pd, next_offset, pinfo->fd, tree);
+	call_dissector(dissect, next_tvb, pinfo, tree);
     else {
 	/* If the Call Req. has not been captured, assume these packets carry IP */
 	if (tvb_get_guint8(tvb, localoffset) == 0x45) {
 	    x25_hash_add_proto_start(vc, pinfo->fd->abs_secs,
-		    pinfo->fd->abs_usecs, dissect_ip);
-	    dissect_ip(next_pd, next_offset, pinfo->fd, tree);
+		    pinfo->fd->abs_usecs, ip_handle);
+	    call_dissector(ip_handle, next_tvb, pinfo, tree);
 	}
 	else {
-	    dissect_data(next_tvb, pinfo, tree);
+	    dissect_data(next_tvb, 0, pinfo, tree);
 	}
     }
 }
@@ -2026,3 +2025,13 @@ proto_register_x25(void)
     proto_register_subtree_array(ett, array_length(ett));
     register_init_routine(&reinit_x25_hashtable);
 }
+
+void
+proto_reg_handoff_x25(void)
+{
+    /*
+     * Get handles for the IP and OSI TP (COTP/CLTP) dissector.
+     */
+    ip_handle = find_dissector("ip");
+    ositp_handle = find_dissector("ositp");
+}
