@@ -1,7 +1,7 @@
 /* plugin_api.h
  * Routines for Ethereal plugins.
  *
- * $Id: plugin_api.h,v 1.7 2000/08/11 13:37:21 deniel Exp $
+ * $Id: plugin_api.h,v 1.12 2000/11/15 07:07:50 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@xiexie.org>
@@ -52,9 +52,24 @@
 #define	proto_register_subtree_array	(*p_proto_register_subtree_array)
 
 #define	dissector_add			(*p_dissector_add)
+#define old_dissector_add		(*p_old_dissector_add)
+#define dissector_delete		(*p_dissector_delete)
 
 #define	heur_dissector_add		(*p_heur_dissector_add)
 
+#define register_dissector		(*p_register_dissector)
+#define find_dissector			(*p_find_dissector)
+#define old_call_dissector		(*p_old_call_dissector)
+#define call_dissector			(*p_call_dissector)
+
+#define dissect_data			(*p_dissect_data)
+#define old_dissect_data		(*p_old_dissect_data)
+
+#define proto_is_protocol_enabled	(*p_proto_is_protocol_enabled)
+
+#define proto_item_get_len		(*p_proto_item_get_len)
+#define proto_item_set_len		(*p_proto_item_set_len)
+#define proto_item_set_text		(*p_proto_item_set_text)
 #define	proto_item_add_subtree		(*p_proto_item_add_subtree)
 #define	proto_tree_add_item		(*p_proto_tree_add_item)
 #define	proto_tree_add_item_hidden	(*p_proto_tree_add_item_hidden)
@@ -95,6 +110,58 @@
 #define	proto_tree_add_text		(*p_proto_tree_add_text)
 #define	proto_tree_add_notext		(*p_proto_tree_add_notext)
 
+#define tvb_new_subset			(*p_tvb_new_subset)
+
+#define tvb_length			(*p_tvb_length)
+#define tvb_length_remaining		(*p_tvb_length_remaining)
+#define tvb_bytes_exist			(*p_tvb_bytes_exist)
+#define tvb_offset_exists		(*p_tvb_offset_exists)
+#define tvb_reported_length		(*p_tvb_reported_length)
+
+#define tvb_get_guint8			(*p_tvb_get_guint8)
+
+#define tvb_get_ntohs			(*p_tvb_get_ntohs)
+#define tvb_get_ntoh24			(*p_tvb_get_ntoh24)
+#define tvb_get_ntohl			(*p_tvb_get_ntohl)
+#ifdef G_HAVE_GINT64
+#define tvb_get_ntohll			(*p_tvb_get_ntohll)
+#endif
+
+#define tvb_get_letohs			(*p_tvb_get_letohs)
+#define tvb_get_letoh24			(*p_tvb_get_letoh24)
+#define tvb_get_letohl			(*p_tvb_get_letohl)
+#ifdef G_HAVE_GINT64
+#define tvb_get_letohll			(*p_tvb_get_letohll)
+#endif
+
+#define tvb_memcpy			(*p_tvb_memcpy)
+#define tvb_memdup			(*p_tvb_memdup)
+
+#define tvb_get_ptr			(*p_tvb_get_ptr)
+
+#define tvb_find_guint8			(*p_tvb_find_guint8)
+#define tvb_pbrk_guint8			(*p_tvb_pbrk_guint8)
+
+#define tvb_strnlen			(*p_tvb_strnlen)
+
+#define tvb_format_text			(*p_tvb_format_text)
+
+#define tvb_get_nstringz		(*p_tvb_get_nstringz)
+#define tvb_get_nstringz0		(*p_tvb_get_nstringz0)
+
+#define tvb_find_line_end		(*p_tvb_find_line_end)
+#define tvb_find_line_end_unquoted	(*p_tvb_find_line_end_unquoted)
+
+#define tvb_strneql			(*p_tvb_strneql)
+#define tvb_strncaseeql			(*p_tvb_strncaseeql)
+
+#define tvb_bytes_to_str		(*p_tvb_bytes_to_str)
+
+#define prefs_register_module		(*p_prefs_register_module)
+#define prefs_register_uint_preference	(*p_prefs_register_uint_preference)
+#define prefs_register_bool_preference	(*p_prefs_register_bool_preference)
+#define prefs_register_enum_preference	(*p_prefs_register_enum_preference)
+
 #define pi	(*p_pi)
 
 #else
@@ -105,6 +172,7 @@
 #endif
 
 #include "packet.h"
+#include "prefs.h"
 #include "dfilter.h"
 
 #include "plugin_table.h"
