@@ -1,7 +1,7 @@
 /* plugin_table.h
  * Table of exported addresses for Ethereal plugins.
  *
- * $Id: plugin_table.h,v 1.4 2000/05/31 18:32:15 gram Exp $
+ * $Id: plugin_table.h,v 1.11 2000/11/18 21:41:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@xiexie.org>
@@ -44,9 +44,26 @@ typedef void (*addr_proto_register_field_array)(int, hf_register_info*, int);
 typedef void (*addr_proto_register_subtree_array)(int**, int);
 
 typedef void (*addr_dissector_add)(const char *, guint32, dissector_t);
+typedef void (*addr_old_dissector_add)(const char *, guint32, old_dissector_t);
+typedef void (*addr_dissector_delete)(const char *, guint32, dissector_t);
 
 typedef void (*addr_heur_dissector_add)(const char *, heur_dissector_t);
 
+typedef void (*addr_register_dissector)(const char *, dissector_t);
+typedef dissector_handle_t (*addr_find_dissector)(const char *);
+typedef void (*addr_old_call_dissector)(dissector_handle_t, const u_char *,
+    int, frame_data *, proto_tree *);
+typedef void (*addr_call_dissector)(dissector_handle_t, tvbuff_t *,
+    packet_info *, proto_tree *);
+
+typedef void (*addr_dissect_data)(tvbuff_t *, int, packet_info *, proto_tree *);
+typedef void (*addr_old_dissect_data)(const u_char *, int, frame_data *, proto_tree *);
+
+typedef gboolean (*addr_proto_is_protocol_enabled)(int);
+
+typedef int (*addr_proto_item_get_len)(proto_item*);
+typedef void (*addr_proto_item_set_len)(proto_item*, gint);
+typedef void (*addr_proto_item_set_text)(proto_item*, const char*, ...);
 typedef proto_tree* (*addr_proto_item_add_subtree)(proto_item*, gint);
 typedef proto_item* (*addr_proto_tree_add_item)(proto_tree*, int, tvbuff_t*, gint, gint, gboolean);
 typedef proto_item* (*addr_proto_tree_add_item_hidden)(proto_tree*, int, tvbuff_t*, gint, gint, gboolean);
@@ -99,6 +116,63 @@ typedef proto_item* (*addr_proto_tree_add_int_format)(proto_tree*, int, tvbuff_t
 typedef proto_item* (*addr_proto_tree_add_text)(proto_tree*, tvbuff_t*, gint, gint, const char*, ...);
 typedef proto_item* (*addr_proto_tree_add_notext)(proto_tree*, tvbuff_t*, gint, gint);
 
+typedef tvbuff_t* (*addr_tvb_new_subset)(tvbuff_t*, gint, gint, gint);
+
+typedef guint (*addr_tvb_length)(tvbuff_t*);
+typedef guint (*addr_tvb_length_remaining)(tvbuff_t*, gint);
+typedef gboolean (*addr_tvb_bytes_exist)(tvbuff_t*, gint, gint);
+typedef gboolean (*addr_tvb_offset_exists)(tvbuff_t*, gint);
+typedef guint (*addr_tvb_reported_length)(tvbuff_t*);
+
+typedef guint8 (*addr_tvb_get_guint8)(tvbuff_t*, gint);
+
+typedef guint16 (*addr_tvb_get_ntohs)(tvbuff_t*, gint);
+typedef guint32 (*addr_tvb_get_ntoh24)(tvbuff_t*, gint);
+typedef guint32 (*addr_tvb_get_ntohl)(tvbuff_t*, gint);
+#ifdef G_HAVE_GINT64
+typedef guint64 (*addr_tvb_get_ntohll)(tvbuff_t*, gint);
+#endif
+
+typedef guint16 (*addr_tvb_get_letohs)(tvbuff_t*, gint);
+typedef guint32 (*addr_tvb_get_letoh24)(tvbuff_t*, gint);
+typedef guint32 (*addr_tvb_get_letohl)(tvbuff_t*, gint);
+#ifdef G_HAVE_GINT64
+typedef guint64 (*addr_tvb_get_letohll)(tvbuff_t*, gint);
+#endif
+
+typedef guint8* (*addr_tvb_memcpy)(tvbuff_t*, guint8* target, gint, gint);
+typedef guint8* (*addr_tvb_memdup)(tvbuff_t*, gint, gint);
+
+typedef guint8* (*addr_tvb_get_ptr)(tvbuff_t*, gint, gint);
+
+typedef gint (*addr_tvb_find_guint8)(tvbuff_t*, gint, guint, guint8);
+typedef gint (*addr_tvb_pbrk_guint8)(tvbuff_t *, gint, guint, guint8 *);
+
+typedef gint (*addr_tvb_strnlen)(tvbuff_t*, gint, guint);
+
+typedef guint8 * (*addr_tvb_format_text)(tvbuff_t*, gint, gint);
+
+typedef gint (*addr_tvb_get_nstringz)(tvbuff_t*, gint, guint, guint8*);
+typedef gint (*addr_tvb_get_nstringz0)(tvbuff_t*, gint, guint, guint8*);
+
+typedef gint (*addr_tvb_find_line_end)(tvbuff_t*, gint, int, gint *);
+typedef gint (*addr_tvb_find_line_end_unquoted)(tvbuff_t*, gint, int, gint *);
+
+typedef gint (*addr_tvb_strneql)(tvbuff_t*, gint, const guint8 *, gint);
+typedef gint (*addr_tvb_strncaseeql)(tvbuff_t*, gint, const guint8 *, gint);
+
+typedef gchar *(*addr_tvb_bytes_to_str)(tvbuff_t*, gint, gint len);
+
+typedef struct pref_module *(*addr_prefs_register_module)(const char *,
+    const char *, void (*)(void));
+typedef void (*addr_prefs_register_uint_preference)(struct pref_module *,
+    const char *, const char *, const char *, guint, guint *);
+typedef void (*addr_prefs_register_bool_preference)(struct pref_module *,
+    const char *, const char *, const char *, gboolean *);
+typedef void (*addr_prefs_register_enum_preference)(struct pref_module *,
+    const char *, const char *, const char *, gint *, const enum_val_t *,
+    gboolean);
+
 typedef struct  {
 	addr_check_col				p_check_col;
 	addr_col_add_fstr			p_col_add_fstr;
@@ -116,8 +190,24 @@ typedef struct  {
 	addr_proto_register_subtree_array	p_proto_register_subtree_array;
 
 	addr_dissector_add			p_dissector_add;
+	addr_old_dissector_add			p_old_dissector_add;
+	addr_dissector_delete			p_dissector_delete;
+
 	addr_heur_dissector_add			p_heur_dissector_add;
 
+	addr_register_dissector			p_register_dissector;
+	addr_find_dissector			p_find_dissector;
+	addr_old_call_dissector			p_old_call_dissector;
+	addr_call_dissector			p_call_dissector;
+
+	addr_dissect_data			p_dissect_data;
+	addr_old_dissect_data			p_old_dissect_data;
+
+	addr_proto_is_protocol_enabled		p_proto_is_protocol_enabled;
+
+	addr_proto_item_get_len			p_proto_item_get_len;
+	addr_proto_item_set_len			p_proto_item_set_len;
+	addr_proto_item_set_text		p_proto_item_set_text;
 	addr_proto_item_add_subtree		p_proto_item_add_subtree;
 	addr_proto_tree_add_item		p_proto_tree_add_item;
 	addr_proto_tree_add_item_hidden		p_proto_tree_add_item_hidden;
@@ -157,6 +247,58 @@ typedef struct  {
 	addr_proto_tree_add_int_format		p_proto_tree_add_int_format;
 	addr_proto_tree_add_text		p_proto_tree_add_text;
 	addr_proto_tree_add_notext		p_proto_tree_add_notext;
+
+	addr_tvb_new_subset			p_tvb_new_subset;
+
+	addr_tvb_length				p_tvb_length;
+	addr_tvb_length_remaining		p_tvb_length_remaining;
+	addr_tvb_bytes_exist			p_tvb_bytes_exist;
+	addr_tvb_offset_exists			p_tvb_offset_exists;
+	addr_tvb_reported_length		p_tvb_reported_length;
+
+	addr_tvb_get_guint8			p_tvb_get_guint8;
+
+	addr_tvb_get_ntohs			p_tvb_get_ntohs;
+	addr_tvb_get_ntoh24			p_tvb_get_ntoh24;
+	addr_tvb_get_ntohl			p_tvb_get_ntohl;
+#ifdef G_HAVE_GINT64
+	addr_tvb_get_ntohll			p_tvb_get_ntohll;
+#endif
+
+	addr_tvb_get_letohs			p_tvb_get_letohs;
+	addr_tvb_get_letoh24			p_tvb_get_letoh24;
+	addr_tvb_get_letohl			p_tvb_get_letohl;
+#ifdef G_HAVE_GINT64
+	addr_tvb_get_letohll			p_tvb_get_letohll;
+#endif
+
+	addr_tvb_memcpy				p_tvb_memcpy;
+	addr_tvb_memdup				p_tvb_memdup;
+
+	addr_tvb_get_ptr			p_tvb_get_ptr;
+
+	addr_tvb_find_guint8			p_tvb_find_guint8;
+	addr_tvb_pbrk_guint8			p_tvb_pbrk_guint8;
+
+	addr_tvb_strnlen			p_tvb_strnlen;
+
+	addr_tvb_format_text			p_tvb_format_text;
+
+	addr_tvb_get_nstringz			p_tvb_get_nstringz;
+	addr_tvb_get_nstringz0			p_tvb_get_nstringz0;
+
+	addr_tvb_find_line_end			p_tvb_find_line_end;
+	addr_tvb_find_line_end_unquoted	p_tvb_find_line_end_unquoted;
+
+	addr_tvb_strneql			p_tvb_strneql;
+	addr_tvb_strncaseeql			p_tvb_strncaseeql;
+
+	addr_tvb_bytes_to_str			p_tvb_bytes_to_str;
+
+	addr_prefs_register_module		p_prefs_register_module;
+	addr_prefs_register_uint_preference	p_prefs_register_uint_preference;
+	addr_prefs_register_bool_preference	p_prefs_register_bool_preference;
+	addr_prefs_register_enum_preference	p_prefs_register_enum_preference;
 } plugin_address_table_t;
 
 #else /* ! PLUGINS_NEED_ACCESS_TABLE */
