@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c,v 1.41 2000/09/12 06:41:56 guy Exp $
+ * $Id: prefs.c,v 1.44 2000/11/18 21:41:36 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -48,6 +48,7 @@
 #include <sys/stat.h>
 #endif
 
+#include <epan.h>
 #include "globals.h"
 #include "packet.h"
 #include "file.h"
@@ -285,7 +286,7 @@ prefs_register_bool_preference(module_t *module, const char *name,
 void
 prefs_register_enum_preference(module_t *module, const char *name,
     const char *title, const char *description, gint *var,
-    const enum_val *enumvals, gboolean radio_buttons)
+    const enum_val_t *enumvals, gboolean radio_buttons)
 {
 	pref_t *preference;
 
@@ -405,7 +406,7 @@ clear_string_list(GList *sl) {
 }
 
 /*
- * Takes a string, a pointer to an array of "enum_val"s, and a default gint
+ * Takes a string, a pointer to an array of "enum_val_t"s, and a default gint
  * value.
  * The array must be terminated by an entry with a null "name" string.
  * If the string matches a "name" strings in an entry, the value from that
@@ -413,7 +414,7 @@ clear_string_list(GList *sl) {
  * third argument is returned.
  */
 gint
-find_val_for_string(const char *needle, const enum_val *haystack,
+find_val_for_string(const char *needle, const enum_val_t *haystack,
     gint default_value)
 {
 	int i = 0;
@@ -533,7 +534,52 @@ read_prefs(int *gpf_errno_return, char **gpf_path_return,
 #ifdef WIN32
     prefs.gui_font_name = g_strdup("-*-lucida console-medium-r-*-*-*-100-*-*-*-*-*-*");
 #else
-    prefs.gui_font_name = g_strdup("-*-fixed-medium-r-semicondensed-*-*-120-*-*-*-*-*-*");
+    /*
+     * XXX - for now, we make the initial font name a pattern that matches
+     * only ISO 8859/1 fonts, so that we don't match 2-byte fonts such
+     * as ISO 10646 fonts.
+     *
+     * Users in locales using other one-byte fonts will have to choose
+     * a different font from the preferences dialog - or put the font
+     * selection in the global preferences file to make that font the
+     * default for all users who don't explicitly specify a different
+     * font.
+     *
+     * Making this a font set rather than a font has two problems:
+     *
+     *	1) as far as I know, you can't select font sets with the
+     *	   font selection dialog;
+     *
+     *  2) if you use a font set, the text to be drawn must be a
+     *	   multi-byte string in the appropriate locale, but
+     *	   Ethereal does *NOT* guarantee that's the case - in
+     *	   the hex-dump window, each character in the text portion
+     *	   of the display must be a *single* byte, and in the
+     *	   packet-list and protocol-tree windows, text extracted
+     *	   from the packet is not necessarily in the right format.
+     *
+     * "Doing this right" may, for the packet-list and protocol-tree
+     * windows, require that dissectors know what the locale is
+     * *AND* know what locale and text representation is used in
+     * the packets they're dissecting, and may be impossible in
+     * the hex-dump window (except by punting and displaying only
+     * ASCII characters).
+     *
+     * GTK+ 2.0 may simplify part of the problem, as it will, as I
+     * understand it, use UTF-8-encoded Unicode as its internal
+     * character set; however, we'd still have to know whatever
+     * character set and encoding is used in the packet (which
+     * may differ for different protocols, e.g. SMB might use
+     * PC code pages for some strings and Unicode for others, whilst
+     * NFS might use some UNIX character set encoding, e.g. ISO 8859/x,
+     * or one of the EUC character sets for Asian languages, or one
+     * of the other multi-byte character sets, or UTF-8, or...).
+     *
+     * I.e., as far as I can tell, "internationalizing" the packet-list,
+     * protocol-tree, and hex-dump windows involves a lot more than, say,
+     * just using font sets rather than fonts.
+     */
+    prefs.gui_font_name = g_strdup("-*-fixed-medium-r-semicondensed-*-*-120-*-*-*-*-iso8859-1");
 #endif
     prefs.gui_marked_fg.pixel = 65535;
     prefs.gui_marked_fg.red   = 65535;
@@ -969,7 +1015,7 @@ write_pref(gpointer data, gpointer user_data)
 {
 	pref_t *pref = data;
 	write_pref_arg_t *arg = user_data;
-	const enum_val *enum_valp;
+	const enum_val_t *enum_valp;
 	const char *val_string;
 
 	fprintf(arg->pf, "\n# %s\n", pref->description);
