@@ -1,3 +1,6 @@
+/*
+ * $Id: snprintf.c,v 1.11 2000/08/11 22:03:02 guy Exp $
+ */
 
 /*
    Unix snprintf implementation.
@@ -8,7 +11,7 @@
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    It can be redistribute also under the terms of GNU Library General
-   Public Lincense.
+   Public License.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,8 +47,11 @@
 # include "config.h"
 #endif
 
-#include "snprintf-imp.h"
+#include <stdlib.h>	/* for atoi and for size_t */
+#include <ctype.h>
+
 #include "snprintf.h"
+#include "snprintf-imp.h"
 
 /*
  * Find the nth power of 10
@@ -201,7 +207,7 @@ char ** fract;
       fp = integral(number, &ip);
       ch = (int)((fp + PRECISION)*base); /* force to round */
       integral_part[i] = (ch <= 9) ? ch + '0' : ch + 'a' - 10;
-      if (! isxdigit(integral_part[i])) /* bail out overflow !! */
+      if (! isxdigit((unsigned char)integral_part[i])) /* bail out overflow !! */
         break; 
       number = ip;
      }
@@ -225,7 +231,7 @@ char ** fract;
 /* the fractionnal part */
   for (i=0, fp=fraction; precision > 0 && i < MAX_FRACT ; i++, precision--	) {
     fraction_part[i] = (int)((fp + PRECISION)*10. + '0');
-    if (! isdigit(fraction_part[i])) /* underflow ? */
+    if (! isdigit((unsigned char)fraction_part[i])) /* underflow ? */
       break;
     fp = (fp*10.0) - (double)(long)((fp + PRECISION)*10.);
   }
@@ -474,7 +480,7 @@ struct DATA * p;
       case '1': case '2': case '3':
       case '4': case '5': case '6':
       case '7': case '8': case '9':     /* gob all the digits */
-        for (i = 0; isdigit(*s); i++, s++) 
+        for (i = 0; isdigit((unsigned char)*s); i++, s++) 
           if (i < MAX_FIELD/2 - 1)
             number[i] = *s;
         number[i] = '\0';
@@ -621,7 +627,7 @@ va_list args;
           case '2': case '3': case '4': case '5':
           case '6': case '7': case '8': case '9':
            /* initialize width and precision */
-            for (i = 0; isflag(*data.pf); i++, data.pf++) 
+            for (i = 0; isflag((unsigned char)*data.pf); i++, data.pf++) 
               if (i < MAX_FIELD - 1)
                 conv_field[i] = *data.pf;
             conv_field[i] = '\0';
