@@ -1,6 +1,6 @@
 /* tethereal.c
  *
- * $Id: tethereal.c,v 1.45 2000/09/10 06:44:37 guy Exp $
+ * $Id: tethereal.c,v 1.58 2000/11/20 17:08:20 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -75,6 +75,9 @@
 #include "getopt.h"
 #endif
 
+#include <glib.h>
+#include <epan.h>
+
 #include "globals.h"
 #include "timestamp.h"
 #include "packet.h"
@@ -119,13 +122,13 @@ static void wtap_dispatch_cb_write(u_char *, const struct wtap_pkthdr *, int,
     union wtap_pseudo_header *, const u_char *);
 static void wtap_dispatch_cb_print(u_char *, const struct wtap_pkthdr *, int,
     union wtap_pseudo_header *, const u_char *);
-static gchar *col_info(frame_data *, gint);
 
 packet_info  pi;
 capture_file cfile;
 FILE        *data_out_file = NULL;
 guint        main_ctx, file_ctx;
 ts_type timestamp_type = RELATIVE;
+static int promisc_mode = TRUE;
 
 static void 
 print_usage(void)
@@ -135,12 +138,12 @@ print_usage(void)
   fprintf(stderr, "This is GNU t%s %s, compiled with %s\n", PACKAGE,
 	  VERSION, comp_info_str);
 #ifdef HAVE_LIBPCAP
-  fprintf(stderr, "t%s [ -vVh ] [ -c count ] [ -f <capture filter> ]\n", PACKAGE);
+  fprintf(stderr, "t%s [ -vVhlp ] [ -c count ] [ -f <capture filter> ]\n", PACKAGE);
   fprintf(stderr, "\t[ -F <capture file type> ] [ -i interface ] [ -n ]\n");
   fprintf(stderr, "\t[ -o <preference setting> ] ... [ -r infile ] [ -R <read filter> ]\n");
   fprintf(stderr, "\t[ -s snaplen ] [ -t <time stamp format> ] [ -w savefile ] [ -x ]\n");
 #else
-  fprintf(stderr, "t%s [ -vVh ] [ -F <capture file type> ] [ -n ]\n", PACKAGE);
+  fprintf(stderr, "t%s [ -vVhl ] [ -F <capture file type> ] [ -n ]\n", PACKAGE);
   fprintf(stderr, "\t[ -o <preference setting> ] ... [ -r infile ] [ -R <read filter> ]\n");
   fprintf(stderr, "\t[ -t <time stamp format> ] [ -w savefile ] [ -x ]\n");
 #endif
@@ -166,6 +169,11 @@ main(int argc, char *argv[])
   extern char          pcap_version[];
 #endif
 #endif
+
+#ifdef WIN32
+  WSADATA		wsaData;
+#endif
+
   char                *gpf_path, *pf_path;
   int                  gpf_open_errno, pf_open_errno;
   int                  err;
@@ -186,7 +194,7 @@ main(int argc, char *argv[])
      "-G" flag, as the "-G" flag dumps a list of fields registered
      by the dissectors, and we must do it before we read the preferences,
      in case any dissectors register preferences. */
-  dissect_init();
+  epan_init(PLUGIN_DIR);
 
   /* Now register the preferences for any non-dissector modules.
      We must do that before we read the preferences as well. */
@@ -217,10 +225,10 @@ main(int argc, char *argv[])
     
   /* Initialize the capture file struct */
   cfile.plist		= NULL;
-  cfile.plist_end		= NULL;
+  cfile.plist_end	= NULL;
   cfile.wth		= NULL;
-  cfile.filename		= NULL;
-  cfile.user_saved		= FALSE;
+  cfile.filename	= NULL;
+  cfile.user_saved	= FALSE;
   cfile.is_tempfile	= FALSE;
   cfile.rfcode		= NULL;
   cfile.dfilter		= NULL;
@@ -229,16 +237,17 @@ main(int argc, char *argv[])
   cfile.cfilter		= g_strdup("");
 #endif
   cfile.iface		= NULL;
-  cfile.save_file		= NULL;
+  cfile.save_file	= NULL;
   cfile.save_file_fd	= -1;
   cfile.snap		= WTAP_MAX_PACKET_SIZE;
   cfile.count		= 0;
   cfile.cinfo.num_cols	= prefs->num_cols;
-  cfile.cinfo.col_fmt      = (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_fmt	= (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
   cfile.cinfo.fmt_matx	= (gboolean **) g_malloc(sizeof(gboolean *) * cfile.cinfo.num_cols);
   cfile.cinfo.col_width	= (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
-  cfile.cinfo.col_title    = (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_title	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
   cfile.cinfo.col_data	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_buf	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
 
   /* Assemble the compile-time options */
   snprintf(comp_info_str, 256,
@@ -284,7 +293,7 @@ main(int argc, char *argv[])
    );
     
   /* Now get our args */
-  while ((opt = getopt(argc, argv, "c:Df:F:hi:no:r:R:s:t:vw:Vx")) != EOF) {
+  while ((opt = getopt(argc, argv, "c:Df:F:hi:lno:pr:R:s:t:vw:Vx")) != EOF) {
     switch (opt) {
       case 'c':        /* Capture xxx packets */
 #ifdef HAVE_LIBPCAP
@@ -323,6 +332,9 @@ main(int argc, char *argv[])
         arg_error = TRUE;
 #endif
         break;
+      case 'l':        /* Line-buffer standard output */
+	setvbuf(stdout, NULL, _IOLBF, 0);
+	break;
       case 'n':        /* No name resolution */
 	g_resolving_actif = 0;
 	break;
@@ -341,6 +353,14 @@ main(int argc, char *argv[])
           break;
         }
         break;
+      case 'p':        /* Don't capture in promiscuous mode */
+#ifdef HAVE_LIBPCAP
+	promisc_mode = 0;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+	break;
       case 'r':        /* Read capture file xxx */
         cf_name = g_strdup(optarg);
         break;
@@ -360,13 +380,15 @@ main(int argc, char *argv[])
           timestamp_type = RELATIVE;
         else if (strcmp(optarg, "a") == 0)
           timestamp_type = ABSOLUTE;
+        else if (strcmp(optarg, "ad") == 0)
+          timestamp_type = ABSOLUTE_WITH_DATE;
         else if (strcmp(optarg, "d") == 0)
           timestamp_type = DELTA;
         else {
           fprintf(stderr, "tethereal: Invalid time stamp type \"%s\"\n",
             optarg);
           fprintf(stderr, "It must be \"r\" for relative, \"a\" for absolute,\n");
-          fprintf(stderr, "or \"d\" for delta.\n");
+          fprintf(stderr, "\"ad\" for absolute with date, or \"d\" for delta.\n");
           exit(1);
         }
         break;
@@ -412,6 +434,11 @@ main(int argc, char *argv[])
     }
   }
 
+#ifdef WIN32
+  /* Start windows sockets */
+  WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
+#endif
+
   /* Notify all registered modules that have had any of their preferences
      changed either from one of the preferences file or from the command
      line that its preferences have changed. */
@@ -431,10 +458,11 @@ main(int argc, char *argv[])
     cfile.cinfo.fmt_matx[i] = (gboolean *) g_malloc0(sizeof(gboolean) *
       NUM_COL_FMTS);
     get_column_format_matches(cfile.cinfo.fmt_matx[i], cfile.cinfo.col_fmt[i]);
+    cfile.cinfo.col_data[i] = NULL;
     if (cfile.cinfo.col_fmt[i] == COL_INFO)
-      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
+      cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
     else
-      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+      cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
   }
 
   if (cfile.snap < 1)
@@ -445,7 +473,7 @@ main(int argc, char *argv[])
   if (rfilter != NULL) {
     if (dfilter_compile(rfilter, &rfcode) != 0) {
       fprintf(stderr, "tethereal: %s\n", dfilter_error_msg);
-      dissect_cleanup();
+      epan_cleanup();
       exit(2);
     }
   }
@@ -453,12 +481,12 @@ main(int argc, char *argv[])
   if (cf_name) {
     err = open_cap_file(cf_name, FALSE, &cfile);
     if (err != 0) {
-      dissect_cleanup();
+      epan_cleanup();
       exit(2);
     }
     err = load_cap_file(&cfile, out_file_type);
     if (err != 0) {
-      dissect_cleanup();
+      epan_cleanup();
       exit(2);
     }
     cf_name[0] = '\0';
@@ -495,7 +523,7 @@ main(int argc, char *argv[])
 #endif
   }
 
-  dissect_cleanup();
+  epan_cleanup();
 
   exit(0);
 }
@@ -517,7 +545,7 @@ capture(int packet_count, int out_file_type)
 #endif
 
   /* Initialize the table of conversations. */
-  conversation_init();
+  epan_conversation_init();
 
   /* Initialize protocol-specific variables */
   init_all_protocols();
@@ -526,7 +554,7 @@ capture(int packet_count, int out_file_type)
   ld.pdh            = NULL;
 
   /* Open the network interface to capture from it. */
-  ld.pch = pcap_open_live(cfile.iface, cfile.snap, 1, 1000, err_str);
+  ld.pch = pcap_open_live(cfile.iface, cfile.snap, promisc_mode, 1000, err_str);
 
   if (ld.pch == NULL) {
     /* Well, we couldn't start the capture. */
@@ -557,7 +585,7 @@ capture(int packet_count, int out_file_type)
 	  "Tethereal may not be able to capture packets.\n"
 	  "\n"
 	  "To fix this, you will need to download the source to Tethereal\n"
-	  "from ethereal.zing.org if you have not already done so, read\n"
+	  "from www.ethereal.com if you have not already done so, read\n"
 	  "the instructions in the \"README.hpux\" file in the source\n"
 	  "distribution, download the source to libpcap if you have not\n"
 	  "already done so, patch libpcap as per the instructions, rebuild\n"
@@ -577,9 +605,14 @@ capture(int packet_count, int out_file_type)
   if (cfile.cfilter) {
     /* A capture filter was specified; set it up. */
     if (pcap_lookupnet (cfile.iface, &netnum, &netmask, err_str) < 0) {
-      snprintf(errmsg, sizeof errmsg,
-        "Can't use filter:  Couldn't obtain netmask info (%s).", err_str);
-      goto error;
+      /*
+       * Well, we can't get the netmask for this interface; it's used
+       * only for filters that check for broadcast IP addresses, so
+       * we just warn the user, and punt and use 0.
+       */
+      fprintf(stderr, 
+        "Warning:  Couldn't obtain netmask info (%s)\n.", err_str);
+      netmask = 0;
     }
     if (pcap_compile(ld.pch, &cfile.fcode, cfile.cfilter, 1, netmask) < 0) {
       snprintf(errmsg, sizeof errmsg, "Unable to parse filter string (%s).",
@@ -847,13 +880,13 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
      this packet. */
   compute_timestamp_diff(&fdata->del_secs, &fdata->del_usecs,
 		fdata->abs_secs, fdata->abs_usecs, prevsec, prevusec);
-  fdata->del_secs = fdata->abs_secs - prevsec;
   prevsec = fdata->abs_secs;
   prevusec = fdata->abs_usecs;
 
   fdata->cinfo = &cf->cinfo;
   for (i = 0; i < fdata->cinfo->num_cols; i++) {
-    fdata->cinfo->col_data[i][0] = '\0';
+    fdata->cinfo->col_buf[i][0] = '\0';
+    fdata->cinfo->col_data[i] = fdata->cinfo->col_buf[i];
   }
 }
 
@@ -868,16 +901,18 @@ wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr, int offset,
   proto_tree   *protocol_tree;
   int           err;
   gboolean      passed;
+  epan_dissect_t *edt;
 
   cf->count++;
   if (cf->rfcode) {
     fill_in_fdata(&fdata, cf, phdr, pseudo_header, offset);
     protocol_tree = proto_tree_create_root();
-    dissect_packet(pseudo_header, buf, &fdata, protocol_tree);
+    edt = epan_dissect_new(pseudo_header, buf, &fdata, protocol_tree);
     passed = dfilter_apply(cf->rfcode, protocol_tree, buf, fdata.cap_len);
   } else {
     protocol_tree = NULL;
     passed = TRUE;
+    edt = NULL;
   }
   if (passed) {
     /* XXX - do something if this fails */
@@ -885,6 +920,8 @@ wtap_dispatch_cb_write(u_char *user, const struct wtap_pkthdr *phdr, int offset,
   }
   if (protocol_tree != NULL)
     proto_tree_free(protocol_tree);
+  if (edt != NULL)
+    epan_dissect_free(edt);
 }
 
 static void
@@ -896,7 +933,9 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
   frame_data    fdata;
   proto_tree   *protocol_tree;
   gboolean      passed;
-  print_args_t print_args;
+  print_args_t  print_args;
+  epan_dissect_t *edt;
+  int           i;
 
   cf->count++;
 
@@ -911,7 +950,7 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
     protocol_tree = proto_tree_create_root();
   else
     protocol_tree = NULL;
-  dissect_packet(pseudo_header, buf, &fdata, protocol_tree);
+  edt = epan_dissect_new(pseudo_header, buf, &fdata, protocol_tree);
   if (cf->rfcode)
     passed = dfilter_apply(cf->rfcode, protocol_tree, buf, fdata.cap_len);
   if (passed) {
@@ -934,32 +973,198 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
     } else {
       /* Just fill in the columns. */
       fill_in_columns(&fdata);
-      if (cf->iface == NULL) {
-         printf("%3s %10s %12s -> %-12s %s %s\n",
-    		  col_info(&fdata, COL_NUMBER),
-		  col_info(&fdata, COL_CLS_TIME),
-		  col_info(&fdata, COL_DEF_SRC),
-		  col_info(&fdata, COL_DEF_DST),
-		  col_info(&fdata, COL_PROTOCOL),
-		  col_info(&fdata, COL_INFO));
-      } else {
-        printf("%12s -> %-12s %s %s\n",
-		  col_info(&fdata, COL_DEF_SRC),
-		  col_info(&fdata, COL_DEF_DST),
-		  col_info(&fdata, COL_PROTOCOL),
-		  col_info(&fdata, COL_INFO));
+
+      /* Now print them. */
+      for (i = 0; i < cf->cinfo.num_cols; i++) {
+        switch (cf->cinfo.col_fmt[i]) {
+	case COL_NUMBER:
+	  /*
+	   * Don't print this if we're doing a live capture from a network
+	   * interface - if we're doing a live capture, you won't be
+	   * able to look at the capture in the future (it's not being
+	   * saved anywhere), so the frame numbers are unlikely to be
+	   * useful.
+	   *
+	   * (XXX - it might be nice to be able to save and print at
+	   * the same time, sort of like an "Update list of packets
+	   * in real time" capture in Ethereal.)
+	   */
+          if (cf->iface != NULL)
+            continue;
+          printf("%3s", cf->cinfo.col_data[i]);
+          break;
+
+        case COL_CLS_TIME:
+        case COL_REL_TIME:
+        case COL_ABS_TIME:
+        case COL_ABS_DATE_TIME:	/* XXX - wider */
+          printf("%10s", cf->cinfo.col_data[i]);
+          break;
+
+        case COL_DEF_SRC:
+        case COL_RES_SRC:
+        case COL_UNRES_SRC:
+        case COL_DEF_DL_SRC:
+        case COL_RES_DL_SRC:
+        case COL_UNRES_DL_SRC:
+        case COL_DEF_NET_SRC:
+        case COL_RES_NET_SRC:
+        case COL_UNRES_NET_SRC:
+          printf("%12s", cf->cinfo.col_data[i]);
+          break;
+
+        case COL_DEF_DST:
+        case COL_RES_DST:
+        case COL_UNRES_DST:
+        case COL_DEF_DL_DST:
+        case COL_RES_DL_DST:
+        case COL_UNRES_DL_DST:
+        case COL_DEF_NET_DST:
+        case COL_RES_NET_DST:
+        case COL_UNRES_NET_DST:
+          printf("%-12s", cf->cinfo.col_data[i]);
+          break;
+
+        default:
+          printf("%s", cf->cinfo.col_data[i]);
+          break;
+        }
+        if (i != cf->cinfo.num_cols - 1) {
+          /*
+	   * This isn't the last column, so we need to print a
+	   * separator between this column and the next.
+	   *
+	   * If we printed a network source and are printing a
+	   * network destination of the same type next, separate
+	   * them with "->"; if we printed a network destination
+	   * and are printing a network source of the same type
+	   * next, separate them with "<-"; otherwise separate them
+	   * with a space.
+	   */
+	  switch (cf->cinfo.col_fmt[i]) {
+
+	  case COL_DEF_SRC:
+	  case COL_RES_SRC:
+	  case COL_UNRES_SRC:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_DST:
+	    case COL_RES_DST:
+	    case COL_UNRES_DST:
+	      printf(" -> ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  case COL_DEF_DL_SRC:
+	  case COL_RES_DL_SRC:
+	  case COL_UNRES_DL_SRC:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_DL_DST:
+	    case COL_RES_DL_DST:
+	    case COL_UNRES_DL_DST:
+	      printf(" -> ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  case COL_DEF_NET_SRC:
+	  case COL_RES_NET_SRC:
+	  case COL_UNRES_NET_SRC:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_NET_DST:
+	    case COL_RES_NET_DST:
+	    case COL_UNRES_NET_DST:
+	      printf(" -> ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  case COL_DEF_DST:
+	  case COL_RES_DST:
+	  case COL_UNRES_DST:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_SRC:
+	    case COL_RES_SRC:
+	    case COL_UNRES_SRC:
+	      printf(" <- ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  case COL_DEF_DL_DST:
+	  case COL_RES_DL_DST:
+	  case COL_UNRES_DL_DST:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_DL_SRC:
+	    case COL_RES_DL_SRC:
+	    case COL_UNRES_DL_SRC:
+	      printf(" <- ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  case COL_DEF_NET_DST:
+	  case COL_RES_NET_DST:
+	  case COL_UNRES_NET_DST:
+	    switch (cf->cinfo.col_fmt[i + 1]) {
+
+	    case COL_DEF_NET_SRC:
+	    case COL_RES_NET_SRC:
+	    case COL_UNRES_NET_SRC:
+	      printf(" <- ");
+	      break;
+
+	    default:
+	      putchar(' ');
+	      break;
+	    }
+	    break;
+
+	  default:
+	    putchar(' ');
+	    break;
+	  }
+	}
       }
+      putchar('\n');
     }
     if (print_hex) {
       print_hex_data(stdout, print_args.format, buf,
 			fdata.cap_len, fdata.flags.encoding);
-      printf("\n");
+      putchar('\n');
     }
     fdata.cinfo = NULL;
   }
   if (protocol_tree != NULL)
     proto_tree_free(protocol_tree);
 
+  epan_dissect_free(edt);
+
   proto_tree_is_visible = FALSE;
 }
 
@@ -1068,7 +1273,7 @@ open_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
   /* The open succeeded.  Fill in the information for this file. */
 
   /* Initialize the table of conversations. */
-  conversation_init();
+  epan_conversation_init();
 
   /* Initialize protocol-specific variables */
   init_all_protocols();
@@ -1106,17 +1311,3 @@ fail:
   fprintf(stderr, "tethereal: %s\n", err_msg);
   return (err);
 }
-
-/* Get the text in a given column */
-static gchar *
-col_info(frame_data *fd, gint el) {
-  int i;
-  
-  if (fd->cinfo) {
-    for (i = 0; i < fd->cinfo->num_cols; i++) {
-      if (fd->cinfo->fmt_matx[i][el])
-        return fd->cinfo->col_data[i];
-    }
-  }
-  return NULL;
-}
