@@ -1,6 +1,6 @@
 /* netmon.c
  *
- * $Id: netmon.c,v 1.33 2000/09/07 05:34:12 gram Exp $
+ * $Id: netmon.c,v 1.35 2000/11/19 03:47:35 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -108,7 +108,7 @@ int netmon_open(wtap *wth, int *err)
 	static const int netmon_encap[] = {
 		WTAP_ENCAP_UNKNOWN,
 		WTAP_ENCAP_ETHERNET,
-		WTAP_ENCAP_TR,
+		WTAP_ENCAP_TOKEN_RING,
 		WTAP_ENCAP_FDDI_BITSWAPPED,
 		WTAP_ENCAP_UNKNOWN,	/* WAN */
 		WTAP_ENCAP_UNKNOWN,	/* LocalTalk */
@@ -298,7 +298,8 @@ static gboolean netmon_read(wtap *wth, int *err, int *data_offset)
 		   free it. */
 		g_free(wth->capture.netmon->frame_table);
 		wth->capture.netmon->frame_table = NULL;
-		return 0;
+		*err = 0;	/* it's just an EOF, not an error */
+		return FALSE;
 	}
 
 	/* Seek to the beginning of the current record, if we're
@@ -412,7 +413,7 @@ netmon_close(wtap *wth)
 static const int wtap_encap[] = {
 	-1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
 	1,		/* WTAP_ENCAP_ETHERNET -> NDIS Ethernet */
-	2,		/* WTAP_ENCAP_TR -> NDIS Token Ring */
+	2,		/* WTAP_ENCAP_TOKEN_RING -> NDIS Token Ring */
 	-1,		/* WTAP_ENCAP_SLIP -> unsupported */
 	-1,		/* WTAP_ENCAP_PPP -> unsupported */
 	3,		/* WTAP_ENCAP_FDDI -> NDIS FDDI */
