@@ -1,6 +1,6 @@
 /* netxray.c
  *
- * $Id: netxray.c,v 1.31 2000/09/07 05:34:14 gram Exp $
+ * $Id: netxray.c,v 1.34 2000/11/19 03:47:35 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -110,14 +110,14 @@ int netxray_open(wtap *wth, int *err)
 	double t;
 	static const int netxray_encap[] = {
 		WTAP_ENCAP_ETHERNET,
-		WTAP_ENCAP_TR,
+		WTAP_ENCAP_TOKEN_RING,
 		WTAP_ENCAP_FDDI_BITSWAPPED,
 		WTAP_ENCAP_ETHERNET,	/* WAN(PPP), but shaped like ethernet */
 		WTAP_ENCAP_UNKNOWN,	/* LocalTalk */
 		WTAP_ENCAP_UNKNOWN,	/* "DIX" - should not occur */
 		WTAP_ENCAP_UNKNOWN,	/* ARCNET raw */
 		WTAP_ENCAP_UNKNOWN,	/* ARCNET 878.2 */
-		WTAP_ENCAP_UNKNOWN,	/* ATM */
+		WTAP_ENCAP_ATM_RFC1483,	/* ATM */
 		WTAP_ENCAP_UNKNOWN,	/* Wireless WAN */
 		WTAP_ENCAP_UNKNOWN	/* IrDA */
 	};
@@ -234,6 +234,7 @@ reread:
 	/* Have we reached the end of the packet data? */
 	if (wth->data_offset == wth->capture.netxray->end_offset) {
 		/* Yes. */
+		*err = 0;	/* it's just an EOF, not an error */
 		return FALSE;
 	}
 	/* Read record header. */
@@ -310,7 +311,7 @@ netxray_close(wtap *wth)
 static const int wtap_encap[] = {
     -1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
     0,		/* WTAP_ENCAP_ETHERNET -> NDIS Ethernet */
-    1,		/* WTAP_ENCAP_TR -> NDIS Token Ring */
+    1,		/* WTAP_ENCAP_TOKEN_RING -> NDIS Token Ring */
     -1,		/* WTAP_ENCAP_SLIP -> unsupported */
     -1,		/* WTAP_ENCAP_PPP -> unsupported */
     2,		/* WTAP_ENCAP_FDDI -> NDIS FDDI */
