@@ -1,6 +1,6 @@
 /* ngsniffer.c
  *
- * $Id: ngsniffer.c,v 1.51 2000/09/07 05:34:16 gram Exp $
+ * $Id: ngsniffer.c,v 1.55 2000/11/19 03:47:36 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -79,6 +79,7 @@ static const char ngsniffer_magic[] = {
 #define REC_VERS	1	/* Version record (f_vers) */
 #define REC_FRAME2	4	/* Frame data (f_frame2) */
 #define	REC_FRAME4	8	/* Frame data (f_frame4) */
+#define REC_FRAME6	12	/* Frame data (f_frame6) (see below) */
 #define REC_EOF		3	/* End-of-file record (no data follows) */
 /*
  * and now for some unknown header types
@@ -88,6 +89,12 @@ static const char ngsniffer_magic[] = {
 #define REC_V2DESC	8	/* In version 2 sniffer traces contains
 				 * infos about this capturing session.
 				 * Collides with REC_FRAME4 */
+#define REC_HEADER3	13	/* Retransmission counts? */
+#define REC_HEADER4	14	/* ? */
+#define REC_HEADER5	15	/* ? */
+#define REC_HEADER6	16	/* More broadcast/retransmission counts? */
+#define REC_HEADER7	17	/* ? */
+
 
 /*
  * Sniffer version record format.
@@ -245,6 +252,24 @@ struct frame4_rec {
 	ATMSaveInfo atm_info;	/* ATM-specific stuff */
 };
 
+/*
+ * XXX - I have a version 5.50 file with a bunch of token ring
+ * records listed as type "12".  The record format below was
+ * derived from frame4_rec and a bit of experimentation.
+ * - Gerald
+ */
+struct frame6_rec {
+	guint16	time_low;	/* low part of time stamp */
+	guint16	time_med;	/* middle part of time stamp */
+	gint8	time_high;	/* high part of time stamp */
+	gint8	time_day;	/* time in days since start of capture */
+	gint16	size;		/* number of bytes of data */
+	gint8	fs;		/* frame error status bits */
+	gint8	flags;		/* buffer flags */
+	gint16	true_size;	/* size of original frame, in bytes */
+	guint8	chemical_x[22];	/* ? */
+};
+
 /* values for V.timeunit */
 #define NUM_NGSNIFF_TIMEUNITS 7
 static double Usec[] = { 15.0, 0.838096, 15.0, 0.5, 2.0, 1.0, 0.1 };
@@ -263,8 +288,14 @@ static int ngsniffer_read_frame4(wtap *wth, gboolean is_random,
     struct frame4_rec *frame4, int *err);
 static void set_pseudo_header_frame4(union wtap_pseudo_header *pseudo_header,
     struct frame4_rec *frame4);
+static int ngsniffer_read_frame6(wtap *wth, gboolean is_random,
+    struct frame6_rec *frame6, int *err);
+static void set_pseudo_header_frame6(union wtap_pseudo_header *pseudo_header,
+    struct frame6_rec *frame6);
 static int ngsniffer_read_rec_data(wtap *wth, gboolean is_random, u_char *pd,
     int length, int *err);
+static void fix_pseudo_header(wtap *wth,
+    union wtap_pseudo_header *pseudo_header);
 static void ngsniffer_sequential_close(wtap *wth);
 static void ngsniffer_close(wtap *wth);
 static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
@@ -291,7 +322,7 @@ int ngsniffer_open(wtap *wth, int *err)
 	guint16	start_date;
 	guint16	start_time;
 	static const int sniffer_encap[] = {
-		WTAP_ENCAP_TR,
+		WTAP_ENCAP_TOKEN_RING,
 		WTAP_ENCAP_ETHERNET,
 		WTAP_ENCAP_ARCNET,
 		WTAP_ENCAP_UNKNOWN,	/* StarLAN */
@@ -539,6 +570,7 @@ static gboolean ngsniffer_read(wtap *wth, int *err, int *data_offset)
 	guint16	type, length;
 	struct frame2_rec frame2;
 	struct frame4_rec frame4;
+	struct frame6_rec frame6;
 	double	t;
 	guint16	time_low, time_med, time_high, true_size, size;
 	u_char	*pd;
@@ -622,10 +654,40 @@ static gboolean ngsniffer_read(wtap *wth, int *err, int *data_offset)
 			set_pseudo_header_frame4(&wth->pseudo_header, &frame4);
 			goto found;
 
+		case REC_FRAME6:
+			/* XXX - Is this test valid? */
+			if (wth->capture.ngsniffer->is_atm) {
+				g_message("ngsniffer: REC_FRAME6 record in an ATM Sniffer file");
+				*err = WTAP_ERR_BAD_RECORD;
+				return FALSE;
+			}
+
+			/* Read the f_frame6_struct */
+			ret = ngsniffer_read_frame6(wth, FALSE, &frame6, err);
+			wth->data_offset += sizeof frame6;
+			time_low = pletohs(&frame6.time_low);
+			time_med = pletohs(&frame6.time_med);
+			time_high = frame6.time_high;
+			size = pletohs(&frame6.size);
+			true_size = pletohs(&frame6.true_size);
+
+			length -= sizeof frame6;	/* we already read that much */
+
+			/*
+			 * XXX - use the "time_day" field?  Is that for captures
+			 * that take a *really* long time?
+			 */
+			t = (double)time_low+(double)(time_med)*65536.0 +
+			    (double)time_high*4294967296.0;
+
+			set_pseudo_header_frame6(&wth->pseudo_header, &frame6);
+			goto found;
+
 		case REC_EOF:
 			/*
 			 * End of file.  Return an EOF indication.
 			 */
+			*err = 0;	/* EOF, not error */
 			return FALSE;
 
 		default:
@@ -678,6 +740,12 @@ found:
 		}
 	}
 
+	/*
+	 * Fix up the pseudo-header; we may have set "x25.flags",
+	 * but, for some traffic, we should set "p2p.sent" instead.
+	 */
+	fix_pseudo_header(wth, &wth->pseudo_header);
+
 	t = t/1000000.0 * wth->capture.ngsniffer->timeunit; /* t = # of secs */
 	t += wth->capture.ngsniffer->start;
 	wth->phdr.ts.tv_sec = (long)t;
@@ -695,6 +763,7 @@ static int ngsniffer_seek_read(wtap *wth, int seek_off,
 	guint16	type, length;
 	struct frame2_rec frame2;
 	struct frame4_rec frame4;
+	struct frame6_rec frame6;
 
 	ng_file_seek_rand(wth, seek_off, SEEK_SET);
 
@@ -728,6 +797,15 @@ static int ngsniffer_seek_read(wtap *wth, int seek_off,
 		set_pseudo_header_frame4(pseudo_header, &frame4);
 		break;
 
+	case REC_FRAME6:
+		/* Read the f_frame6_struct */
+		ret = ngsniffer_read_frame6(wth, TRUE, &frame6, &err);
+
+		length -= sizeof frame6;	/* we already read that much */
+
+		set_pseudo_header_frame6(pseudo_header, &frame6);
+		break;
+
 	default:
 		/*
 		 * "Can't happen".
@@ -736,6 +814,12 @@ static int ngsniffer_seek_read(wtap *wth, int seek_off,
 		return -1;
 	}
 
+	/*
+	 * Fix up the pseudo-header; we may have set "x25.flags",
+	 * but, for some traffic, we should set "p2p.sent" instead.
+	 */
+	fix_pseudo_header(wth, pseudo_header);
+
 	/*
 	 * Got the pseudo-header (if any), now get the data.
 	 */
@@ -847,6 +931,28 @@ static void set_pseudo_header_frame4(union wtap_pseudo_header *pseudo_header,
 	pseudo_header->ngsniffer_atm.aal5t_chksum = pletohl(&frame4->atm_info.Trailer.aal5t_chksum);
 }
 
+static int ngsniffer_read_frame6(wtap *wth, gboolean is_random,
+    struct frame6_rec *frame6, int *err)
+{
+	int bytes_read;
+
+	/* Read the f_frame6_struct */
+	bytes_read = ng_file_read(frame6, 1, sizeof *frame6, wth, is_random,
+	    err);
+	if (bytes_read != sizeof *frame6) {
+		if (*err == 0)
+			*err = WTAP_ERR_SHORT_READ;
+		return -1;
+	}
+	return 0;
+}
+
+static void set_pseudo_header_frame6(union wtap_pseudo_header *pseudo_header,
+    struct frame6_rec *frame6)
+{
+	/* XXX - Once the frame format is divined, something will most likely go here */
+}
+
 static int ngsniffer_read_rec_data(wtap *wth, gboolean is_random, u_char *pd,
     int length, int *err)
 {
@@ -862,6 +968,20 @@ static int ngsniffer_read_rec_data(wtap *wth, gboolean is_random, u_char *pd,
 	return 0;
 }
 
+static void fix_pseudo_header(wtap *wth,
+    union wtap_pseudo_header *pseudo_header)
+{
+	switch (wth->file_encap) {
+
+	case WTAP_ENCAP_LAPD:
+		if (pseudo_header->x25.flags == 0x00)
+			pseudo_header->p2p.sent = TRUE;
+		else
+			pseudo_header->p2p.sent = FALSE;
+		break;
+	}
+}
+
 /* Throw away the buffers used by the sequential I/O stream, but not
    those used by the random I/O stream. */
 static void ngsniffer_sequential_close(wtap *wth)
@@ -893,7 +1013,7 @@ static void ngsniffer_close(wtap *wth)
 static const int wtap_encap[] = {
     -1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
     1,		/* WTAP_ENCAP_ETHERNET */
-    0,		/* WTAP_ENCAP_TR */
+    0,		/* WTAP_ENCAP_TOKEN_RING */
     -1,		/* WTAP_ENCAP_SLIP -> unsupported */
     7,		/* WTAP_ENCAP_PPP -> Internetwork analyzer (synchronous) FIXME ! */
     -1,		/* WTAP_ENCAP_FDDI -> unsupported */
