@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c,v 1.47 2000/09/07 05:34:22 gram Exp $
+ * $Id: wtap.c,v 1.49 2000/11/15 05:41:48 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -115,6 +115,13 @@ const static struct encap_type_info {
 
 	/* WTAP_ENCAP_V120 */
 	{ "V.120", "v120" },
+
+	/* WTAP_ENCAP_PPP_WITH_PHDR */
+	{ "PPP with Directional Info", "ppp-with-direction" },
+
+	/* WTAP_ENCAP_IEEE_802_11 */
+	{ "IEEE 802.11 Wireless LAN", "ieee-802-11" },
+
 };
 
 /* Name that should be somewhat descriptive. */
